/*
 * AuthenticatorDiscoveryPresentation.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.authenticator;

import java.io.IOException;
import java.io.FileNotFoundException;

import bees.core.Node;
import bees.core.Assert;
import bees.core.TypeID;
import bees.core.ByteArray;
import bees.core.NeighborImpl;
import bees.core.CapabilityReference;
import bees.core.MissingCapabilityException;

import bees.core.resource.Logger;
import bees.core.resource.NetworkService;

import bees.core.security.EnumerationException;

import apps.discovery.DiscoveryProtocolPresentation;

/**
 * The AuthenticatorDiscoveryPresentation is a DiscoveryProtocolPresentation
 * specialized for use with authenticators.  Currently, it will try to convert
 * advertised material into NeighborImpl objects.
 */
public class AuthenticatorDiscoveryPresentation
    extends DiscoveryProtocolPresentation
{
    /**
     * The authenticator's parameters.
     */
    private AuthenticatorParameters ap;
    
    /**
     * The authenticator NeighborManager.
     */
    private NeighborManager nm;

    /**
     * Construct an AuthenticatorDiscoveryPresentation with the given values.
     *
     * @param ap The authenticator's parameters.
     * @param nm The authenticator NeighborManager.
     */
    AuthenticatorDiscoveryPresentation(AuthenticatorParameters ap,
				       NeighborManager nm)
    {
	Assert.require(ap);
	Assert.require(nm);

	this.ap = ap;
	this.nm = nm;
    }

    /**
     * @see bees.core.ExtendedProtocolPresentation#addNeighborReply(Node, CapabilityReference)
     */
    public synchronized void addNeighborReply(Node node,
					      CapabilityReference cr)
    {
	if( cr != null )
	{
	    try
	    {
		NeighborImpl ni;
		int index;
		
		ni = (NeighborImpl)cr.dereference();
		if( (index = this.ap.getBroadcastNeighborIndex(ni)) != -1 )
		{
		    byte data[], sig[] = null;
		    
		    data = ByteArray.
			fromFile(this.ap.getAdvertiseMaterial()[index]);
		    super.advertise(ni, data, sig);
		}
	    }
	    catch(FileNotFoundException e)
	    {
		Logger.log(e);
	    }
	    catch(IOException e)
	    {
		Logger.log(e);
	    }
	    catch(MissingCapabilityException e)
	    {
		Logger.log(e);
	    }
	}
    }

    /**
     * @see apps.discovery.DiscoveryProtocolPresentation#found(CapabilityReference, TypeID, byte[], byte[])
     */
    public void found(CapabilityReference cr,
		      TypeID fp,
		      byte material[],
		      byte signature[])
    {
	Assert.require(fp);
	Assert.require(material);
	
	try
	{
	    AdvertiseMaterial am;
	    NeighborImpl src, ni;
	    int index;

	    src = (NeighborImpl)cr.dereference();
	    am = AdvertiseMaterial.instantiate(material);
	    if( (index = this.ap.getBroadcastNeighborIndex(src)) != -1 )
	    {
		NetworkService local, remote;

		local = this.ap.getLocalNetworkServices()[index];
		remote = this.ap.getRemoteNetworkServices()[index];
		ni = this.nm.create(src, local, remote, am);
		this.nm.addNeighbor(ni, fp);
		Logger.log(Logger.INFO, "Found neighbor: " + ((bees.core.resource.NodeOSNeighbor)ni).toVerboseString());
		Logger.log(Logger.INFO, "From: " + ((bees.core.resource.NodeOSNeighbor)src).toVerboseString());
	    }
	    else
	    {
		Logger.log(Logger.ERROR, "Unknown broadcast neighbor: " + src);
	    }
	}
	catch(MissingCapabilityException e)
	{
	    Logger.log(e);
	}
	catch(EnumerationException e)
	{
	    Logger.log(e);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "AuthenticatorDiscoveryPresentation["
	    + super.toString()
	    + "; nm="
	    + this.nm
	    + "]";
    }
}
