/*
 * AuthenticatorParameters.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.authenticator;

import java.util.Enumeration;

import bees.core.Node;
import bees.core.Assert;
import bees.core.ObjectMap;
import bees.core.NeighborImpl;

import bees.core.resource.File;
import bees.core.resource.Logger;
import bees.core.resource.NetworkService;

/**
 * An AuthenticatorParameters object is used to hold any parameters to the
 * authenticator domain.
 */
public class AuthenticatorParameters
{
    /**
     * The set of neighbors to be used when broadcasting advertisements.
     */
    private NeighborImpl broadcastNeighbor[];
    
    /**
     * The material describing the local characteristics needed when creating
     * neighbors.
     */
    private File localMaterial[];

    /**
     * The material to advertise to neighbors interested in communicating with
     * this node.
     */
    private File advertiseMaterial[];

    /**
     * The set of local addresses this authenticator was given access to.
     */
    private NetworkService localNetworkServices[];

    /**
     * The set of remote addresses this authenticator was given access to.
     */
    private NetworkService remoteNetworkServices[];

    /**
     * Construct and initialize an AuthenticatorParameters object from the
     * given values.
     *
     * @param node The current Node object.
     */
    AuthenticatorParameters(Node node)
    {
	Enumeration enum;
	ObjectMap om;
	int lpc;
	
	Assert.require(node);

	om = node.getCapabilityMap("nic");
	enum = om.elements();
	this.broadcastNeighbor = new NeighborImpl[om.size()];
	this.localMaterial = new File[this.broadcastNeighbor.length];
	this.advertiseMaterial = new File[this.broadcastNeighbor.length];
	this.localNetworkServices =
	    new NetworkService[this.broadcastNeighbor.length];
	this.remoteNetworkServices =
	    new NetworkService[this.broadcastNeighbor.length];
	for( lpc = 0; enum.hasMoreElements(); lpc++ )
	{
	    ObjectMap sub;

	    sub = (ObjectMap)enum.nextElement();
	    this.broadcastNeighbor[lpc] = (NeighborImpl)
		sub.get("broadcastNeighbor");
	    this.localMaterial[lpc] = (File)sub.get("local-material");
	    this.advertiseMaterial[lpc] = (File)sub.get("advertise-material");
	    this.localNetworkServices[lpc] =
		(NetworkService)sub.get("ip-udp-all");
	    Logger.log(Logger.INFO,
		       "Local service: "
		       + this.localNetworkServices[lpc]);
	    this.remoteNetworkServices[lpc] =
		(NetworkService)sub.get("remote-ip-udp-all");
	    Logger.log(Logger.INFO,
		       "Remote service: "
		       + this.remoteNetworkServices[lpc]);
	    Logger.log(Logger.INFO,
		       "Broadcast Neighbor: "
		       + this.broadcastNeighbor[lpc]
		       + "; Local Material: "
		       + this.localMaterial[lpc]
		       + "; Advertise Material: "
		       + this.advertiseMaterial[lpc]);
	}
    }

    /**
     * @return The set of neighbors to be used when broadcasting
     * advertisements.
     */
    NeighborImpl[] getBroadcastNeighbor()
    {
	return this.broadcastNeighbor;
    }

    /**
     * @param ni The NeighborImpl to search for.
     * @return The index of the NeighborImpl in the set of BroadcastNeigbhors
     * or -1 if a match couldn't be found.
     */
    int getBroadcastNeighborIndex(NeighborImpl ni)
    {
	int lpc, retval = -1;

	for( lpc = 0; lpc < this.broadcastNeighbor.length; lpc++ )
	{
	    if( this.broadcastNeighbor[lpc] == ni )
	    {
		retval = lpc;
		break;
	    }
	}
	return retval;
    }
    
    /**
     * @return The material describing the local characteristics needed when
     * creating neighbors.
     */
    File[] getLocalMaterial()
    {
	return this.localMaterial;
    }

    /**
     * @return The material to advertise to neighbors interested in
     * communicating with this node.
     */
    File[] getAdvertiseMaterial()
    {
	return this.advertiseMaterial;
    }

    /**
     * @return The set of local addresses this authenticator was given access
     * to.
     */
    NetworkService[] getLocalNetworkServices()
    {
	return this.localNetworkServices;
    }
    
    /**
     * @return The set of remote addresses this authenticator was given access
     * to.
     */
    NetworkService[] getRemoteNetworkServices()
    {
	return this.remoteNetworkServices;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "AuthenticatorParameters[broadcastNeighbor="
	    + this.broadcastNeighbor
	    + "; localMaterial="
	    + this.localMaterial
	    + "; advertiseMaterial="
	    + this.advertiseMaterial
	    + "; localNetworkServices="
	    + this.localNetworkServices
	    + "; remoteNetworkServices="
	    + this.remoteNetworkServices
	    + "]";
    }
}
