/*
 * Everything.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.authenticator;

import bees.core.PairChain;
import bees.core.Capability;
import bees.core.UnknownProtocolPresentation;

import bees.core.bdomain.CapabilityTree;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.CountPermission;

/**
 * A null authenticator that lets everything in.
 */
public class Everything
    extends Authenticator
{
    /**
     * The default resource bounds given to every BDomain started by this
     * Auth App.
     */
    private static final PermissionSet DEFAULT_RESOURCES =
	new PermissionSet(new Permission[] {
	    new CountPermission("cpu.shares", 10),
	    new CountPermission("cpu.threads", 2),
	    new CountPermission("mem.pages", 1 * 1024 * 1024 / 4096),
	    });
    
    /**
     * Construct an empty Everything authenticator.
     */
    public Everything()
    {
    }
    
    /**
     * The actual UnknownProtocol authenticator.
     */
    private class MyUnknownProtocolPresentation
	extends AuthenticatorUnknownProtocolPresentation
    {
	/**
	 * Construct a MyUnknownProtocolPresentation with the given values.
	 *
	 * @param auth The Authenticator this presentation is being used by.
	 */
	MyUnknownProtocolPresentation(Authenticator auth)
	{
	    super(auth);
	}

        /**
         * Allow everything in.
         * 
         * @see apps.authenticator.AuthenticatorUnknownProtocolPresentation#validate(PairChain)
         */
	protected boolean validate(PairChain pc)
	{
	    return true;
	}

        /**
         * Just returns null, we cannot give more authority.
         * 
         * @see apps.authenticator.AuthenticatorUnknownProtocolPresentation#computeAuthority(PairChain)
         */
	protected CapabilityTree computeAuthority(PairChain pc)
	{
	    return null;
	}

        /**
         * @see apps.authenticator.AuthenticatorUnknownProtocolPresentation#computeInitialCapabilities(PairChain)
         */
	protected Capability[] computeInitialCapabilities(PairChain pc)
	{
	    return super.getAuthenticator().getInitialCapabilities();
	}

        /**
         * @see apps.authenticator.AuthenticatorUnknownProtocolPresentation#computeResources(PairChain)
         */
	protected PermissionSet computeResources(PairChain pc)
	{
	    return DEFAULT_RESOURCES;
	}
	
        /**
         * @see java.lang.Object#toString()
         */
	public String toString()
	{
	    return "MyUnknownProtocolPresentation["
		+ super.toString()
		+ "]";
	}
    }

    /**
     * @see apps.authenticator.Authenticator#createUnknownProtocolPresentation()
     */
    protected UnknownProtocolPresentation createUnknownProtocolPresentation()
    {
	return new MyUnknownProtocolPresentation(this);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Everything["
	    + super.toString()
	    + "]";
    }
}
