/*
 * MaterialBuilder.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.authenticator;

import java.util.Stack;
import java.util.Vector;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.ByteArray;
import bees.core.PairChain;
import bees.core.Application;
import bees.core.XdrByteArray;

import bees.core.resource.File;
import bees.core.resource.Logger;
import bees.core.resource.IPAddressPermission;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.IntegerPermission;
import bees.core.security.EnumerationPermission;


/**
 * The MaterialBuilder application is used to build a file containing
 * data that can be used in a discovery advertisement.
 */
public final class MaterialBuilder
    extends Application
{
    /**
     * Construct an empty MaterialBuilder application object.
     */
    public MaterialBuilder()
    {
    }

    /**
     * Split a string at the first instance of a character.  For example,
     * "split("foo=bar", '=')" will return "new String[] { "foo", "bar" }".
     *
     * @param str The string to split.
     * @param token The token to look for in "str".
     * @return The pair of strings created from the split.
     * @throws IllegalArgumentException If the token cannot be found in the
     * string.
     */
    private String[] split(String str, char token)
	throws IllegalArgumentException
    {
	String retval[] = new String[2];
	int index;
	
	if( (index = str.indexOf(token)) != -1 )
	{
	    retval[0] = str.substring(0, index);
	    retval[1] = str.substring(index + 1);
	}
	else
	{
	    throw new IllegalArgumentException("Token \""
					       + token
					       + "\" not found in string: "
					       + str);
	}
	return retval;
    }

    /**
     * @see bees.core.Application#start(Node, String[])
     */
    public void start(Node node, String args[])
	throws Throwable
    {
	if( (args.length % 2) == 0 )
	{
	    Stack setStack = new Stack();
	    Vector permissions = null;
	    PermissionSet ps = null;
	    byte data[] = null;
	    XdrByteArray xba;
	    Permission perm;
	    String spair[];
	    PairChain pc;
	    int lpc;

	    for( lpc = 0; lpc < args.length; lpc += 2 )
	    {
		if( args[lpc].equals("-start-xdr") )
		{
		}
		else if( args[lpc].equals("-end-xdr") )
		{
		    pc = (PairChain)setStack.peek();
		    pc.add(args[lpc + 1], data);
		}
		else if( args[lpc].equals("-int") )
		{
		    xba = new XdrByteArray(Xdr.INT);
		    xba.PUT(Integer.decode(args[lpc + 1]).intValue());
		    data = ByteArray.concat(data, xba.toBytes());
		}
		else if( args[lpc].equals("-long") )
		{
		    xba = new XdrByteArray(Xdr.LONG);
		    xba.PUT(Long.decode(args[lpc + 1]).longValue());
		    data = ByteArray.concat(data, xba.toBytes());
		}
		else if( args[lpc].equals("-string") )
		{
		    xba = new XdrByteArray(Xdr.STRING(args[lpc + 1]));
		    xba.PUT(args[lpc + 1]);
		    data = ByteArray.concat(data, xba.toBytes());
		}
		else if( args[lpc].equals("-start-permission-set") )
		{
		    permissions = new Vector();
		}
		else if( args[lpc].equals("-end-permission-set") )
		{
		    ps = new PermissionSet(permissions);
		    permissions = null;
		    data = ByteArray.concat(data, ps.toBytes());
		}
		else if( args[lpc].equals("-enum-permission") )
		{
		    spair = this.split(args[lpc + 1], '=');
		    perm = new EnumerationPermission(spair[0], spair[1]);
		    if( permissions != null )
		    {
			permissions.addElement(perm);
		    }
		    else
		    {
			data = ByteArray.concat(data, perm.toBytes());
		    }
		}
		else if( args[lpc].equals("-int-permission") )
		{
		    long integer;
			
		    spair = this.split(args[lpc + 1], '=');
		    integer = Long.decode(spair[1]).longValue();
		    perm = new IntegerPermission(spair[0], integer);
		    if( permissions != null )
		    {
			permissions.addElement(perm);
		    }
		    else
		    {
			data = ByteArray.concat(data, perm.toBytes());
		    }
		}
		else if( args[lpc].equals("-ip-permission") )
		{
		    spair = this.split(args[lpc + 1], '=');
		    perm = new IPAddressPermission(spair[0], spair[1]);
		    if( permissions != null )
		    {
			permissions.addElement(perm);
		    }
		    else
		    {
			data = ByteArray.concat(data, perm.toBytes());
		    }
		}
	    }

	    {
		File file;

		file = (File)node.getCapability("file");
		ByteArray.toFile(file, data);
	    }
	}
	node.shutdown();
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "MaterialBuilder["
	    + super.toString()
	    + "]";
    }
}
