/*
 * NeighborManager.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.authenticator;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import bees.core.Assert;
import bees.core.TypeID;
import bees.core.HashMap;
import bees.core.ObjectMap;
import bees.core.NeighborImpl;
import bees.core.ExtendedProtocolPresentation;

import bees.core.resource.Logger;
import bees.core.resource.NetworkService;
import bees.core.resource.NodeOSNeighbor;
import bees.core.resource.NodeOSNeighborFactory;

import bees.core.security.PermissionSet;
import bees.core.security.EnumerationException;

/**
 * The NeighborManager is used to track neighbors as they are discovered and
 * lost.
 */
final class NeighborManager
{
    /**
     * Notify any interested parties that a new neighbor has been added.
     */
    private static final int NOTIFY_TYPE_ADD = 0;
    
    /**
     * Notify any interested parties that a neighbor has been removed.
     */
    private static final int NOTIFY_TYPE_REM = 1;

    /**
     * Factory used to create NodeOS neighbors from their advertisements.
     */
    private NodeOSNeighborFactory nf = new NodeOSNeighborFactory();

    /**
     * The local material that is used to produce neighbor objects.
     */
    private final Hashtable localMaterial = new Hashtable();

    /**
     * Map of advertisement finger prints to Neighbor objects.
     */
    private final ObjectMap fpToNeighbors = new HashMap();

    /**
     * The reverse of the above mapping.
     */
    private final ObjectMap neighbors = new HashMap();

    /**
     * Set of ExtendedProtocolPresentations that are interested in hearing
     * about new/lost neighbors.
     */
    private final Vector interestedProtocols = new Vector();

    /**
     * Construct an empty NeighborManager.
     */
    NeighborManager()
    {
    }

    /**
     * @param ni The neighbor that will be broadcasting the material.
     * @param am The local material that is used to produce neighbor objects.
     */
    void addLocalMaterial(NeighborImpl ni, AdvertiseMaterial am)
    {
	this.localMaterial.put(ni, am);
    }

    /**
     * @return The local material that is used to produce neighbor objects.
     */
    Hashtable getLocalMaterial()
    {
	return this.localMaterial;
    }

    /**
     * Add a protocol interested in hearing about new/lost neighbors.
     *
     * @param epp An ExtendedProtocolPresentation that is interested in hearing
     * about new/lost neighbors.
     */
    public synchronized
	void addInterestedProtocol(ExtendedProtocolPresentation epp)
    {
	Enumeration enum;
	
	this.interestedProtocols.addElement(epp);
	enum = this.neighbors();
	while( enum.hasMoreElements() )
	{
	    NeighborImpl ni;

	    ni = (NeighborImpl)enum.nextElement();
	    epp.addNeighbor(ni);
	}
    }

    /**
     * Remove a protocol from the set that wishes to learn about new/lost
     * neighbors.
     *
     * @param epp An ExtendedProtocolPresentation that is no longer interested
     * in hearing about new/lost neighbors.
     */
    public synchronized
	void remInterestedProtocol(ExtendedProtocolPresentation epp)
    {
	this.interestedProtocols.removeElement(epp);
    }

    /**
     * @param type A NOTIFY_TYPE value.
     * @param ni The neighbor object this concerns.
     */
    private void notifyInterestedProtocols(int type, NeighborImpl ni)
    {
	int lpc, size;
	
	size = this.interestedProtocols.size();
	for( lpc = 0; lpc < size; lpc++ )
	{
	    ExtendedProtocolPresentation epp;

	    epp = (ExtendedProtocolPresentation)this.interestedProtocols.
		elementAt(lpc);
	    switch( type )
	    {
	    case NOTIFY_TYPE_ADD:
		epp.addNeighbor(ni);
		break;
	    case NOTIFY_TYPE_REM:
		epp.remNeighbor(ni);
		break;
	    default:
		Assert.check(false);
		break;
	    }
	}
    }

    /**
     * @param ni The source of the peer material.
     * @param localServices The local NetworkServices capability.
     * @param remoteServices The remote NetworkServices capability.
     * @param peerMaterial The material advertised by an unknown neighbor.
     * @return An initialized NeighborImpl.
     * @throws EnumerationException If the material could not be mapped to
     * the protocols and addresses this object was given access to.
     */
    NeighborImpl create(NeighborImpl ni,
			NetworkService localServices,
			NetworkService remoteServices,
			AdvertiseMaterial peerMaterial)
	throws EnumerationException
    {
	NeighborImpl retval = null;
	AdvertiseMaterial lam;
	PermissionSet ps;
	
	lam = (AdvertiseMaterial)this.localMaterial.get(ni);
	ps = new PermissionSet(new PermissionSet[] {
	    lam.getAddress(),
	    peerMaterial.getAddress(),
	});
	Logger.log("ni = "
		   + ((NodeOSNeighbor)ni).toVerboseString()
		   + "; local="
		   + localServices);
	retval = this.nf.create(localServices,
				remoteServices,
				ps);
	return retval;
    }

    /**
     * @param ni The neighbor to add.
     * @param fp The finger print of the neighbor's material.
     */
    synchronized void addNeighbor(NeighborImpl ni, TypeID fp)
    {
	this.neighbors.put(ni, fp);
	this.fpToNeighbors.put(fp, ni);

	this.notifyInterestedProtocols(NOTIFY_TYPE_ADD, ni);
    }

    /**
     * @param ni The neighbor to remove.
     * @return The material finger print of the removed neighbor.
     */
    synchronized TypeID remNeighbor(NeighborImpl ni)
    {
	TypeID retval;

	if( (retval = (TypeID)this.neighbors.remove(ni)) != null )
	{
	    this.fpToNeighbors.remove(retval);
	    this.notifyInterestedProtocols(NOTIFY_TYPE_REM, ni);
	}
	return retval;
    }

    /**
     * @return An Enumeration of all the NeighborImpl's currently known to
     * the manager.
     */
    Enumeration neighbors()
    {
	return this.neighbors.keys();
    }

    /**
     * @return An ObjectMap that converts material finger prints to neighbor
     * objects.
     */
    ObjectMap namedNeighbors()
    {
	return this.fpToNeighbors;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "NeighborManager[nf="
	    + this.nf
	    + "; localMaterial="
	    + this.localMaterial
	    + "; fpToNeighbors="
	    + this.fpToNeighbors
	    + "; neighbors="
	    + this.neighbors
	    + "; interestedProtocols="
	    + this.interestedProtocols
	    + "]";
    }
}
