/*
 * BenchmarkParameters.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.benchmark;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import bees.core.Node;
import bees.core.NeighborImpl;
import bees.core.ProtocolSession;
import bees.core.ProtocolParameters;

import bees.core.resource.Clock;
import bees.core.resource.Logger;

public class BenchmarkParameters
    extends ProtocolParameters
{
    static final long DEFAULT_STAT_INTERVAL = 1000; // ms

    static final int DEFAULT_MIN_PAYLOAD = 0;

    static final int DEFAULT_MAX_PAYLOAD = 1200;

    static final int DEFAULT_PAYLOAD_STEP = 100;

    static final int DEFAULT_CYCLES = 250000;
    
    static final int DEFAULT_GAP = 1000;

    long statInterval;

    int minPayload;

    int maxPayload;

    int payloadStep;

    int cycles;

    int gap;

    Clock clock;
    
    public BenchmarkParameters(ProtocolSession ps)
    {
	super(ps.getByteArray("apps/benchmark/protocol.properties"));
    }

    public void initialize(Node node, BenchmarkSession bs)
    {
	this.statInterval =
	    super.getMillisecondValue("parameter.stat-interval",
				      DEFAULT_STAT_INTERVAL);
	this.minPayload = super.getInteger("parameter.min-payload",
					   DEFAULT_MIN_PAYLOAD);
	this.maxPayload = super.getInteger("parameter.max-payload",
					   DEFAULT_MAX_PAYLOAD);
	this.payloadStep = super.getInteger("parameter.payload-step",
					    DEFAULT_MAX_PAYLOAD);
	this.cycles = super.getInteger("parameter.cycles",
				       DEFAULT_CYCLES);
	this.gap = super.getInteger("parameter.gap", DEFAULT_GAP);
	this.clock = (Clock)node.getCapability("clock");

	try
	{
	    Enumeration enum;

	    enum = node.getCapabilityMap("neighbors").keys();
	    while( enum.hasMoreElements() )
	    {
		NeighborImpl ni;
		String name;

		name = (String)enum.nextElement();
		ni = (NeighborImpl)node.getCapability("neighbors/" + name);
		Logger.log("adding " + name);
		bs.addNeighbor(node, ni);
		if( name.startsWith("src") )
		    bs.previous = ni.getNeighbor(bs);
		else if( name.startsWith("dst") )
		    bs.next = ni.getNeighbor(bs);
	    }
	}
	catch(NoSuchElementException e)
	{
	}

	Logger.log(Logger.INFO, "Statistic interval: " + this.statInterval);
	Logger.log(Logger.INFO, "Minimum payload: " + this.minPayload);
	Logger.log(Logger.INFO, "Maximum payload: " + this.maxPayload);
	Logger.log(Logger.INFO, "Payload step: " + this.payloadStep);
	Logger.log(Logger.INFO, "Cycles: " + this.cycles);
	Logger.log(Logger.INFO, "Clock: " + this.clock);
    }

    public long getStatInterval()
    {
	return this.statInterval;
    }

    public int getMinPayload()
    {
	return this.minPayload;
    }

    public int getMaxPayload()
    {
	return this.maxPayload;
    }

    public int getPayloadStep()
    {
	return this.payloadStep;
    }

    public Clock getClock()
    {
	return this.clock;
    }

    public String toString()
    {
	return "BenchmarkParameters[statInterval="
	    + this.statInterval
	    + "; minPayload="
	    + this.minPayload
	    + "; maxPayload="
	    + this.maxPayload
	    + "; payloadStep="
	    + this.payloadStep
	    + "; clock="
	    + this.clock
	    + "]";
    }
}
