/*
 * PrimerCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.benchmark;

import java.io.IOException;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.Capsule;
import bees.core.Neighbor;
import bees.core.NodeCache;
import bees.core.ProtocolSession;
import bees.core.EncodingException;

import bees.core.resource.Logger;

import bees.core.security.IntegerPermission;

public final class PrimerCapsule
    extends Capsule
    implements NodeCache.EventHandler
{
    static final IntegerPermission PRIME_FREQUENCY =
	new IntegerPermission("interval", 500);

    private boolean pong = false;
    private int hop = 1;
    
    public PrimerCapsule()
    {
    }

    public void removed(Node node, ProtocolSession ps)
    {
    }

    public IntegerPermission collected(Node node, ProtocolSession ps)
    {
	BenchmarkSession bs;

	bs = (BenchmarkSession)ps;
	synchronized( bs )
	{
	    if( bs.next != null )
	    {
		try
		{
		    bs.next.send(this);
		}
		catch(EncodingException e)
		{
		    Logger.log(Logger.ERROR, e);
		}
		catch(IOException e)
		{
		    Logger.log(Logger.WARNING, e);
		}
	    }
	}
	return PRIME_FREQUENCY;
    }
    
    public void encode(Xdr xdr)
    {
	xdr.PUT(this.pong);
	xdr.PUT(this.hop);
    }

    public void decode(Xdr xdr)
    {
	this.pong = xdr.BOOLEAN();
	this.hop = xdr.INT();
    }

    public void evaluate(Node node, ProtocolSession ps)
	throws EncodingException,
	       IOException
    {
	BenchmarkSession bs = (BenchmarkSession)ps;

	if( !this.pong )
	{
	    Neighbor next;
	    
	    switch( this.hop )
	    {
	    case 1:
		if( (next = bs.setPrevious(this.getSource())) != null )
		{
		    Logger.log("Sending to final neighbor.");
		    this.hop += 1;
		    next.send(this);
		}
		else
		{
		    /* Next neighbor hasn't been found yet, drop the packet. */
		    Logger.log("Still waiting for third neighbor.");
		}
		break;
	    case 2:
		Logger.log("Returning...");
		bs.setPrevious(this.getSource());
		this.pong = true;
		this.hop -= 1;
		this.getSource().send(this);
		break;
	    }
	}
	else
	{
	    switch( this.hop )
	    {
	    case 1:
		Logger.log("Returning...");
		this.hop -= 1;
		bs.previous.send(this);
		break;
	    case 0:
		bs.getCache().remove("primer");
		
		bs.currentRun.fromCapsule(this);
		bs.turnOver();
		break;
	    }
	}
    }
    
    public String toString()
    {
	return "PrimerCapsule["
	    + super.toString()
	    + "; pong="
	    + this.pong
	    + "; hop="
	    + this.hop
	    + "]";
    }
}
