/*
 * DiscoveryParameters.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.discovery;

import java.util.NoSuchElementException;

import bees.core.Node;
import bees.core.Assert;
import bees.core.ProtocolParameters;

import bees.core.resource.Logger;

import bees.core.security.PublicKey;
import bees.core.security.IntegerPermission;

/**
 * The DiscoveryParameters class is used to hold the various parameters for
 * the flow.
 */
public final class DiscoveryParameters
    extends ProtocolParameters
{
    /**
     * Default time before any discovered material is dropped.
     */
    private static final int DEFAULT_EXPIRATION_TIME = 45 * 1000;
    
    /**
     * Default frequency of advertisements.
     */
    private static final int DEFAULT_ADVERTISE_FREQUENCY = 15 * 1000;

    /**
     * The material expiration time.
     */
    private IntegerPermission expirationTime;
    
    /**
     * The advertisement frequency.
     */
    private IntegerPermission advertiseFrequency;

    /**
     * The public key used to check peer material.
     */
    private PublicKey peerPublicKey;

    /**
     * The crypto algorithm to use when checking the material.
     */
    private String algorithm;

    /**
     * Construct a DiscoveryParameters object using information obtained from
     * the given values.
     *
     * @param dps The ProtocolSession for this flow.
     */
    DiscoveryParameters(DiscoveryProtocolSession dps)
    {
	super(dps.getByteArray("apps/discovery/protocol.properties"));
    }

    /**
     * Initialize the parameters.
     *
     * @param node The current Node object.
     */
    public void initialize(Node node)
    {
	Assert.require(node);

	try
	{
	    this.peerPublicKey = (PublicKey)
		node.getCapability("peerPublicKey");
	}
	catch(NoSuchElementException e)
	{
	    Logger.log(Logger.WARNING,
		       "No peerPublicKey, cannot check signatures.");
	}

	this.expirationTime = new IntegerPermission(
		"interval",
		super.getMillisecondValue("parameter.expiration-time",
					  DEFAULT_EXPIRATION_TIME));

	this.advertiseFrequency = new IntegerPermission(
		"interval",
		super.getMillisecondValue("parameter.advertise-frequency",
					  DEFAULT_ADVERTISE_FREQUENCY));

	this.algorithm = super.getString("parameter.algorithm", "DSA");
	
	Logger.log(Logger.INFO,
		   "Expiration time: " + this.expirationTime);
	Logger.log(Logger.INFO,
		   "Advertise frequency time: " + this.advertiseFrequency);
	if( this.peerPublicKey != null )
	{
	    Logger.log(Logger.INFO, "Crypto algorithm: " + this.algorithm);
	}
    }

    /**
     * @return The material expiration time.
     */
    IntegerPermission getExpirationTime()
    {
	return this.expirationTime;
    }

    /**
     * @return The advertisement frequency.
     */
    IntegerPermission getAdvertiseFrequency()
    {
	return this.advertiseFrequency;
    }

    /**
     * @return The public key used to check peer material.
     */
    PublicKey getPublicKey()
    {
	return this.peerPublicKey;
    }

    /**
     * @return The crypto algorithm to use when checking the material.
     */
    String getAlgorithm()
    {
	return this.algorithm;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "DiscoveryParameters[expirationTime="
	    + this.expirationTime
	    + "; advertiseFrequency="
	    + this.advertiseFrequency
	    + "; peerPublicKey="
	    + this.peerPublicKey
	    + "; algorithm="
	    + this.algorithm
	    + "]";
    }
}
