/*
 * DiscoveryProtocolPresentation.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.discovery;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.Strict;
import bees.core.TypeID;
import bees.core.NeighborImpl;
import bees.core.ExternalCapsule;
import bees.core.CapabilityReference;
import bees.core.ExtendedProtocolPresentation;

/**
 * A DiscoveryProtocolPresentation is used to communicate with the
 * DiscoveryProtocolSession in the protocol's domain.
 */
public class DiscoveryProtocolPresentation
    extends ExtendedProtocolPresentation
{
    /**
     * Construct an empty DiscoveryProtocolPresentation.
     */
    public DiscoveryProtocolPresentation()
    {
    }

    /**
     * Ask the protocol to advertise some data for us.
     *
     * @param ni The neighbor to use when broadcasting the advertisement.
     * @param material The material to advertise.
     * @param signature The signature covering "material" or NULL.
     */
    public void advertise(NeighborImpl ni, byte material[], byte signature[])
    {
	ExternalCapsule ec;
	Xdr xdr;
	
	ec = new ExternalCapsule(new CapabilityReference[] {
	    new CapabilityReference(ni)
	}).init();
	xdr = ec.getXDR();
	xdr.PUT(DiscoveryProtocolSession.METHOD_ADVERTISE);
	xdr.PUT(material.length);
	xdr.PUT(material);
	if( signature != null )
	{
	    xdr.PUT(signature.length);
	    xdr.PUT(signature);
	}
	else
	{
	    xdr.PUT(0);
	}
	this.sendCapsule(ec);
    }
    
    /**
     * Tell the protocol that some discovered material has expired and should
     * be removed from its internal state.
     *
     * XXX Is this still needed?
     *
     * @param fp The finger print of the expired data.
     */
    public void died(TypeID fp)
    {
	ExternalCapsule ec;
	Xdr xdr;
	
	ec = new ExternalCapsule().init();
	xdr = ec.getXDR();
	xdr.PUT(DiscoveryProtocolSession.METHOD_DIED);
	xdr.PUT(fp);
	this.sendCapsule(ec);
    }

    /**
     * Callback used to tell an application about a new neighbor.
     *
     * @param cr A reference to the neighbor object that received the
     * advertisement.
     * @param fp The finger print of the discovered material.
     * @param material The discovered material.
     * @param signature The signature of the material.
     */
    public void found(CapabilityReference cr,
		      TypeID fp, byte material[], byte signature[])
    {
    }

    /**
     * @see bees.core.ExtendedProtocolPresentation#userReceive(Node, int, ExternalCapsule)
     */
    protected void userReceive(Node node, int method, ExternalCapsule ec)
    {
	Xdr xdr;

	xdr = ec.getXDR();
	switch( method )
	{
	case DiscoveryProtocolSession.METHOD_FOUND:
	    this.found(ec.getReferences()[0],
		       xdr.TYPEID(),
		       xdr.BYTES(xdr.INT()),
		       xdr.BYTES(xdr.INT()));
	    break;
	default:
	    Strict.fail();
	    break;
	}
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "DiscoveryProtocolPresentation["
	    + super.toString()
	    + "]";
    }
}
