/*
 * HealthExtraParameters.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.health;

import kaffe.sensors.Sensor;

import java.util.Hashtable;
import java.util.NoSuchElementException;

import java.security.NoSuchAlgorithmException;

import java.security.spec.InvalidKeySpecException;

import bees.core.Node;
import bees.core.ObjectMap;
import bees.core.ProtocolSession;

import bees.core.resource.Clock;
import bees.core.resource.Logger;

import bees.core.security.PublicKey;
import bees.core.security.PrivateKey;
import bees.core.security.PublicKeySpec;

/**
 * HealthExtraParameters holds the privileged parameters for a health flow.
 * It is not required for the flow to run, it can still forward and perform
 * unprivileged operations, however, any crypto/sensors stuff will not work.
 *
 * @see bees.core.security.Signature
 */
public class HealthExtraParameters
    extends HealthParameters
{
    /**
     * The server's public key, used to verify server poll requests.
     */
    private PublicKeySpec serverPublicKey;

    /**
     * The server's private key, should only be given to the server.  Used to
     * sign poll requests.
     */
    private PrivateKey serverPrivateKey;

    /**
     * The local node's key, used to sign health reports.
     */
    private PrivateKey nodePrivateKey;

    /**
     * Map of node keys, used to sign/verify health data.
     */
    private ObjectMap nodeKeys;

    /**
     * A clock for adding time stamps to node health reports.
     */
    private Clock clock;

    /**
     * The table of sensors available on this node.
     */
    private Hashtable sensors;

    /**
     * Construct a HealthExtraParameters object for the given ProtocolSession.
     *
     * @param ps The HealthSession object that this parameters object should
     * be bound to.
     */
    public HealthExtraParameters(ProtocolSession ps)
    {
	super(ps);
    }

    /**
     * Initialize the parameters.
     *
     * @param node The current Node object.
     * @param ps The HealthSession object that this parameters object should
     * be bound to.
     */
    public void initialize(Node node, ProtocolSession ps)
    {
	super.initialize(node, ps);

	try
	{
	    /* Detect the motherboard sensors. */
	    this.sensors = Sensor.detect();
	}
	catch(UnsatisfiedLinkError e)
	{
	    Logger.log("Sensor native library not installed.");
	}
	catch(NoClassDefFoundError e)
	{
	    Logger.log("Cannot access sensor classes.");
	}
	catch(ExceptionInInitializerError e)
	{
	    Throwable cause;

	    cause = e.getException();
	    if( cause instanceof UnsatisfiedLinkError )
	    {
		Logger.log("Sensors not supported.");
	    }
	    else
	    {
		Logger.log(cause);
	    }
	}
	catch(Throwable th)
	{
	    Logger.log("phooey");
	}

	try
	{
	    /*
	     * Construct a public key from the bits included in the
	     * protocol.
	     */
	    this.serverPublicKey = new PublicKeySpec();
	    this.serverPublicKey.specFromBytes(ps.getByteArray("publicKey"));
	}
	catch(NoSuchAlgorithmException e)
	{
	    Logger.log(e);
	}
	catch(InvalidKeySpecException e)
	{
	    Logger.log(e);
	}
	
	try
	{
	    ObjectMap om;

	    /*
	     * Get the server key pair, we currently only care about the
	     * private key.
	     */
	    om = node.getCapabilityMap("serverKeyPair");
	    this.serverPrivateKey = (PrivateKey)om.get("private");
	}
	catch(NoSuchElementException e)
	{
	    Logger.log(Logger.DBG, e);
	}
	
	try
	{
	    /* The node private key. */
	    this.nodePrivateKey = (PrivateKey)
		node.getCapability("nodePrivateKey");
	}
	catch(NoSuchElementException e)
	{
	    Logger.log(Logger.DBG, e);
	}

	this.clock = (Clock)node.getCapability("clock");
	
	try
	{
	    /*
	     * The node keys.  We expect a map containing the names of the
	     * nodes, subdivided by the algorithms they have keys for, and
	     * then the keys.
	     */
	    this.nodeKeys = node.getCapabilityMap("nodeKeys");
	}
	catch(NoSuchElementException e)
	{
	    Logger.log(Logger.DBG, e);
	}

	if( this.sensors != null )
	{
	    Logger.log(Logger.INFO, "Sensors: " + this.sensors.keySet());
	}
	Logger.log(Logger.INFO, "Server public key: " + this.serverPublicKey);
	Logger.log(Logger.INFO, "Server private key: "
		   + this.serverPrivateKey);
	Logger.log(Logger.INFO, "Node private key: " + this.nodePrivateKey);
	if( this.nodeKeys != null )
	{
	    Logger.log(Logger.INFO, "Node key(s): " + this.nodeKeys.keys());
	}
    }

    /**
     * @return The server's public key, used to verify server poll requests.
     */
    public PublicKeySpec getServerPublicKey()
    {
	return this.serverPublicKey;
    }
    
    /**
     * @return The server's private key, should only be given to the server.
     * Used to sign poll requests.
     */
    public PrivateKey getServerPrivateKey()
    {
	return this.serverPrivateKey;
    }

    /**
     * @return The local node's key, used to sign health reports.
     */
    public PrivateKey getNodePrivateKey()
    {
	return this.nodePrivateKey;
    }

    /**
     * @return Map of node keys, used to sign/verify health data.
     */
    public ObjectMap getNodeKeys()
    {
	return this.nodeKeys;
    }

    /**
     * Get the public key for a given node.
     *
     * @param name The name of the node.
     * @return The PublicKey for the given node and configured algorithm.
     */
    public PublicKey getNodePublicKey(String name)
    {
	PublicKey retval = null;
	ObjectMap om;

	if( ((om = (ObjectMap)this.nodeKeys.get(name)) != null) &&
	    ((om = (ObjectMap)om.get(super.algorithm)) != null) )
	{
	    retval = (PublicKey)om.get("public");
	}
	return retval;
    }

    /**
     * @return A clock for adding time stamps to node health reports.
     */
    public Clock getClock()
    {
	return this.clock;
    }

    /**
     * @return The table of sensors available on this node.
     */
    public Hashtable getSensors()
    {
	return this.sensors;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "HealthExtraParameters["
	    + super.toString()
	    + "; serverPublicKey="
	    + this.serverPublicKey
	    + "; serverPrivateKey="
	    + this.serverPrivateKey
	    + "; nodePrivateKey="
	    + this.nodePrivateKey
	    + "; nodeKeys="
	    + this.nodeKeys
	    + "; clock="
	    + this.clock
	    + "; sensors="
	    + this.sensors
	    + "]";
    }
}
