/*
 * SimpleNodeHealth.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.health;

import java.util.Hashtable;
import java.util.Enumeration;

import java.io.IOException;

import kaffe.sensors.Sensor;
import kaffe.sensors.SensorFeature;

import bees.core.resource.Logger;

import bees.core.security.IntegerPermission;

/**
 * A SimpleNodeHealth object implements health polling using the kaffe.sensors
 * wrapper for motherboard sensors.  It will try and guess the acceptable
 * limits of the sensors by looking for SensorFeatures with names ending in
 * "_min" and "_max".
 *
 * @see apps.health.HealthExtraParameters
 */
public class SimpleNodeHealth
    extends NodeHealth
{
    /**
     * @param name The name of the node this health information pertains to.
     */
    public SimpleNodeHealth(String name)
    {
	super(name);
    }

    /**
     * @see apps.health.NodeHealth#poll(HealthSession)
     */
    public IntegerPermission poll(HealthSession hs)
    {
	HealthExtraParameters hp;
	IntegerPermission retval;
	Enumeration sensors;
	Hashtable ignored;

	retval = super.poll(hs);
	this.state = STATE_FINE;
	hp = (HealthExtraParameters)hs.getHealthParameters();
	ignored = hp.getIgnoredFeatures();
	sensors = hp.getSensors().elements();
	/* Walk all the available Sensors and their SensorFeatures. */
	while( sensors.hasMoreElements() )
	{
	    Enumeration features;
	    Sensor sensor;

	    sensor = (Sensor)sensors.nextElement();
	    features = sensor.getFeatures();
	    while( features.hasMoreElements() )
	    {
		SensorFeature sf, actual;
		String name;
		
		sf = (SensorFeature)features.nextElement();
		try
		{
		    name = sf.getName();
		    if( name.endsWith("_min") )
		    {
			/*
			 * A lower bound, lookup the actual SensorFeature and
			 * check the values.
			 */
			name = name.substring(0, name.length() - 4);
			if( !ignored.containsKey(name) )
			{
			    if( (actual = sensor.getFeature(name)) != null )
			    {
				double value;

				value = actual.read();
				Logger.log("SensorFeature: "
					   + name
					   + " = "
					   + value);
				if( Math.abs(value) < Math.abs(sf.read()) )
				{
				    Logger.log("Out of range: "
					       + actual
					       + "; "
					       + sf
					       + "; "
					       + actual.read()
					       + " < "
					       + sf.read());
				    this.state = STATE_ON_FIRE;
				}
			    }
			    else
			    {
				Logger.log("Minimum limit without actual? "
					   + name);
			    }
			}
			else
			{
			    /* Ignored... */
			}
		    }
		    else if( name.endsWith("_max") ||
			     name.endsWith("_over") )
		    {
			/*
			 * An upper bound, lookup the actual SensorFeature and
			 * check the values.
			 */
			name = name.substring(0, name.lastIndexOf('_'));
			if( !ignored.containsKey(name) )
			{
			    if( (actual = sensor.getFeature(name)) != null )
			    {
				double value;

				value = actual.read();
				if( Math.abs(value) > Math.abs(sf.read()) )
				{
				    Logger.log("Out of range: "
					       + actual
					       + "; "
					       + sf
					       + "; "
					       + actual.read()
					       + " > "
					       + sf.read());
				    this.state = STATE_ON_FIRE;
				}
			    }
			    else
			    {
				Logger.log("Maximum limit without actual? "
					   + name);
			    }
			}
			else
			{
			    /* Ignored... */
			}
		    }
		    else if( !ignored.containsKey(sf.getName()) )
		    {
			/*
			Logger.log("Unchecked SensorFeature: "
				   + sf.getName()
				   + " = "
				   + sf.read());
			*/
		    }
		}
		catch(IOException e)
		{
		    Logger.log(e);
		}
	    }
	}
	return retval;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "SimpleNodeHealth["
	    + super.toString()
	    + "]";
    }
}
