/*
 * LivenessApplication.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.liveness;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import bees.core.Node;
import bees.core.Application;
import bees.core.EventChannel;
import bees.core.NeighborImpl;
import bees.core.ProtocolFlow;
import bees.core.CapabilityEnvelope;
import bees.core.CapabilityReference;
import bees.core.MissingCapabilityException;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;

import bees.core.bdomain.CapabilityTree;

/**
 * The LivenessApplication is a test application for the liveness protocol.
 */
public final class LivenessApplication
    extends Application
{
    /**
     * @see java.lang.Object#Object()
     */
    public LivenessApplication()
    {
    }

    /**
     * @see bees.core.Application#start(Node, String[])
     */
    public void start(Node node, String args[])
    {
	CapabilityTree ct, neighborTree = null;
	LivenessPresentation lp;
	ProtocolFlow pf;

	/*
	 * Create a custom presentation for talking with the protocol and
	 * handling any interesting events.
	 */
	lp = new LivenessPresentation() {
		public void lost(CapabilityReference cr)
		{
		    NeighborImpl ni;

		    try
		    {
			ni = (NeighborImpl)cr.dereference();
			Logger.log("Neighbor died: ni=" + ni);
			this.remNeighbor(ni);
		    }
		    catch(MissingCapabilityException e)
		    {
			Logger.log(e);
		    }
		}
	    };
	try
	{
	    /* Grab the tree of neighbors. */
	    neighborTree =
		new CapabilityTree("user/neighbors",
				   node.getCapabilityMap("neighbors"));
	}
	catch(NoSuchElementException e)
	{
	    Logger.log(e);
	}
	/* Describe the protocol. */
	pf = new ProtocolFlow();
	pf.setPrimaryProtocol("apps.liveness.LivenessProtocol");
	ct = new CapabilityTree(new CapabilityEnvelope[] {
	    /* Protocol version string. */
	    new CapabilityEnvelope("user/liveVersion",
				   node.getCapability("liveVersion")),
	    new CapabilityEnvelope("user/clock",
				   node.getCapability("protocolClock")),
	    new CapabilityEnvelope("node", node).
		revokePermissions(new PermissionSet(new Permission[] {
		    Node.THREAD_START,
		    Node.PROTOCOL_REGISTER,
		})),
	    new CapabilityEnvelope("logger",
				   node.getCapability("protocolLogger")),
	    new CapabilityEnvelope(node.getCapability("protocolClasses")),
	    new CapabilityEnvelope(EventChannel.currentChannel()),
	},
				new CapabilityTree[] {
	    neighborTree,
	});
	try
	{
	    Enumeration enum;
	    
	    node.register(lp, pf, ct);
	    Logger.log("Started LivenessProtocol.");
	    lp.register();
	    /* Tell it what neighbors we want monitored. */
	    enum = node.getCapabilityMap("neighbors").elements();
	    while( enum.hasMoreElements() )
	    {
		lp.monitor((NeighborImpl)enum.nextElement());
	    }
	}
	catch(Throwable th)
	{
	    Logger.log(th);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LivenessApplication["
	    + super.toString()
	    + "]";
    }
}
