/*
 * LivenessParameters.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.liveness;

import bees.core.Node;
import bees.core.Assert;
import bees.core.ProtocolParameters;

import bees.core.resource.Clock;
import bees.core.resource.Logger;

import bees.core.security.IntegerPermission;

/**
 * The LivenessParameters class is used to track the parameters for a liveness
 * protocol flow.
 */
final class LivenessParameters
    extends ProtocolParameters
{
    /**
     * The default frequency for poking neighbors.
     */
    private static final int DEFAULT_POKE_FREQUENCY = 5 * 1000;

    /**
     * The default time to wait before considering a neighbor as lost.
     */
    private static final int DEFAULT_EXPIRATION_TIME = 20 * 1000;

    /**
     * A Clock capability used to track response times.
     */
    private Clock clock;

    /**
     * The frequency for poking neighbors.
     */
    private IntegerPermission pokeFrequency;
    
    /**
     * The time to wait before considering a neighbor as lost.
     */
    private IntegerPermission expirationTime;

    /**
     * Construct a LivenessParameters object from the given values.
     *
     * @param ls The LivenessSession for this flow.
     */
    LivenessParameters(LivenessSession ls)
    {
	super(ls.getByteArray("apps/liveness/protocol.properties"));
    }

    /**
     * Initialize the parameters.
     *
     * @param node The current Node object.
     */
    public void initialize(Node node)
    {
	Assert.require(node);
	
	this.clock = (Clock)node.getCapability("clock");
	
	this.pokeFrequency = new IntegerPermission(
		"interval",
		super.getMillisecondValue("parameter.poke-frequency",
					  DEFAULT_POKE_FREQUENCY));

	this.expirationTime = new IntegerPermission(
		"interval",
		super.getMillisecondValue("parameter.expiration-time",
					  DEFAULT_EXPIRATION_TIME));

	Logger.log(Logger.INFO,
		   "Expiration time: " + this.expirationTime);
	Logger.log(Logger.INFO,
		   "Poke frequency time: " + this.pokeFrequency);
    }

    /**
     * @return A Clock capability used to track response times.
     */
    Clock getClock()
    {
	return this.clock;
    }

    /**
     * @return The frequency for poking neighbors.
     */
    IntegerPermission getPokeFrequency()
    {
	return this.pokeFrequency;
    }

    /**
     * @return The time to wait before considering a neighbor as lost.
     */
    IntegerPermission getExpirationTime()
    {
	return this.expirationTime;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LivenessParameters[clock="
	    + this.clock
	    + "; pokeFrequency="
	    + this.pokeFrequency
	    + "; expirationTime="
	    + this.expirationTime
	    + "]";
    }
}
