/*
 * LivenessSession.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.liveness;

import java.util.Enumeration;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.Strict;
import bees.core.Neighbor;
import bees.core.NeighborImpl;
import bees.core.ExternalCapsule;
import bees.core.CapabilityReference;
import bees.core.MulticastNeighborhood;
import bees.core.ExtendedProtocolSession;
import bees.core.MissingCapabilityException;

import bees.core.resource.Logger;

/**
 * The LivenessSession tracks information for the liveness protocol and
 * manages communication with any interested applications.
 */
public final class LivenessSession
    extends ExtendedProtocolSession
{
    /**
     * Major version number.
     */
    static final int VERSION_MAJOR = 0;

    /**
     * Minor version number.
     */
    static final int VERSION_MINOR = 5;

    /**
     * Method identifier for a monitor request.
     */
    static final int METHOD_MONITOR = METHOD_USER + 1;
    
    /**
     * Method identifier for a lost neighbor message.
     */
    static final int METHOD_LOST = METHOD_USER + 2;

    /**
     * The parameters for this protocol flow.
     */
    private LivenessParameters lp;

    /**
     * The neighborhood to use when multicasting PokeCapsules.
     */
    private final MulticastNeighborhood neighborhood =
	new MulticastNeighborhood();

    /**
     * Construct an empty LivenessSession.
     */
    public LivenessSession()
    {
    }

    /**
     * @param lp The LivenessParameters to use for this protocol flow.
     */
    public void setLivenessParameters(LivenessParameters lp)
    {
	this.lp = lp;
    }

    /**
     * @return The LivenessParameters to use for this protocol flow.
     */
    public LivenessParameters getLivenessParameters()
    {
	return this.lp;
    }

    /**
     * @return The neighborhood to use when multicasting PokeCapsules.
     */
    public MulticastNeighborhood getNeighborhood()
    {
	return this.neighborhood;
    }

    /**
     * Notify the applications that the neighbor is unresponsive.
     *
     * @param ni The unresponsive neighbor.
     */
    public void lostNeighbor(NeighborImpl ni)
    {
	ExternalCapsule ec;
	Enumeration enum;
	int lpc, size;
	Xdr xdr;

	Logger.log(Logger.INFO,
		   "Lost neighbor: "
		   + ni.getFingerPrint());
	/* Stop sending to the neighbor. */
	this.neighborhood.removeNeighbor(ni);
	ec = new ExternalCapsule(new CapabilityReference[] {
	    new CapabilityReference(ni),
	}).init();
	xdr = ec.getXDR();
	xdr.PUT(METHOD_LOST);
	super.notifyInterests(ec);
    }

    /**
     * @see bees.core.ProtocolSession#attachNeighbor(Node, Neighbor)
     */
    public void attachNeighbor(Node node, Neighbor neighbor)
    {
	/* Use our specialized Neighbor type. */
	super.attachNeighbor(node, new LiveNeighbor(neighbor.
						    getNeighborImpl()));
    }
    
    /**
     * @see bees.core.ProtocolSession#detachNeighbor(Node, Neighbor)
     */
    public void detachNeighbor(Node node, Neighbor neighbor)
    {
	this.neighborhood.removeNeighbor(neighbor.getNeighborImpl());
	super.getCache().remove(neighbor);
	super.detachNeighbor(node, neighbor);
    }

    /**
     * @see bees.core.ExtendedProtocolSession#userReceive(Node, int, ExternalCapsule)
     */
    public synchronized void userReceive(Node node,
					 int method,
					 ExternalCapsule ec)
    {
	Xdr xdr;

	xdr = ec.getXDR();
	switch( method )
	{
	case METHOD_MONITOR:
	    try
	    {
		NeighborImpl ni;
		LiveNeighbor ln;
		
		ni = (NeighborImpl)ec.getReferences()[0].dereference();
		ln = (LiveNeighbor)ni.getNeighbor(this);
		/* Add the neighbor to the cache. */
		if( super.getCache().put(ln,
					 ln,
					 this.lp.getExpirationTime()) == null )
		{
		    Logger.log(Logger.INFO,
			       "Monitoring neighbor: "
			       + ni.getFingerPrint());
		    this.neighborhood.addNeighbor(ni);
		}
		else
		{
		    Logger.log(Logger.INFO,
			       "Already monitoring neighbor: "
			       + ni.getFingerPrint());
		}
	    }
	    catch(MissingCapabilityException e)
	    {
		Logger.log(e);
	    }
	    break;
	default:
	    Strict.fail();
	    break;
	}
	ec.getSource().recycleCapsule(ec);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LivenessSession["
	    + super.toString()
	    + "; lp="
	    + this.lp
	    + "; neighborhood="
	    + this.neighborhood
	    + "]";
    }
}
