/*
 * ConfigurationCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.pathfinder;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.Panic;
import bees.core.Capsule;
import bees.core.Neighbor;
import bees.core.NodeCache;
import bees.core.NeighborImpl;
import bees.core.ProtocolSession;
import bees.core.EncodingException;
import bees.core.ReencodableCapsule;

import bees.core.resource.Logger;

import bees.core.security.IntegerPermission;

/**
 * Capsule used to tell neighbors about the path to the root of the tree.  A
 * capsule of this type will be added to the NodeCache and continuously
 * transmitted if this node is the root of the tree or a root neighbor has
 * been found.
 */
public class ConfigurationCapsule
    extends Capsule
    implements ReencodableCapsule, NodeCache.EventHandler
{
    /**
     * Cached reference to the root Neighbor.
     */
    private NeighborImpl rootNeighbor;
    
    /**
     * Hops to the root.
     */
    private int hops;

    /**
     * Construct an empty ConfigurationCapsule.
     */
    public ConfigurationCapsule()
    {
    }

    /**
     * @param rootNeighbor Cached reference to the root Neighbor.
     */
    public void setRootNeighbor(Neighbor rootNeighbor)
    {
	if( rootNeighbor != null )
	{
	    this.rootNeighbor = rootNeighbor.getNeighborImpl();
	}
	else
	{
	    this.rootNeighbor = null;
	}
    }

    /**
     * @return Hops to the root.
     */
    public int getHops()
    {
	return this.hops;
    }

    /**
     * @param hops Hops to the root.
     */
    public void setHops(int hops)
    {
	this.hops = hops;
    }

    /**
     * @see bees.core.NodeCache.EventHandler#removed(Node, ProtocolSession)
     */
    public void removed(Node node, ProtocolSession ps)
    {
	throw new Panic("ConfigurationCapsule shouldn't be removed!");
    }

    /**
     * @see bees.core.NodeCache.EventHandler#collected(Node, ProtocolSession)
     */
    public IntegerPermission collected(Node node, ProtocolSession ps)
    {
	IntegerPermission retval;
	PathfinderSession pfs;

	Logger.log(Logger.DBG, "Sending config capsule!");
	pfs = (PathfinderSession)ps;
	retval = pfs.getPathfinderParameters().getUpdateFrequency();
	synchronized( pfs )
	{
	    if( (this.rootNeighbor != null) || pfs.isRoot() )
	    {
		try
		{
		    /* Tell the neighborhood who we think the root is. */
		    pfs.getPeers().send(this);
		}
		catch(EncodingException e)
		{
		    Logger.log(e);
		}
	    }
	    else
	    {
		/* Don't know who the root is yet. */
	    }
	}
	return retval;
    }

    /**
     * @see bees.core.Capsule#encode(Xdr)
     */
    public void encode(Xdr xdr)
    {
	xdr.SKIP(Xdr.INT);
    }

    /**
     * @see bees.core.ReencodableCapsule#reencode(Xdr, NeighborImpl)
     */
    public void reencode(Xdr xdr, NeighborImpl ni)
    {
	if( ni == this.rootNeighbor )
	{
	    xdr.PUT(-1);
	}
	else
	{
	    xdr.PUT(this.hops);
	}
    }

    /**
     * @see bees.core.Capsule#decode(Xdr)
     */
    public void decode(Xdr xdr)
    {
	this.hops = xdr.INT();
    }

    /**
     * @see bees.core.Capsule#evaluate(Node)
     */
    public void evaluate(Node node)
    {
	PathfinderSession ps;

	ps = (PathfinderSession)this.getSession();
	synchronized( ps )
	{
	    if( (this.hops > 0) && (this.hops < ps.getHops()) )
	    {
		/* Fewer hops, use this path. */
		Logger.log(Logger.INFO,
			   "** Found new root: "
			   + this.getSource().getNeighborImpl().
			   getFingerPrint()
			   + "; hops="
			   + this.hops);
		synchronized( ps )
		{
		    ps.setRootNeighbor(node, this.getSource());
		    ps.setHops(this.hops);
		    /*
		     * Add the PathfinderSession to the cache so it can clear
		     * the root neighbor after a timeout has been reached.
		     */
		    ps.getCache().put(ps,
				      ps,
				      ps.getPathfinderParameters().
				      getRootTimeout());
		}
	    }
	    else if( this.hops == ps.getHops() )
	    {
		/*
		 * Same number of hops, either its the same parent or we have
		 * more than one parent available.  If the latter, we'll just
		 * arbitrarily pick the first one and use them as the router.
		 * In any case, we need to refresh the timeout on the
		 * PathfinderSession in the NodeCache to keep it from clearing
		 * root neighbor.
		 */

		ps.getCache().refresh(ps,
				      ps.getPathfinderParameters().
				      getRootTimeout());
	    }
	    else if( this.hops == -1 )
	    {
		/*
		 * Negative hops is from a child, make sure they're in the
		 * list.  (They could've been removed at some other time by
		 * the case below).
		 */
		synchronized( ps )
		{
		    ps.getChildren().addNeighbor(this.getSource().
						 getNeighborImpl());
		}
		Logger.log(Logger.NOTICE,
			   "Child: "
			   + this.getSource().getNeighborImpl().
			   getFingerPrint()
			   + "; children="
			   + ps.getChildren());
	    }
	    else
	    {
		/*
		 * Extra hops, most likely from a sibling, remove them from
		 * the children list so they don't get multiple copies of
		 * MulticastCapsules.
		 */
		synchronized( ps )
		{
		    ps.getChildren().removeNeighbor(this.getSource().
						    getNeighborImpl());
		}
		Logger.log(Logger.NOTICE,
			   "Ignoring lesser root: "
			   + this.getSource().getNeighborImpl().
			   getFingerPrint()
			   + "; children="
			   + ps.getChildren());
	    }
	}
	this.addToPool();
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "ConfigurationCapsule["
	    + super.toString()
	    + "; rootNeighbor="
	    + this.rootNeighbor
	    + "; hops="
	    + this.hops
	    + "]";
    }
}
