/*
 * PathfinderProtocol.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.pathfinder;

import java.io.FileNotFoundException;

import bees.core.Panic;
import bees.core.Protocol;
import bees.core.DLProtocol;
import bees.core.IDTranslationProtocol;
import bees.core.CapsuleCompanionProtocol;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.CountPermission;
import bees.core.security.MinimumIntegerPermission;

/**
 * The PathfinderProtocol is a simple routing protocol that builds a tree of
 * nodes for forwarding client packets to a server and multicasting server
 * packets to the clients.
 */
public class PathfinderProtocol
    extends Protocol
    implements CapsuleCompanionProtocol
{
    /**
     * Construct the PathfinderProtocol description.
     */
    public PathfinderProtocol()
    {
	try
	{
	    this.startProtocolDefn();
	    {
		/* Add the config file. */
		this.addFile("apps/pathfinder/protocol.properties");

		this.addCapsule("apps.pathfinder.ConfigurationCapsule",
				new PermissionSet(new Permission[] {
				    new CountPermission("neighborhood", 1),
				    new CountPermission("returns", 1),
				    
				    new CountPermission("cached", 1),
				    new MinimumIntegerPermission("interval",
								 2 * 1000),
				    
				    new CountPermission("threadCount", 1),
				    new CountPermission("bufferCount", 2),
				}));
		this.setBootStrap("apps.pathfinder.PathfinderBootCapsule",
				  new PermissionSet(new Permission[] {
				      new CountPermission("threadCount", 0),
				      new CountPermission("bufferCount", 0),

				      new CountPermission(
					"transformTo.apps.pathfinder."
					+ "ConfigurationCapsule",
					1),
				      new MinimumIntegerPermission("interval",
								   0),
				  }));
		
		this.addCode("apps.pathfinder.SinkCapsule");
		this.addCode("apps.pathfinder.PathfinderParameters");
		this.setSession("apps.pathfinder.PathfinderSession");
		
		this.addCompanion(new DLProtocol());
		this.addCompanion(new IDTranslationProtocol());
	    }
	    this.endProtocolDefn();
	}
	catch(FileNotFoundException e)
	{
	    Logger.log(e);
	    throw new Panic(e);
	}
	catch(ClassNotFoundException e)
	{
	    Logger.log(e);
	    throw new Panic(e);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "PathfinderProtocol["
	    + super.toString()
	    + "]";
    }
}
