/*
 * ApplicationBDomain.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import edu.utah.janos.nodeos.Domain;

import bees.core.resource.Logger;

import bees.core.bdomain.BDomain;
import bees.core.bdomain.BDomainLocal;

/**
 * A domain for starting up a Bees application.
 */
public final class ApplicationBDomain
{
    /**
     * The current application for this domain.
     */
    static final BDomainLocal CURRENT_APPLICATION = new BDomainLocal() {
	    protected Object initialValue()
	    {
		throw new SecurityException("Not an application!");
	    }
	};

    /**
     * @return The current application for this domain.
     */
    public static Application currentApplication()
    {
	return (Application)CURRENT_APPLICATION.get();
    }
    
    /**
      * The "main" method for the domain.  This method will assume the first
      * argument is the name of an Application class and the rest are the
      * arguments to this class.  It will then construct an instance of the
      * Application class and call its "start" method with the given arguments.
      * 
      * @param args The arguments to the domain.
      * @param capabilities The capability arguments to the domain.
      */
    public static void main(String args[], ObjectTree capabilities)
    {
	Assert.require(args);
	Assert.require(capabilities);
	
	if( args.length >= 1 )
	{
	    boolean success = false;
	    
	    try
	    {
		String appArgs[] = new String[args.length - 1];
		Domain domain = Domain.currentDomain();
		Node node = Node.currentNode();
		BDomain bdomain = BDomain.currentBDomain();
		Application app;
		Class cl;

		/*
		 * XXX Should we really allow applications to access anything
		 * they want?
		 */
		domain.enableAccessControls(false);
		cl = domain.loadClass(args[0]);
		app = (Application)cl.newInstance();
		CURRENT_APPLICATION.set(app);
		/* We consumed the first argument, shift the rest. */
		System.arraycopy(args, 1,
				 appArgs, 0,
				 args.length - 1);
		try
		{
		    app.start(node, appArgs);
		    /* Success, don't immediately kill the domain. */
		    success = true;
		}
		catch(ThreadDeath td)
		{
		    /* Ignore. */
		    success = true;
		}
		catch(Throwable th)
		{
		    Logger.log("Application threw exception in start:");
		    Logger.log(th);
		}
	    }
	    catch(ClassNotFoundException e)
	    {
		Logger.log("Cannot find class: " + e.getMessage());
	    }
	    catch(ClassCastException e)
	    {
		Logger.log("Class `"
			   + args[0]
			   + "' isn't an Bees Application.");
	    }
	    catch(IllegalAccessException e)
	    {
		Logger.log("Cannot access cosntructor in class: "
			   + args[0]);
	    }
	    catch(InstantiationException e)
	    {
		Logger.log("Cannot instantiate Application class: "
			   + args[0]);
	    }
	    catch(ThreadDeath td)
	    {
		/* Ignore. */
	    }
	    catch(Throwable th)
	    {
		Logger.log("Ack we're toast cowboy!");
		th.printStackTrace();
	    }
	    finally
	    {
		if( !success )
		{
		    Node.currentNode().shutdown();
		}
	    }
	}
	else
	{
	    /* No application class. */
	    Logger.log("usage: <application class> [application args]");
	    Node.currentNode().shutdown();
	}
    }
    
    /**
     * Static only class.
     */   
    private ApplicationBDomain()
    {
    }
}
