/*
 * ByteArrayRequestCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;

import bees.core.resource.Logger;

/**
 * A ByteArrayRequestCapsule is used to request a FlowByteArrayType from a peer
 * node.
 */
final class ByteArrayRequestCapsule
    extends ReliableDataRequestCapsule
{
    /**
     * Construct an empty ByteArrayRequestCapsule.
     */
    ByteArrayRequestCapsule()
    {
    }

    /**
     * Construct a ByteArrayRequestCapsule for the given flow code identifier.
     *
     * @param flowCodeID The flow code identifier thats being requested.
     */
    ByteArrayRequestCapsule(TypeID flowCodeID)
    {
	super(DLProtocol.
	      getInProgressByteCode(FlowByteArrayType.
				    getBDomainFlowByteArray(flowCodeID).
				    getByteArrayType()));
    }

    /**
     * Resend this capsule to the destination.
     *
     * @param node The current node.
     * @param ps The DLProtocolSession for the flow this Capsule belongs to.
     */
    public void resend(Node node, ProtocolSession ps)
    {
	try
	{
	    ((DLProtocolSession)ps).getProgress().getPreviousNeighbor().
		send(this);
	}
	catch(EncodingException e)
	{
	    Logger.log(e);
	}
	catch(IOException e)
	{
	    Logger.log(e);
	}
    }

    /**
     * @see bees.core.Capsule#evaluate(Node)
     */
    public void evaluate(Node node)
    {
	TypeID specializer;

	if( (specializer = this.getSpecializer()) != null )
	{
	    FlowByteArrayType fct;
	    
	    /* Lookup the flow code's type. */
	    if( (fct = FlowByteArrayType.
		 getBDomainFlowByteArray(specializer)) != null )
	    {
		ByteArrayResponseCapsule crc;
		
		/* Make a response with the requested data. */
		crc = new ByteArrayResponseCapsule(this,
						   fct.getByteArrayType());
		crc.fromCapsule(this, specializer);
		try
		{
		    this.getSource().send(crc);
		}
		catch(EncodingException e)
		{
		    Logger.log(e);
		}
		catch(IOException e)
		{
		    Logger.log(e);
		}
	    }
	    else
	    {
		Logger.log(Logger.WARNING,
			   "No FlowByteArrayType for specializer: "
			   + specializer);
	    }
	}
	else
	{
	    Logger.log(Logger.WARNING,
		       "Empty flow code request specializer.");
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "ByteArrayRequestCapsule["
	    + super.toString()
	    + "]";
    }
}
