/*
 * CapabilityEnvelope.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import edu.utah.janosvm.sys.CrossTeamCloneable;

import edu.utah.janos.nodeos.Domain;

import bees.core.kernel.KernelObjectStore;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.PermissionSet;

import bees.core.bdomain.BDomain;

import bees.util.MD5;

/**
 * Container class used to transmit capabilities between domains.  Passing an
 * envelope to another Domain will introduce it to that domain, if it did not
 * previously have the Capability.  If the domain already had the Capability
 * then any extra permissions in the envelope will be granted to the Capability
 * in the domain.
 *
 * @see Capability
 * @see CapabilityReference
 * @see PermissionSet
 */
public final class CapabilityEnvelope
    implements Cloneable, CrossTeamCloneable
{
    /**
     * Find a CapabilityEnvelope with the given name in the given array.
     *
     * @param ce The array of envelopes to search.
     * @param name The envelope name to retrieve.
     * @return The first CapabilityEnvelope that matches the given name or
     * null if it couldn't be found.
     */
    public static CapabilityEnvelope find(CapabilityEnvelope ce[], String name)
    {
	CapabilityEnvelope retval = null;
	int lpc;

	Assert.require(ce);
	Assert.require(name);

	for( lpc = 0; (lpc < ce.length) && (retval == null); lpc++ )
	{
	    if( (ce[lpc].name != null) && ce[lpc].name.equals(name) )
	    {
		retval = ce[lpc];
		break;
	    }
	}
	return retval;
    }

    public static void updateHash(KernelObjectStore kos,
				  CapabilityEnvelope ce[],
				  MD5 digest)
    {
	if( ce != null )
	{
	    int lpc;
	    
	    for( lpc = 0; lpc < ce.length; lpc++ )
	    {
		ce[lpc].updateHash(kos, digest);
	    }
	}
    }
    
    private byte seal[];
    private long sealID = KernelObjectStore.INVALID_ID;

    /**
     * The local "name" of the capability.
     */
    private String name;

    /**
     * The capability type.
     */
    private final Class cl;

    /**
     * The capability id.
     */
    private final long id;

    /**
     * The permission set to be passed.
     */
    private PermissionSet ps;

    /**
     * Construct a CapabilityEnvelope from an InputStream.
     *
     * @param kos The KernelObjectStore that holds the capability this envelope
     * references.
     * @param dis The InputStream containing the flattened envelope.
     * @throws ClassNotFoundException if the Capability class couldn't be
     * found.
     * @throws IOException if there is a problem reading from the InputStream.
     */
    CapabilityEnvelope(KernelObjectStore kos, DataInputStream dis)
	throws ClassNotFoundException,
	       IOException
    {
	XdrByteArray xba;
	Domain domain;
	byte hash[];

	domain = Domain.currentDomain();
	if( (this.name = dis.readUTF()).equals("") )
	{
	    this.name = null;
	}
	this.cl = domain.loadClass(dis.readUTF());
	hash = new byte[Xdr.TYPEID];
	dis.readFully(hash);
	this.id = kos.putLate(new TypeID(hash));
	xba = new XdrByteArray(dis.readInt());
	dis.readFully(xba.buf);
	if( xba.buf.length > 0 )
	{
	    this.ps = PermissionSet.instantiate(xba);
	}
    }

    /**
     * Construct a CapabilityEnvelope enclosing the given capability and
     * mapped to the given name.
     *
     * @param name The domain local name this capability should be registered
     * under or null.
     * @param cap The capability to enclose.
     */
    public CapabilityEnvelope(String name, Capability cap)
    {
	Assert.require(cap);

	this.name = name;
	/*
	 * Synchronize to get a consistent snapshot of the capability.
	 *
	 * XXX Is this needed?  I'm mostly worried about a revocation between
	 * getting the id and copying the permissions...
	 */
	synchronized( cap )
	{
	    this.cl = cap.getClass();
	    this.id = cap.getID();
	    if( this.id == Capability.INVALID_ID )
	    {
		throw new IllegalArgumentException("Invalid capability");
	    }
	    /* Clone the current set of permissions */
	    if( cap.getPermissions() != null )
	    {
		this.ps = (PermissionSet)cap.getPermissions().clone();
	    }
	}
    }
    
    /**
     * Construct a CapabilityEnvelope enclosing the given capability.
     *
     * @param cap The capability to enclose.
     */
    public CapabilityEnvelope(Capability cap)
    {
	this(null, cap);
    }

    /**
     * Construct a copy of the given CapabilityEnvelope with a new name.
     *
     * @param name The new envelope name.
     * @param ce The CapabilityEnvelope to copy.
     */
    public CapabilityEnvelope(String name, CapabilityEnvelope ce)
    {
	this.name = name;
	this.cl = ce.cl;
	this.id = ce.id;
	this.ps = (PermissionSet)ce.ps.clone();
    }

    /**
     * Construct a copy of the given CapabilityEnvelope with an anonymous name.
     *
     * @param ce The CapabilityEnvelope to copy.
     */
    public CapabilityEnvelope(CapabilityEnvelope ce)
    {
	this(null, ce);
    }

    /**
     * @return The local name of the capability.
     */
    public String getName()
    {
	return this.name;
    }

    /**
     * Revoke some permissions from the enclosed capability.
     *
     * @param ps The set of permissions to revoke.
     * @return this
     */
    public CapabilityEnvelope revokePermissions(PermissionSet ps)
    {
	Assert.require(ps);

	if( this.ps != null )
	{
	    this.ps.revokePermissions(ps);
	}
	return this;
    }

    /**
     * Bound the enclosed capabilities to the given set.
     *
     * @param ps The set of permissions to bound the permissions by.
     * @return this
     */
    public CapabilityEnvelope boundPermissions(PermissionSet ps)
    {
	Assert.require(ps);

	if( this.ps != null )
	{
	    this.ps.boundPermissions(ps);
	}
	return this;
    }

    /**
     * @return The permission set to be passed.
     */
    PermissionSet getPermissions()
    {
	return this.ps;
    }

    public CapabilityEnvelope seal(Sealer sealer)
    {
	Assert.require(sealer);
	
	this.seal = sealer.getSeal(this.id);
	this.sealID = ((Capability)sealer).getID();
	return this;
    }

    private void checkSeal(Unsealer unsealer)
    {
	if( unsealer != null )
	{
	    if( ((Capability)unsealer).getID() == this.sealID )
	    {
		if( !unsealer.checkSeal(this.seal, this.id) )
		{
		    throw new SecurityException();
		}
	    }
	    else
	    {
		throw new SecurityException();
	    }
	}
	else if( this.sealID != KernelObjectStore.INVALID_ID )
	{
	    throw new SecurityException();
	}
    }

    public KernelObjectCoroutine lookup(Unsealer unsealer, StateResponder sr)
    {
	Assert.require(sr);
	
	try
	{
	    KernelObjectCoroutine retval;

	    this.checkSeal(unsealer);
	    /* XXX Optimize me */
	    retval = new KernelObjectCoroutine(sr);
	    retval.lookup(BDomain.currentBDomain().getBDomainCoroutine(),
			  (Capability)this.cl.newInstance(),
			  this.id);
	    return retval;
	}
	catch(InstantiationException e)
	{
	    throw new Panic(e);
	}
	catch(IllegalAccessException e)
	{
	    throw new Panic(e);
	}
    }

    /**
     * Send a lookup event for the enclosed capability.
     *
     * @param sr The object to use when handling the coroutine state
     * transitions.
     * @return The KernelObjectCoroutine created to handle the messages.
     */
    public KernelObjectCoroutine lookup(StateResponder sr)
    {
	return this.lookup(null, sr);
    }

    /**
     * Unseal the capability returned by a lookup reply.
     *
     * @param koc The coroutine that performed the lookup.
     * @return The referenced capability.
     * @throws MissingCapabilityException if the referenced capability has
     * been deleted.
     */
    public synchronized Capability unseal(KernelObjectCoroutine koc)
	throws MissingCapabilityException
    {
	Capability retval = null;

	Assert.require(koc);
	
	switch( koc.getState() )
	{
	case Coroutine.STATE_COMPLETE:
	    retval = (Capability)koc.getCapability();
	    break;
	default:
	    throw new MissingCapabilityException(this.id);
	}
	return retval;
    }

    public synchronized Capability unseal(Unsealer unsealer)
	throws MissingCapabilityException,
	       InstantiationException
    {
	CapabilityTracker ct = CapabilityTracker.current();
	Capability retval;
	Long id;

	this.checkSeal(unsealer);
	id = new Long(this.id);
	if( (retval = ct.get(id, this.cl)) == null )
	{
	    retval = CapabilityFactory.create(this.cl, this.id, this.ps);
	}
	else if( this.ps != null )
	{
	    CapabilityFactory.grant(retval, this.ps);
	}
	return retval;
    }

    /**
     * Unseal the envelope to get the enclosed capability.
     *
     * @return The enclosed capability.
     * @throws InstantiationException if there was a problem creating the
     * capability.
     * @throws MissingCapabilityException if the referenced capability has
     * been deleted.
     */
    public Capability unseal()
	throws MissingCapabilityException,
	       InstantiationException
    {
	return this.unseal((Unsealer)null);
    }

    void marshal(KernelObjectStore kos, DataOutputStream dos)
	throws IOException
    {
	XdrByteArray xba;

	if( this.name != null )
	{
	    dos.writeUTF(this.name);
	}
	else
	{
	    dos.writeUTF("");
	}
	dos.writeUTF(this.cl.getName());
	dos.write(kos.get(this.id).getHash().getHash());
	if( this.ps != null )
	{
	    xba = new XdrByteArray(this.ps.marshalledLength());
	    this.ps.marshal(xba);
	    dos.writeInt(xba.buf.length);
	    dos.write(xba.buf);
	}
	else
	{
	    dos.writeInt(0);
	}
    }

    public synchronized void updateHash(KernelObjectStore kos, MD5 digest)
    {
	if( this.name != null )
	{
	    digest.update(this.name);
	}
	digest.update(this.cl.getName());
	digest.update(kos.get(this.id).getHash().getHash());
	if( this.ps != null )
	{
	    this.ps.updateHash(digest);
	}
    }
    
    /**
     * @see java.lang.Object#clone()
     */
    public Object clone()
    {
	try
	{
	    Object retval = super.clone();
	    CapabilityEnvelope ce;

	    ce = (CapabilityEnvelope)retval;
	    if( this.seal != null )
	    {
		ce.seal = (byte[])this.seal.clone();
	    }
	    if( this.name != null )
	    {
		ce.name = CloneFactory.clone(this.name);
	    }
	    if( this.ps != null )
	    {
		ce.ps = (PermissionSet)this.ps.clone();
	    }
	    return retval;
	}
	catch(CloneNotSupportedException e)
	{
	    throw new Panic(e);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapabilityEnvelope["
	    + super.toString()
	    + "; seal="
	    + ByteArray.toString(this.seal, 32)
	    + "; sealID="
	    + this.sealID
	    + "; name="
	    + this.name
	    + "; cl="
	    + this.cl
	    + "; id="
	    + this.id
	    + "; ps="
	    + this.ps
	    + "]";
    }
}
