/*
 * CapabilityFactory.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.Node;

import bees.core.kernel.KernelPanic;
import bees.core.kernel.KernelObjectStore;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.PermissionSet;

import bees.core.bdomain.BDomain;

/**
 * Factory class used to create capabilities.
 */
public final class CapabilityFactory
{
    public static CapabilityEnvelope readEnvelope(KernelObjectStore kos,
						  DataInputStream dis)
	throws ClassNotFoundException,
	       IOException
    {
	return new CapabilityEnvelope(kos, dis);
    }
    
    public static void writeEnvelope(KernelObjectStore kos,
				     DataOutputStream dos,
				     CapabilityEnvelope ce)
	throws IOException
    {
	ce.marshal(kos, dos);
    }

    public static long getID(Capability cap)
    {
	return cap.getID();
    }
    
    public static Capability instantiate(Capability cap,
					 long id,
					 byte object[],
					 CapabilityEnvelope sub[],
					 PermissionSet ps)
	throws InstantiationException
    {
	Capability retval;

	Assert.require(cap);
	Assert.require(id >= 0);
	Assert.require(object);
	
	try
	{
	    CapabilityTracker ct;
	    Capability old;

	    if( ps != null )
	    {
		ps = (PermissionSet)ps.clone();
	    }
	    cap.setPermissions(ps);
	    cap.setID(id);
	    cap.fromBytes(object, sub);
	    ct = CapabilityTracker.current();
	    retval = ct.put(new Long(id), cap);
	    if( (retval != cap) && (ps != null) )
	    {
		synchronized( retval )
		{
		    retval.getPermissions().grantPermissions(ps);
		}
	    }
	}
	catch(Throwable e)
	{
	    Node.printStackTrace(e);
	    throw new InstantiationException(e.getMessage());
	}
	return retval;
    }

    public static Capability instantiate(Capability cap,
					 long id,
					 byte object[],
					 PermissionSet ps)
	throws InstantiationException
    {
	return instantiate(cap, id, object, null, ps);
    }
    
    public static PermissionSet getPermissions(Capability cap)
    {
	return cap.getPermissions();
    }
    
    public static PermissionSet getPermissions(CapabilityEnvelope ce)
    {
	return ce.getPermissions();
    }

    public static void grant(Capability cap, PermissionSet ps)
    {
	Assert.require(cap);
	Assert.require(ps);

	synchronized( cap )
	{
	    if( cap.getPermissions() != null )
	    {
		cap.getPermissions().grantPermissions(ps);
	    }
	    else
	    {
		cap.setPermissions(ps);
	    }
	}
    }
    
    public static void create(Capability cap,
			      long id,
			      byte object[],
			      PermissionSet ps)
	throws InstantiationException
    {
	Capability old;

	old = instantiate(cap, id, object, ps);
	
	Assert.ensure(old == cap);
    }
    
    public static void create(Capability cap,
			      long id,
			      byte object[])
	throws InstantiationException
    {
	create(cap, id, object, null);
    }
    
    public static void create(Capability cap, long id, PermissionSet ps)
	throws MissingCapabilityException,
	       InstantiationException
    {
	KernelObjectCoroutine koc;

	koc = new KernelObjectCoroutine();
	koc.lookup(BDomain.currentBDomain().getBDomainCoroutine(), cap, id);
	switch( koc.waitForState(KernelObjectCoroutine.STATE_LOADED) )
	{
	case KernelObjectCoroutine.STATE_LOADED:
	    koc.instantiate(ps);
	    break;
	default:
	    throw new MissingCapabilityException(id);
	}
    }
    
    public static Capability create(Class cl, long id, PermissionSet ps)
	throws InstantiationException,
	       MissingCapabilityException
    {
	Capability retval;

	Assert.require(cl);
	
	try
	{
	    retval = (Capability)cl.newInstance();
	    create(retval, id, ps);
	}
	catch(IllegalAccessException e)
	{
	    throw new KernelPanic("Unable to access constructor of class: "
				  + cl.getName());
	}
	return retval;
    }
    
    private CapabilityFactory()
    {
    }
}
