/*
 * CapabilityStreamer.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import edu.utah.janos.nodeos.Domain;

import bees.core.kernel.Continuation;
import bees.core.kernel.KernelObject;
import bees.core.kernel.KernelObjectStore;
import bees.core.kernel.KernelObjectStreamer;

import bees.core.security.PermissionSet;

/**
 * Helper class used to encode and decode capabilities to data streams.
 *
 * Warning: This can be used to reveal a capabilities private data, it must
 * not be exported to untrusted code.
 */
public final class CapabilityStreamer
{
    /**
     * The current JNodeOS domain.
     */
    private final Domain domain = Domain.currentDomain();

    /**
     * The streamer that will read/write the KernelObject (the capability id
     * and private data).
     */
    private final KernelObjectStreamer streamer = new KernelObjectStreamer();

    /**
     * The KernelObjectStore used to hold KernelObjects.
     */
    private final KernelObjectStore store;

    /**
     * Construct a CapabilityStreamer that can read/write capabilities from
     * the given store.
     *
     * @param store The KernelObjectStore used to hold KernelObjects.
     */
    public CapabilityStreamer(KernelObjectStore store)
    {
	if( store == null )
	{
	    throw new IllegalArgumentException("Null store value");
	}
	this.store = store;
    }

    /**
     * Read a capability from the given input stream.
     *
     * @param is The DataInputStream used to read capabilities.
     * @return The manufactured capability.
     * @throws ClassNotFoundException if the Capability class couldn't be
     * found.
     * @throws InstantiationException if there was a problem instantiating the
     * capability object.
     * @throws IllegalAccessException if there was a problem accessing the
     * capability's constructor method.
     * @throws IOException if there was a problem reading the flattened version
     * of the capability.
     */
    public Object read(DataInputStream is)
	throws ClassNotFoundException,
	       InstantiationException,
	       IllegalAccessException,
	       IOException
    {
	final PermissionSet ps;
	final KernelObject ko;
	final Capability cap;
	Object retval = null;
	XdrByteArray xba;
	String className;
	final long id;
	int length;
	Class cl;

	Assert.require(is);

	className = is.readUTF();
	xba = new XdrByteArray(is.readInt());
	is.readFully(xba.buf);
	ps = PermissionSet.instantiate(xba);
	id = this.streamer.read(this.store, is);
	ko = this.store.get(id);
	cl = this.domain.loadClass(className);
	cap = (Capability)cl.newInstance();
	if( ko.getSubcapabilities() != null )
	{
	    retval = new Continuation() {
		    public Object run()
			throws Throwable
		    {
			return CapabilityFactory.
			    instantiate(cap,
					id,
					ko.getData(),
					ko.getSubcapabilities(),
					ps);
		    }
		};
	}
	else
	{
	    retval = CapabilityFactory.instantiate(cap,
						   id,
						   ko.getData(),
						   null,
						   ps);
	}
	return retval;
    }

    /**
     * Write a capability to the given output stream.
     *
     * @param cap The capability to write to the stream.
     * @param os The DataOutputStream used to write capabilities.
     * @throws IOException if there was a problem reading the flattened version
     * of the capability.
     */
    public void write(Capability cap, DataOutputStream os)
	throws IOException
    {
	XdrByteArray xba;
	KernelObject ko;
	
	Assert.require(cap);
	Assert.require(os);
	
	synchronized( cap )
	{
	    int length;

	    ko = this.store.get(cap.getID());
	    os.writeUTF(cap.getClass().getName());
	    xba = new XdrByteArray(cap.getPermissions().marshalledLength());
	    cap.getPermissions().marshal(xba);
	}
	os.writeInt(xba.buf.length);
	os.write(xba.buf);
	this.streamer.write(this.store, ko, os);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapabilityStreamer[streamer="
	    + this.streamer
	    + "; store="
	    + this.store
	    + "]";
    }
}
