/*
 * CapabilityTracker.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.util.Hashtable;
import java.util.Enumeration;

import bees.core.resource.Logger;

import bees.core.bdomain.BDomainLocal;

/**
 * Class used to track capabilities available to a domain.
 */
final class CapabilityTracker
{
    /**
     * The domain local CapabilityTracker.
     */
    private static final BDomainLocal CAPABILITIES = new BDomainLocal() {
	    protected Object initialValue()
	    {
		return new CapabilityTracker();
	    }
	};

    /**
     * @return The CapabilityTracker for the current domain.
     */
    static CapabilityTracker current()
    {
	return (CapabilityTracker)CAPABILITIES.get();
    }

    /**
     * Two level map of unique identifiers to capability classes and finally
     * the capability.
     */
    private final Hashtable idToCapabilityMap = new Hashtable();

    /**
     * Internal constructor, only used by the BDomainLocal above.
     */
    private CapabilityTracker()
    {
    }

    /**
     * Get a capability.
     *
     * @param id The unique identifier of the capability.
     * @param cl The type of capability.
     * @return The capability or null if it doesn't exist in this domain.
     */
    synchronized Capability get(Long id, Class cl)
    {
	Capability retval = null;
	Hashtable ht;

	if( (ht = (Hashtable)this.idToCapabilityMap.get(id)) != null )
	{
	    retval = (Capability)ht.get(cl);
	}
	return retval;
    }

    /**
     * Get a capability.
     *
     * @param id The unique identifier of the capability.
     * @param cl The type of capability.
     * @return The capability or null if it doesn't exist in this domain.
     */
    synchronized Capability get(long id, Class cl)
    {
	return this.get(new Long(id), cl);
    }

    /**
     * Put a capability into the tracker.
     *
     * @param id The unique identifier of the capability.
     * @param cap The capability to add to the tracker.
     * @return "cap" or a previously added capability.
     */
    synchronized Capability put(Long id, Capability cap)
    {
	Capability retval;
	Hashtable ht;

	if( (ht = (Hashtable)this.idToCapabilityMap.get(id)) == null )
	{
	    ht = new Hashtable();
	    this.idToCapabilityMap.put(id, ht);
	}
	if( (retval = (Capability)ht.get(cap.getClass())) == null )
	{
	    ht.put(cap.getClass(), cap);
	    retval = cap;
	}
	return retval;
    }

    /**
     * Remove a specific type of capability from the tracker.
     *
     * @param id The unique identifier of the capability.
     * @param cl The type of capability.
     * @return The capability or null if it was never added or previously
     * removed.
     */
    synchronized Capability remove(Long id, Class cl)
    {
	Capability retval = null;
	Hashtable ht;

	if( (ht = (Hashtable)this.idToCapabilityMap.get(id)) != null )
	{
	    retval = (Capability)ht.remove(cl);
	    if( ht.size() == 0 )
	    {
		this.idToCapabilityMap.remove(id);
	    }
	}
	return retval;
    }

    /**
     * Remove a specific type of capability from the tracker.
     *
     * @param id The unique identifier of the capability.
     * @param cl The type of capability.
     * @return The capability or null if it was never added or previously
     * removed.
     */
    Capability remove(long id, Class cl)
    {
	return this.remove(new Long(id), cl);
    }

    /**
     * Remove a capability from the tracker.
     *
     * @param id The unique identifier of the capability.
     * @return An enumeration containing all types of the removed capability or
     * null if it was never added or previously removed.
     */
    Enumeration remove(long id)
    {
	Enumeration retval = null;
	Hashtable ht;

	if( (ht = (Hashtable)this.idToCapabilityMap.remove(new Long(id)))
	    != null )
	{
	    retval = ht.elements();
	}
	return retval;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapabilityTracker["
	    + super.toString()
	    + "; idToCapabilityMap="
	    // + this.idToCapabilityMap
	    + "]";
    }
}
