/*
 * Capsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import bees.core.kernel.KernelPanic;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionException;

/**
 * Capsule defines the base for packets that can be sent or received by a node.
 * Subclasses are expected to implement the encode() and decode() methods to
 * marshal and unmarshal object state, and to implement the evaluate() method
 * which handles the arrival of a new Capsule at a node.  However, they are not
 * expected to create a unique capsule identifier and transfer their code
 * between nodes, the system will perform these functions.
 * <p>
 * Because of the need to restrict the propagation of Capsules in the system
 * and network, each capsule object is associated with an authorization state.
 * This state tracks the number of times the capsule can be used in some
 * operation, such as sending the capsule to a neighbor.  After a capsule has
 * been used to perform these operations it is considered "depleted" and needs
 * to be "replenished" before it can be used again.  Capsules can be
 * replenished through one of the following methods:
 *
 * <ul>
 * <li>Using fromCapsule() with a usable Capsule object, for example, the boot
 * capsule or one received from a neighbor.
 * <li>Using fromExternal() with an ExternalCapsule received from an
 * application.
 * <li>Using specializeCapsule() on a companion capsule with the capsule it
 * should be specialized to.
 * <li>Adding the Capsule to a NodeCache, when the mapping is collected, the
 * object will be replenished.  Useful for periodic replenishment.
 * </ul>
 *
 * The set of operations that a capsule is actually allowed to perform is
 * specified with a PermissionSet when the Protocol is being built.
 * @see Protocol#addCapsule(String, PermissionSet)
 *
 * @see Protocol
 */
public abstract class Capsule
    extends ANEPPacket
{
    /**
     * The pool this capsule was pulled from.  Only set if the capsule was
     * received from a neighbor.
     */
    private CapsulePool cp;

    /**
     * Holds the FLAG_* bits.
     */
    private int flags;

    /**
     * The type of this capsule.
     */
    private CapsuleType ct;

    private InternalPacketAuthorization ipa;

    /**
     * The specializer identifier for this capsule.
     */
    private TypeID specializer;

    /**
     * Create a Capsule with the default values.
     */
    public Capsule()
    {
	super.setType(BeesVersion.ANEP_TYPE);
	super.setHeaderLength(ANEPPacket.NO_OPTION_HEADER_LENGTH);
    }

    /**
     * @param cp The capsule pool this capsule is a part of.
     * @return this
     */
    Capsule setPool(CapsulePool cp)
    {
	this.cp = cp;
	this.ct = cp.getCapsuleType();
	this.setPacketAuthorization(new PacketAuthorization(
		this.ct.getCodeType().getPermissions()));
	this.ipa = (InternalPacketAuthorization)
	    this.ct.getInternalPacketAuthorization().clone();
	return this;
    }

    Capsule setInternalPacketAuthorization(InternalPacketAuthorization ipa)
    {
	this.ipa = ipa;
	return this;
    }

    /**
     * @return The type of this capsule.
     */
    CapsuleType getCapsuleType()
    {
	return this.ct;
    }

    /**
     * @return The session type for this capsule.
     */
    SessionType getSessionType()
    {
	return SessionType.getBDomainSession(this.ct.getFlowType().getFlowID(),
					     this.ct.getProtocolType().
					     getProtocolID());
    }

    /**
     * @return The unique identifier for this type of capsule.
     */
    public final TypeID getCapsuleID()
    {
	Assert.require(this.ct);
	
	return this.ct.getCapsuleID();
    }

    /**
     * @return A copy of the PairChain describing the flow this capsule belongs
     * too.
     */
    public final PairChain getFlowDesc()
    {
	return (PairChain)this.ct.getFlowType().getPairChain().clone();
    }

    /**
     * Get the ProtocolSession for the given class name with the context of
     * this Capsule's flow.
     *
     * @param companionSessionName The class name of the session to retrieve.
     * @return The ProtocolSession object that matches the given name and is
     * within the context of this Capsule's flow.
     * @throws ClassNotFoundException if the given class name couldn't be
     * found.
     * @throws IllegalStateException if the capsule has yet to be associated
     * with protocol session.
     */
    public final ProtocolSession getSession(String companionSessionName)
	throws ClassNotFoundException
    {
	ProtocolType pt;

	if( this.ct == null )
	{
	    throw new IllegalStateException(
		"Capsule not yet associated with a protocol session");
	}
	pt = this.ct.getProtocolType().getCompanion(companionSessionName);
	if( pt == null )
	{
	    throw new ClassNotFoundException(companionSessionName);
	}
	try
	{
	    ProtocolSession retval;
	    SessionType st;

	    st = SessionType.getBDomainSession(this.ct.getFlowType().
					       getFlowID(),
					       pt.getProtocolID());
	    retval = st.getSession();

	    Assert.ensure(retval);

	    return retval;
	}
	catch(InstantiationException e)
	{
	    throw new Panic(e);
	}
	catch(IllegalAccessException e)
	{
	    throw new Panic(e);
	}
	catch(ClassNotFoundException e)
	{
	    throw new Panic(e);
	}
    }

    /**
     * @return The ProtocolSession linked to this flow.
     */
    public final ProtocolSession getSession()
    {
	try
	{
	    ProtocolSession retval;
	    SessionType st;
	    
	    st = this.getSessionType();
	    retval = st.getSession();

	    Assert.ensure(retval);

	    return retval;
	}
	catch(InstantiationException e)
	{
	    throw new Panic(e);
	}
	catch(IllegalAccessException e)
	{
	    throw new Panic(e);
	}
	catch(ClassNotFoundException e)
	{
	    throw new Panic(e);
	}
    }

    /**
     * @param flags Set the FLAG_* bits.
     * @return this
     */
    Capsule setBeesFlags(int flags)
    {
	this.flags = flags;
	return this;
    }

    /**
     * @return The FLAG_* bits.
     */
    int getBeesFlags()
    {
	return this.flags;
    }

    /**
     * @param obj The specializer identifier for this capsule.
     * @return this
     */
    Capsule setSpecializer(TypeID obj)
    {
	this.specializer = obj;
	return this;
    }

    /**
     * @return The specializer identifier for this capsule.
     */
    TypeID getSpecializer()
    {
	return this.specializer;
    }

    final void fromFlowType(FlowType ft, TypeID specializer)
    {
	if( (this.ct = CapsuleType.getBDomainCapsuleType(ft.getFlowID(),
						      this.getClass(),
						      specializer)) != null )
	{
	    if( this.getPacketAuthorization() == null )
	    {
		this.setPacketAuthorization(new PacketAuthorization(
			this.ct.getCodeType().getPermissions()));
	    }
	    if( this.ipa == null )
	    {
		this.ipa = (InternalPacketAuthorization)
		    this.ct.getInternalPacketAuthorization().clone();
	    }
	    this.ipa.replenish();
	    this.specializer = this.ct.getSpecializedCapsuleID();
	    
	    super.fromExternal();
	}
	else
	{
	    if( (specializer == CapsuleType.NULL_SPECIALIZER) ||
		((this.ct = CapsuleType.getBDomainCapsuleType(
			ft.getFlowID(),
			this.getClass(),
			CapsuleType.NULL_SPECIALIZER))
		 == null) )
	    {
		throw new KernelPanic("");
	    }
	    else
	    {
		if( this.getPacketAuthorization() == null )
		{
		    this.setPacketAuthorization(new PacketAuthorization(
			this.ct.getCodeType().getPermissions()));
		}
		if( this.ipa == null )
		{
		    this.ipa = (InternalPacketAuthorization)
			this.ct.getInternalPacketAuthorization().clone();
		}
		this.ipa.replenish();
		this.specializer = specializer;

		super.fromExternal();
	    }
	}
    }

    /**
     * Attach an authorization to this capsule from the given external capsule.
     *
     * @param ec The external capsule to use as authorization.
     * @param specializer The specializer for this capsule.
     */
    final void fromExternal(ExternalCapsule ec, TypeID specializer)
    {
	if( ec.getSource() == null )
	{
	    throw new SecurityException();
	}
	ec.consume();
	this.fromFlowType(ec.getSessionType().getFlowType(), specializer);
    }
    
    /**
     * Replenish this capsule from the authorization in the given external
     * capsule.
     *
     * @param ec The external capsule to use as authorization.
     */
    public final void fromExternal(ExternalCapsule ec)
    {
	this.fromExternal(ec, CapsuleType.NULL_SPECIALIZER);
    }

    /**
     * Replenish this capsule based on the receipt of a packet from the given
     * neighbor.
     *
     * @param neighbor The source of the capsule.
     */
    final void fromNeighbor(Neighbor neighbor)
    {
	// Assert.require(this.ct);

	this.ipa.replenish();
	super.fromNeighbor(neighbor);
    }

    /**
     * Replenish this capsule from the authorization in the given capsule.
     *
     * @param cap The capsule to use as authorization.
     * @param specializer The specialized identifier for this capsule.
     */
    final synchronized void fromCapsule(Capsule cap, TypeID specializer)
    {
	FlowType ft;

	cap.ipa.consume(this.getClass());
	this.ipa = cap.ipa;
	ft = cap.ct.getFlowType();
	if( (this.ct = CapsuleType.getBDomainCapsuleType(ft.getFlowID(),
						      this.getClass(),
						      specializer)) != null )
	{
	    if( this.getPacketAuthorization() == null )
	    {
		this.setPacketAuthorization(new PacketAuthorization(
			this.ct.getCodeType().getPermissions()));
	    }
	    this.specializer = this.ct.getSpecializedCapsuleID();

	    super.fromPacket(cap);
	}
	else
	{
	    if( (specializer == CapsuleType.NULL_SPECIALIZER) ||
		((this.ct = CapsuleType.getBDomainCapsuleType(
			ft.getFlowID(),
			this.getClass(),
			CapsuleType.NULL_SPECIALIZER))
		 == null) )
	    {
		throw new KernelPanic("");
	    }
	    else
	    {
		if( this.getPacketAuthorization() == null )
		{
		    this.setPacketAuthorization(new PacketAuthorization(
			this.ct.getCodeType().getPermissions()));
		}
		this.specializer = specializer;

		super.fromPacket(cap);
	    }
	}
    }

    /**
     * Replenish this capsule from the authorization in the given capsule.
     *
     * @param cap The capsule to use as authorization.
     */
    public final void fromCapsule(Capsule cap)
    {
	this.fromCapsule(cap, CapsuleType.NULL_SPECIALIZER);
    }

    /**
     * Replenish this companion capsule and specialize it from the given
     * Capsule.
     *
     * @param cap The capsule this one should be specialized to.
     */
    public final void specializeCapsule(Capsule cap)
    {
	this.fromCapsule(cap, cap.ct.getCapsuleID());
    }

    final void anepDecode(Xdr xdr)
	throws InvalidPacketException, DroppedPacketException
    {
	super.internalDecode(xdr);
	if( (this.specializer = this.ct.getSpecializedCapsuleID()) != null )
	{
	    xdr.SKIP(Xdr.TYPEID);
	}
    }

    final void anepEncode(Xdr xdr)
    {
	super.internalEncode(xdr);
    }

    protected final void internalDecode(Xdr xdr)
	throws InvalidPacketException, DroppedPacketException
    {
	ProtocolType pt;
	
	super.internalDecode(xdr);
	this.skipOptions(xdr);
	xdr.SKIP(Xdr.INT);	// Version
	this.flags = xdr.INT();
	xdr.SKIP(Xdr.TYPEID);	// CapsuleID
	this.specializer = this.ct.getSpecializedCapsuleID();
	if( (pt = this.ct.getProtocolType()) != null )
	{
	    switch( pt.getCompanionType() )
	    {
	    case ProtocolType.COMPANION_BYTEARRAY:
	    case ProtocolType.COMPANION_FLOW:
	    case ProtocolType.COMPANION_CAPSULE:
		if( this.specializer == null )
		{
		    this.specializer = xdr.TYPEID();
		}
		else
		{
		    xdr.SKIP(Xdr.TYPEID);
		}
		break;
	    }
	}
    }

    protected final void internalEncode(Xdr xdr)
    {
	super.internalEncode(xdr);
//  	this.encodeIPv4AddressOption(xdr,
//  				     ANEPPacket.OPTION_TYPE_SOURCE,
//  				     0);
//  	this.encodeIPv4AddressOption(xdr,
//  				     ANEPPacket.OPTION_TYPE_DESTINATION,
//  				     0);
	xdr.PUT(BeesVersion.CAPSULE_VERSION);
	xdr.PUT(this.flags);
	xdr.PUT(this.ct.getCapsuleID());
	if( this.specializer != null )
	{
	    xdr.PUT(this.specializer);
	}
	else
	{
	    switch( this.ct.getProtocolType().getCompanionType() )
	    {
	    case ProtocolType.COMPANION_BYTEARRAY:
	    case ProtocolType.COMPANION_FLOW:
	    case ProtocolType.COMPANION_CAPSULE:
		xdr.PUT(CapsuleType.NULL_SPECIALIZER);
		break;
	    }
	}
    }

    /**
     * Reset this Capsule's state.  This method will be called when this object
     * is added back into a pool via addToPool().
     */
    public void reset()
    {
    }

    public void decodingException(Throwable th)
    {
	Logger.log(th);
    }

    /**
     * Decode this Capsule's state from the given Xdr.
     *
     * @param xdr An Xdr object that contains this Capsule's data.
     * @throws Throwable if anything goes wrong.
     */
    public abstract void decode(Xdr xdr)
	throws Throwable;

    /**
     * Encode this Capsule's state into the given Xdr.
     *
     * @param xdr An Xdr object that should be used to encode this Capsule's
     * state.
     * @throws Throwable if anything goes wrong.
     */
    public abstract void encode(Xdr xdr)
	throws Throwable;

    /**
     * Evaluate this capsule.  Basically, this is the function that performs
     * any computation and forwarding.
     *
     * @param node The current Node object.
     * @throws Throwable if anything goes wrong.
     */
    public void evaluate(Node node)
	throws Throwable
    {
    }

    /**
     * Evaluate this capsule.  Basically, this is the function that performs
     * any computation and forwarding.
     *
     * @param node The current Node object.
     * @param ps The ProtocolSession bound to this capsule type.
     * @throws Throwable if anything goes wrong.
     */
    public void evaluate(Node node, ProtocolSession ps)
	throws Throwable
    {
	this.evaluate(node);
    }

    /**
     * @see bees.core.security.PermissionSet#hasPermission(Permission)
     */
    private final Permission hasPermission(Permission what)
    {
	Permission retval = null;
	
	if( this.ct != null )
	{
	    retval = this.ct.getCodeType().getPermissions().
		hasPermission(what);
	}
	return retval;
    }

    /**
     * Check the capsule for the given permission.
     * 
     * @param what The Permission to check for.
     * @throws PermissionException if the Capsule doesn't have the given
     * Permission.
     */
    public final void checkPermission(Permission what)
	throws PermissionException
    {
	if( this.hasPermission(what) == null )
	{
	    throw new PermissionException("Invalid access:  Capsule, "
					  + this
					  + ", doesn't have permission, "
					  + what.toString());
	}
    }

    /**
     * Add this capsule back to the flow local pool.
     */
    public void addToPool()
    {
	if( this.cp != null )
	{
	    this.cp.releaseCapsule(this);
	}
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( (obj instanceof Capsule) && super.equals(obj) )
	{
	    Capsule cap = (Capsule)obj;

	    if( this.flags == cap.flags )
	    {
		retval = true;
	    }
	}
	return retval;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Capsule["
	    + super.toString()
	    + "; cp="
	    + this.cp
	    + "; flags=0x"
	    + Integer.toHexString(this.flags)
	    + "; ct="
	    + this.ct
	    + "; specializer="
	    + this.specializer
	    + "]";
    }
}
