/*
 * CapsulePool.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.util.Hashtable;

import bees.core.bdomain.BDomainLocal;

import bees.core.resource.Logger;

/**
 * A CapsulePool maintains a pool of Capsule objects that a CapsuleReceiver can
 * pull from.  Thus, the system need not allocate new Capsule objects for every
 * capsule received.
 */
final class CapsulePool
{
    /**
     * The maximum size of the pool.
     */
    private static final int MAX_CACHE_SIZE = 5;

    /**
     * The set of pools for the current BDomain.
     */
    private static final BDomainLocal BDOMAIN_POOLS = new BDomainLocal() {
	    protected Object initialValue()
	    {
		return new Hashtable();
	    }
	};

    /**
     * @return The Hashtable containing the mapping of CapsuleTypes to
     * CapsulePools.
     */
    static Hashtable getBDomainCapsulePools()
    {
	return (Hashtable)BDOMAIN_POOLS.get();
    }

    /**
     * @param ct The CapsuleType whose pool should be retrieved/created.
     * @return A CapsulePool object that corresponds to the given type.
     */
    static CapsulePool getBDomainCapsulePool(CapsuleType ct)
    {
	Hashtable ht = (Hashtable)BDOMAIN_POOLS.get();
	CapsulePool retval;
	
	synchronized( ht )
	{
	    if( (retval = (CapsulePool)ht.get(ct)) == null )
	    {
		retval = new CapsulePool(ct);
		ht.put(ct, retval);
	    }
	}
	return retval;
    }

    /**
     * @param ct The CapsuleType whose pool should be created and added to the
     * current BDomain.
     */
    static void addBDomainCapsulePool(CapsuleType ct)
    {
	Hashtable ht = (Hashtable)BDOMAIN_POOLS.get();
	
	synchronized( ht )
	{
	    if( !ht.containsKey(ct) )
	    {
		ht.put(ct, new CapsulePool(ct));
	    }
	}
    }

    /**
     * The type of Capsules this pool manages.
     */
    private final CapsuleType ct;

    /**
     * The pool of capsules.
     */
    private final Capsule cache[] = new Capsule[MAX_CACHE_SIZE];

    /**
     * The actual number of capsules available in the pool.
     */
    private int cacheSize = 0;

    /**
     * Construct a CapsulePool for the given type.
     *
     * @param ct The type of Capsules this pool should manage.
     */
    private CapsulePool(CapsuleType ct)
    {
	this.ct = ct;
    }

    /**
     * @return The type of Capsules this pool manages.
     */
    CapsuleType getCapsuleType()
    {
	return this.ct;
    }

    /**
     * @return A capsule from the pool or a freshly created one.
     * @throws InstantiationException if there is a problem creating the
     * Capsule object.
     * @throws IllegalAccessException if the Capsule's constructor is not
     * accessible.
     */
    synchronized Capsule obtainCapsule()
	throws InstantiationException,
	       IllegalAccessException
    {
	Capsule retval = null;

	if( this.cacheSize == 0 )
	{
	    retval = (Capsule)this.ct.getCodeType().getCodeClass().
		newInstance();
	    retval.setPool(this);
	}
	else
	{
	    this.cacheSize -= 1;
	    retval = this.cache[this.cacheSize];
	}
	return retval;
    }

    /**
     * @param capsule The capsule to put back in the pool.  If the pool is
     * currently full, the capsule reference is just dropped.
     */
    synchronized void releaseCapsule(Capsule capsule)
    {
	if( this.cacheSize < MAX_CACHE_SIZE )
	{
	    capsule.internalReset();
	    capsule.reset();
	    this.cache[this.cacheSize] = capsule;
	    this.cacheSize += 1;
	}
    }

    /**
     * @return A concise description of this CapsulePool object.
     */
    public String toShortString()
    {
	return "CapsulePool[ct="
	    + this.ct.getCodeType().getByteArrayName()
	    + "]";
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapsulePool[ct="
	    + this.ct
	    + "; cache="
	    + this.cache
	    + "; cacheSize="
	    + this.cacheSize
	    + "]";
    }
}
