/*
 * CapsuleReceiver.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.BufferHandle;
import edu.utah.janos.nodeos.DemultiplexKey;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.CountPermission;

import bees.core.security.Mac;
import bees.core.security.SecretKey;

/**
 * Class used to receive Capsules from the NeighborImpl and dispatch to the
 * protocol.
 */
public class CapsuleReceiver
{
    /**
     * Permission type for the maximum thread count allowed for a Capsule.
     */
    static final Permission THREAD_COUNT_PERMISSION =
	new Permission("threadCount", "execute");
    
    /**
     * Permission type for the maximum buffer count allowed for a Capsule.
     */
    static final Permission BUFFER_COUNT_PERMISSION =
	new Permission("bufferCount", "execute");

    /**
     * Default thread count for Capsules.
     */
    static int DEFAULT_THREAD_COUNT = 3;

    /**
     * Default buffer count for Capsules.
     */
    static int DEFAULT_BUFFER_COUNT = 3;

    /**
     * The current Node object for this BDomain.
     */
    private final Node node = Node.currentNode();

    /**
     * The domain local object representing the Neighbor.
     */
    private final Neighbor neighbor;

    /**
     * The Xdr object used to decode the Capsule headers.
     */
    private final Xdr internalXdr = new Xdr();

    /**
     * The Xdr object used to decode the user part of the Capsule.  A separate
     * Xdr object is used to prevent the user from tampering with the decoding
     * of header fields.
     */
    private final Xdr userXdr = new Xdr();

    /**
     * The CapsulePool for this Capsule type.
     */
    private final CapsulePool cp;

    /**
     * The ProtocolSession that goes with the capsule type received by this
     * object.
     */
    private final ProtocolSession ps;

    /**
     * Construct a CapsuleReceiver with the given values.
     *
     * @param neighbor The domain local object representing the Neighbor.
     * @param cp The CapsulePool for this Capsule type.
     * @param ps The ProtocolSession that goes with the capsule type received
     * by this object.
     */
    CapsuleReceiver(Neighbor neighbor, CapsulePool cp, ProtocolSession ps)
    {
	Assert.require(neighbor);
	
	this.neighbor = neighbor;
	this.cp = cp;
	this.ps = ps;
    }

    /**
     * @return The domain local object representing the Neighbor.
     */
    public Neighbor getNeighbor()
    {
	return this.neighbor;
    }

    /**
     * @return The CapsulePool for this Capsule type.
     */
    CapsulePool getPool()
    {
	return this.cp;
    }

    /**
     * @return The class name of the Capsule type received by this object.
     */
    public String getCapsuleName()
    {
	return this.cp.getCapsuleType().getCodeType().getByteArrayName();
    }

    /**
     * @return The unique identifier for this Capsule type.
     */
    public TypeID getCapsuleID()
    {
	return this.cp.getCapsuleType().getCapsuleID();
    }

    /**
     * @return The unique specialized identifier for this companion Capsule
     * type.
     */
    public TypeID getSpecializedCapsuleID()
    {
	return this.cp.getCapsuleType().getSpecializedCapsuleID();
    }

    /**
     * Helper function used to get a CountPermission coupled with a Capsule.
     *
     * @param what The type of permission to get.
     * @param defaultValue The default value if no CountPermission is found.
     * @return The value in the CountPermission if it was found or the
     * default value.
     */
    private int getCountPermission(Permission what, int defaultValue)
    {
	int retval = defaultValue;
	CountPermission perm;
	PermissionSet ps;
	CapsuleType ct;
	
	ct = this.cp.getCapsuleType();
	ps = ct.getCodeType().getPermissions();
	if( (perm = (CountPermission)ps.hasPermission(what)) != null )
	{
	    retval = perm.getMaxCount();
	}
	return retval;
    }

    /**
     * @return The maximum thread count for Capsules.
     */
    public int getThreadCount()
    {
	return this.getCountPermission(THREAD_COUNT_PERMISSION,
				       DEFAULT_THREAD_COUNT);
    }

    /**
     * @return The maximum buffer count for Capsules.
     */
    public int getBufferCount()
    {
	return this.getCountPermission(BUFFER_COUNT_PERMISSION,
				       DEFAULT_BUFFER_COUNT);
    }

    /**
     * Create a DemultiplexKey that matches the enclosed Capsule type.
     *
     * @return The manufactured DemultiplexKey.
     */
    public DemultiplexKey createDemultiplexKey()
    {
	Domain domain = Domain.currentDomain();
	DemultiplexKey retval;
	CapsuleType ct;
	TypeID spec;
	
	ct = this.cp.getCapsuleType();
	retval = new BeesDemultiplexKey(domain, ct.getCapsuleID());
	spec = ct.getSpecializedCapsuleID();
	if( spec != null )
	{
	    byte bytes[];
	    
	    bytes = spec.getHash();
	    retval.add(bytes.length, bytes, null);
	}
	return retval;
    }

    /**
     * Receive a Capsule from the NeighborImpl.
     *
     * @param bh A BufferHandle holding an encoded Capsule.
     * @param offset The offset into "bh" where the payload begins.
     * @return The Capsule decoded capsule.
     */
    public synchronized Capsule receive(BufferHandle bh, int offset)
    {
	Capsule retval;
	
	this.node.used();

	try
	{
	    retval = (Capsule)this.cp.obtainCapsule();
	    try
	    {
		SecretKey sk;
		Mac mac;
		int len;
		
		retval.fromNeighbor(this.neighbor);
		/* Reset the Xdr with the new buffer. */
		this.internalXdr.readReset(bh);
		/* Decode the system headers. */
		retval.internalDecode(this.internalXdr);
		len = bh.getLength();
		if( len != retval.getPacketLength() )
		{
		    throw new InvalidPacketException(
			"ANEP packet length ("
			+ retval.getPacketLength()
			+ ") doesn't match buffer length("
			+ bh.getLength()
			+ ").");
		}
		/* Decode the user payload. */
		this.userXdr.readReset(bh, this.internalXdr.getReadCursor());
		retval.decode(this.userXdr);
		if( this.userXdr.getReadCursor() != len )
		{
		    throw new InvalidPacketException(
			"Packet data wasn't completely decoded: "
			+ retval
			+ "; "
			+ this.userXdr);
		}
		this.userXdr.reset();
	    }
	    catch(Throwable th)
	    {
		retval.decodingException(th);
		retval = null;
	    }
	}
	catch(Throwable th)
	{
	    Logger.log("Error obtaining capsule from: "
		       + this.cp.toShortString());
	    Logger.log(th);
	    retval = null;
	}
	return retval;
    }

    /**
     * Evaluate the capsule.
     *
     * @param cap The capsule to evaluate.
     */
    public void evaluate(Capsule cap)
    {
	try
	{
	    this.node.used();
	    cap.evaluate(this.node, this.ps);
	}
	catch(Throwable th)
	{
	    Logger.log(th);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapsuleReceiver[node="
	    + this.node.toShortString()
	    + "; neighbor="
	    // XXX + this.neighbor
	    + "; internalXdr="
	    + this.internalXdr
	    + "; userXdr="
	    + this.userXdr
	    + "; cp="
	    + this.cp.toShortString()
	    + "]";
    }
}
