/*
 * CodeType.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.lang.reflect.Modifier;

import java.util.Hashtable;

import edu.utah.janos.nodeos.Domain;

import bees.core.resource.Logger;

import bees.core.security.VersionString;
import bees.core.security.PermissionSet;

import bees.util.MD5;

/**
 * CodeType is a ByteArrayType that encodes Java byte code.
 */
final class CodeType
    extends ByteArrayType
{
    /**
     * The class object.
     */
    private Class cl;

    /**
     * If a capsule class, the permissions for propagation within the node.
     */
    private PermissionSet permissions = new PermissionSet();

    /**
     * Construct an empty CodeType object.
     */
    CodeType()
    {
    }
	
    /**
     * This will load and cache the class object.
     *
     * @return The class object.
     */
    synchronized Class getCodeClass()
    {
	Class retval;
	
	if( this.cl == null )
	{
	    try
	    {
		Hashtable ht = getBDomainByteArray();
		Domain domain = Domain.currentDomain();

		this.cl = domain.loadClass(super.getByteArrayName());
		if( super.getState() != STATE_BUILTIN )
		{
		    super.setState(STATE_BOUND);
		}
	    }
	    catch(ClassNotFoundException e)
	    {
		Logger.log(e);
		throw new Panic("Unable to load class, "
				+ super.getByteArrayName()
				+ ", which should have been added to "
				+ "the class loader.");
	    }
	}
	retval = this.cl;
	return retval;
    }

    /**
     * Test if this ByteArrayType's class is a certain type.
     *
     * @param cl The type to check against.
     * @return True if this ByteArrayType is a form of "cl".
     */
    boolean isA(Class cl)
    {
	return cl.isAssignableFrom(this.getCodeClass());
    }

    /**
     * @return True if the ByteArrayType's class is final.
     */
    boolean isFinal()
    {
	return Modifier.isFinal(this.getCodeClass().getModifiers());
    }

    /**
     * @param ps If a capsule class, the permissions for propagation within the
     * node.
     * @return this
     */
    ByteArrayType setPermissions(PermissionSet ps)
    {
	this.permissions = ps;
	return this;
    }

    /**
     * @return If a capsule class, the permissions for propagation within the
     * node.
     */
    PermissionSet getPermissions()
    {
	return this.permissions;
    }
    
    /**
     * @see bees.core.ByteArrayType#extraDigest(MD5)
     */
    void extraDigest(MD5 digest)
    {
	if( this.permissions != null )
	{
	    digest.update(this.permissions.toString());
	}
    }

    /**
     * Import a system class.
     *
     * @param cl The class to import.
     * @param versionID ...
     * @return this
     */
    synchronized CodeType importClass(Class cl, VersionString versionID)
    {
	Hashtable ht = getBDomainByteArray();
	String version;
	
	Assert.require(cl);
	Assert.require(super.getState() == STATE_NEW);
	Assert.require(super.getByteArrayName().equals(cl.getName()));

	this.cl = cl;
	ht.put(this.cl, this);
	if( versionID == null )
	{
	    version = "Unknown";
	}
	else
	{
	    version = versionID.getValue();
	}
	super.setByteArrayID(new TypeID(MD5.hash(super.getByteArrayName()
						 + " - "
						 + version)));
	super.setState(STATE_BUILTIN);
	return this;
    }
    
    /**
     * @param byteCode The class' byte code.
     * @param versionID The version string that should be used for
     * manufacturing the unique code identifier or null if the byte code
     * should be used.
     * @return this
     */
    synchronized CodeType importClass(byte byteCode[], VersionString versionID)
    {
    	if( super.getByteArray() == null )
    	{
	    Domain domain = Domain.currentDomain();
	    
    	    super.importByteArray(byteCode, versionID);
	    // XXX Move this
	    domain.addClass(super.getByteArrayName(), byteCode);
	    super.setState(STATE_LOADED);
	}
	return this;
    }

    /**
     * @param codeType The class' byte code.
     * @return this
     */
    CodeType importClass(byte codeType[])
    {
	return this.importClass(codeType, null);
    }
    
    /**
     * @see bees.core.ByteArrayType#marshalledLength()
     */
    int marshalledLength()
    {
    	return (super.marshalledLength()
    		+ this.permissions.marshalledLength());
    }

    /**
     * @see bees.core.ByteArrayType#marshal(XdrByteArray)
     */
    void marshal(XdrByteArray xba)
    {
	super.marshal(xba);
	this.permissions.marshal(xba);
    }
	
    /**
     * @see bees.core.ByteArrayType#unmarshal(XdrByteArray)
     */
    void unmarshal(XdrByteArray xba)
    {
	super.unmarshal(xba);
		
	this.permissions = PermissionSet.instantiate(xba);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CodeType["
	    + super.toString()
	    + "; cl="
	    + this.cl
	    + "; permissions="
	    + this.permissions
	    + "]";
    }
}
