/*
 * Coroutine.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.util.Enumeration;

import edu.utah.janosvm.kit.util.SparseArray;
import edu.utah.janosvm.kit.util.SparseArrayElement;
import edu.utah.janosvm.kit.util.ObjectSparseArrayElement;

import bees.core.kernel.KernelPanic;

import bees.core.bdomain.BDomain;
import bees.core.bdomain.BDomainLocal;
import bees.core.bdomain.EventHandler;

/**
 * A Coroutine is a base class for handling events exchanged between domains.
 */
public abstract class Coroutine
{
    /**
     * An invalid coroutine identifier.
     */
    public static final long INVALID_ID = SparseArray.INVALID_ID;

    /**
     * The coroutines local to this domain.
     */
    private static final BDomainLocal BDOMAIN_COROUTINES = new BDomainLocal() {
	    protected Object initialValue()
	    {
		BDomain ad = BDomain.currentBDomain();
		final SparseArray retval;
		
		retval = new SparseArray();
		ad.pushCleanupHandler(new Runnable() {
			public void run()
			{
			    synchronized( retval )
			    {
				Enumeration enum;

				enum = retval.elements();
				while( enum.hasMoreElements() )
				{
				    ObjectSparseArrayElement osae;
				    Coroutine coroutine;

				    osae = (ObjectSparseArrayElement)
					enum.nextElement();
				    coroutine = (Coroutine)osae.getValue();
				    coroutine.cleanup();
				}
			    }
			}
		    });
		return retval;
	    }
	};

    /**
     * @param id The coroutine id within this domain to get.
     * @return The matching Coroutine object or NULL if it couldn't be found.
     */
    static Coroutine getCoroutine(long id)
    {
	SparseArray sa = (SparseArray)BDOMAIN_COROUTINES.get();
	ObjectSparseArrayElement elem;
	Coroutine retval = null;

	Assert.require(id >= 0);

	synchronized( sa )
	{
	    if( (elem = (ObjectSparseArrayElement)sa.findElement(id)) != null )
	    {
		retval = (Coroutine)elem.getValue();
	    }
	}
	return retval;
    }

    /**
     * @param ae The BeesEvent that should be matched to a coroutine.
     * @return The Coroutine matching the given event or a newly created
     * Coroutine object if one doesn't already exist.
     */
    public static Coroutine getCoroutine(BeesEvent ae)
    {
	Coroutine retval = null;

	Assert.require(ae);
	
	if( !ae.isReply() )
	{
	    long id;

	    id = ae.getServerCoroutineID();
	    if( id == INVALID_ID )
	    {
		try
		{
		    Assert.require(ae.getCoroutineClass());
		    
		    retval = (Coroutine)ae.getCoroutineClass().newInstance();
		    ae.setServerCoroutineID(retval.id);
		}
		catch(InstantiationException e)
		{
		    throw new KernelPanic(e);
		}
		catch(IllegalAccessException e)
		{
		    throw new KernelPanic(e);
		}
	    }
	    else
	    {
		retval = getCoroutine(id);
	    }
	}
	else
	{
	    retval = getCoroutine(ae.getClientCoroutineID());
	}
	return retval;
    }

    public static final int STATE_ERROR_USER		= -1000;
    public static final int STATE_ERROR_TIMEOUT		= -4;
    public static final int STATE_ERROR_CAGED		= -3;
    public static final int STATE_ERROR_INTERNAL	= -2;
    public static final int STATE_ERROR			= -1;
    public static final int STATE_ANY			= 0;
    public static final int STATE_NEWBORN		= 1;
    public static final int STATE_USER			= 1000;
    public static final int STATE_COMPLETE		= Integer.MAX_VALUE;

    /**
     * Default timeout for events sent between coroutines.
     */
    public static final long DEFAULT_TIMEOUT = 5 * 1000;

    /**
     * The Coroutine's unique identifier.
     */
    private long id = INVALID_ID;

    /**
     * The current state.
     */
    private int state = STATE_NEWBORN;

    /**
     * An exception that disabled the Coroutine.
     */
    private Throwable throwable;

    /**
     * Construct a coroutine and initialize it with a unique identifier.
     */
    protected Coroutine()
    {
	SparseArray sa;

	sa = (SparseArray)BDOMAIN_COROUTINES.get();
	synchronized( sa )
	{
	    ObjectSparseArrayElement elem;

	    elem = (ObjectSparseArrayElement)sa.getElement();
	    elem.setValue(this);
	    this.id = elem.getID();
	}
    }

    /**
     * @return The Coroutines unique identifier or INVALID_ID if the Coroutine
     * has reached an error or completed state.
     */
    public final long getID()
    {
	return this.id;
    }

    /**
     * Set the state of the Coroutine and notify any waiting threads.  If the
     * new state value is less than the current state it is ignored.  If the
     * new state value is an error or STATE_COMPLETE the Coroutine is disabled.
     *
     * @param newState The new state value.  */
    protected synchronized void setState(int newState)
    {
	if( (newState > this.state) || (newState <= STATE_ERROR) )
	{
	    this.state = newState;
	    this.notifyAll();
	    if( (newState <= STATE_ERROR) || (newState == STATE_COMPLETE) )
	    {
		this.drop();
	    }
	}
    }

    /**
     * @return The current state of the Coroutine.
     */
    public final int getState()
    {
	return this.state;
    }

    /**
     * @param throwable The exception that disabled this Coroutine.
     */
    protected final void setThrowable(Throwable throwable)
    {
	Assert.require(throwable);

	this.throwable = throwable;
	this.setState(STATE_ERROR);
    }

    /**
     * @return The exception that disabled this Coroutine.
     */
    public final Throwable getThrowable()
    {
	return this.throwable;
    }

    /**
     * @param nextState The state to wait for.
     * @param timeout The amount of time to wait before setting the state to
     * STATE_ERROR_TIMEOUT and returning.
     * @return The state reached during the wait.
     */
    public synchronized int waitForState(int nextState, long timeout)
    {
	long end;

	Assert.require(!EventHandler.isEventHandler());
	
	if( timeout == 0 )
	{
	    end = Long.MAX_VALUE;
	}
	else
	{
	    end = System.currentTimeMillis() + timeout;
	}
	while( (this.state > STATE_ERROR) && (this.state < nextState) )
	{
	    try
	    {
		this.wait(end - System.currentTimeMillis());
		if( System.currentTimeMillis() >= end )
		{
		    this.setState(STATE_ERROR_TIMEOUT);
		}
	    }
	    catch(InterruptedException e)
	    {
	    	/* XXX */
	    }
	}
	return this.state;
    }

    /**
     * @param state The state to wait for.
     * @return The state reached during the wait.
     */
    public int waitForState(int state)
    {
	return this.waitForState(state, DEFAULT_TIMEOUT);
    }
    
    /**
     * Wait for the Coroutine to reach STATE_COMPLETE.
     *
     * @param timeout The amount of time to wait before setting the state to
     * STATE_ERROR_TIMEOUT and returning.
     * @return True if the Coroutine reached STATE_COMPLETE.
     */
    public boolean waitForCompletion(long timeout)
    {
	return (this.waitForState(STATE_COMPLETE, timeout) == STATE_COMPLETE);
    }

    /**
     * Wait for the Coroutine to reach STATE_COMPLETE.
     *
     * @return True if the Coroutine reached STATE_COMPLETE.
     */
    public boolean waitForCompletion()
    {
	return this.waitForCompletion(DEFAULT_TIMEOUT);
    }

    /**
     * Callback used to handle BeesEvents directed to this Coroutine.
     *
     * @param ae The BeesEvent to handle.
     */
    public abstract void handle(BeesEvent ae);

    /**
     * Call back used to cleanup coroutines that are still in progress when a
     * BDomain has been shutdown.
     */
    public synchronized void cleanup()
    {
	if( this.state != STATE_COMPLETE )
	{
	    this.setState(STATE_ERROR_CAGED);
	}
    }

    /**
     * Drop the Coroutine, which entails setting the unique identifier to
     * INVALID_ID.
     */
    public synchronized void drop()
    {
	SparseArrayElement sae;
	SparseArray sa;

	sa = (SparseArray)BDOMAIN_COROUTINES.get();
	synchronized( sa )
	{
	    if( (sae = sa.findElement(this.id)) != null )
	    {
		sa.dropElement(sae);
	    }
	}
	this.id = INVALID_ID;
    }

    /**
     * @see java.lang.Object#finalize()
     */
    protected void finalize()
    {
	if( this.id != INVALID_ID )
	{
	    this.setState(STATE_ERROR_CAGED);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Coroutine["
	    + super.toString()
	    + "; id="
	    + this.id
	    + "; state="
	    + this.state
	    + "; throwable="
	    + this.throwable
	    + "]";
    }
}
