/*
 * DataRequestCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import bees.core.resource.Logger;

/**
 * A DataRequestCapsule is base class used for capsules that wish to request
 * some "large" data from a peer node.
 */
public abstract class DataRequestCapsule
    extends Capsule
{
    /**
     * The DataAggregator window that is being requested.
     */
    private int currentWindow;

    /**
     * The set of DataAggregator chunks that is being requested.
     */
    private long windowChunks;

    /**
     * Construct an empty DataRequestCapsule.
     */
    protected DataRequestCapsule()
    {
    }

    /**
     * Construct and initialize DataRequestCapsule from the given
     * DataAggregator.
     *
     * @param da The DataAggregator object that should be used in determining
     * the needed window and chunk set.  If NULL, the current window is
     * assumed to be zero and the needed chunks set is all one's.
     */
    protected DataRequestCapsule(DataAggregator da)
    {
	if( da != null )
	{
	    this.currentWindow = da.getCurrentWindow();
	    this.windowChunks = ~da.getWindowChunks();
	}
	else
	{
	    this.currentWindow = 0;
	    this.windowChunks = ~0;
	}
    }

    /**
     * @see bees.core.Capsule#decode(Xdr)
     */
    public void decode(Xdr xdr)
    {
	this.currentWindow = xdr.INT();
	this.windowChunks = xdr.LONG();
    }
    
    /**
     * @see bees.core.Capsule#encode(Xdr)
     */
    public void encode(Xdr xdr)
    {
	xdr.PUT(this.currentWindow);
	xdr.PUT(this.windowChunks);
    }

    /**
     * @return The DataAggregator window that is being requested.
     */
    protected int getCurrentWindow()
    {
	return this.currentWindow;
    }

    /**
     * @return The set of DataAggregator chunks that is being requested.
     */
    protected long getWindowChunks()
    {
	return this.windowChunks;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "DataRequestCapsule["
	    + super.toString()
	    + "; currentWindow="
	    + this.currentWindow
	    + "; windowChunks=0x"
	    + Long.toHexString(this.windowChunks)
	    + "]";
    }
}
