/*
 * FlowRequestCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;

import bees.core.kernel.KernelPanic;

import bees.core.resource.Logger;

/**
 * A ByteArrayRequestCapsule is used to request a flow tuple from a peer node.
 *
 * @see bees.core.ProtocolFlow
 */
final class FlowRequestCapsule
    extends ReliableDataRequestCapsule
{
    /**
     * The neighbor to ask for the flow description.
     */
    private final Neighbor neighbor;

    /**
     * Construct an empty FlowRequestCapsule.
     */
    FlowRequestCapsule()
    {
	this.neighbor = null;
    }

    /**
     * Construct a FlowRequestCapsule with the given values.
     *
     * @param neighbor The neighbor to ask for the flow description.
     * @param flowID The flow identifier to download.
     */
    FlowRequestCapsule(Neighbor neighbor, TypeID flowID)
    {
	super(UnknownProtocol.getFlowDesc(flowID));

	Assert.require(neighbor);
	
	this.neighbor = neighbor;
    }

    /**
     * @return The neighbor to ask for the flow description.
     */
    Neighbor getNeighbor()
    {
	return this.neighbor;
    }

    /**
     * Resend this capsule to the destination.
     *
     * @param node The current node.
     * @param ps The ProtocolSession this capsule is bound to.
     */
    public void resend(Node node, ProtocolSession ps)
    {
	try
	{
	    this.neighbor.send(this);
	}
	catch(EncodingException e)
	{
	    Logger.log(e);
	}
	catch(IOException e)
	{
	    Logger.log(e);
	}
    }

    /**
     * @see bees.core.Capsule#evaluate(Node)
     */
    public void evaluate(Node node)
    {
	TypeID specializer;

	/*
	 * The specializer contains the flow ID thats they're trying to
	 * download.
	 */
	if( (specializer = this.getSpecializer()) != null )
	{
	    FlowType ft;
	    
	    if( (ft = FlowType.getBDomainFlow(specializer)) != null )
	    {
		FlowResponseCapsule reply;
		
		reply = new FlowResponseCapsule(this, ft);
		reply.fromCapsule(this, specializer);
		try
		{
		    this.getSource().send(reply);
		}
		catch(EncodingException e)
		{
		    Logger.log(e);
		}
		catch(IOException e)
		{
		    Logger.log(e);
		}
	    }
	    else
	    {
		/* We do not have the requested flow identifier. */
		Logger.log(Logger.WARNING,
			   "No FlowType for specializer: " + specializer);
	    }
	}
	else
	{
	    Logger.log(Logger.WARNING, "Empty flow request specializer.");
	}
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "FlowRequestCapsule["
	    + super.toString()
	    + "; neighbor="
	    + this.neighbor
	    + "]";
    }
}
