/*
 * FlowResponseCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;

import bees.core.resource.Logger;

/**
 * A FlowResponseCapsule is used to respond to a flow tuple request from a peer
 * node.
 */
final class FlowResponseCapsule
    extends DataResponseCapsule
{
    /**
     * Construct an empty FlowResponseCapsule.
     */
    FlowResponseCapsule()
    {
    }

    /**
     * Construct a response to a FlowRequestCapsule.
     *
     * @param req The request capsule.
     * @param ct The FlowType containing the flow tuple.
     */
    FlowResponseCapsule(FlowRequestCapsule req, FlowType ft)
    {
	super(req, UnknownProtocol.getFlowDesc(ft.getFlowID()).getBytes());
    }

    /**
     * @see bees.core.Capsule#evaluate(Node)
     */
    public void evaluate(Node node)
    {
	TypeID specializer;
	DataAggregator da;

	specializer = this.getSpecializer();
	if( (da = UnknownProtocol.getFlowDesc(specializer,
					      this.getTotalLength())) != null )
	{
	    this.getSession().getCache().remove(specializer);
	    if( super.merge(da) )
	    {
		ExternalCapsule ec;
		byte desc[];
		Xdr xdr;
		
		ec = new ExternalCapsule(new CapabilityReference[] {
		    new CapabilityReference(this.getSource().getNeighborImpl())
		}).init();
		xdr = ec.getXDR();
		xdr.PUT(UnknownProtocolSession.TYPE_NEW_PROTOCOL);
		desc = da.getBytes();
		xdr.PUT(desc.length);
		xdr.PUT(desc);
		((UnknownProtocolSession)this.getSession()).
		    notifyInterests(ec);

		UnknownProtocol.removeFlowDesc(specializer);
	    }
	    else
	    {
		FlowRequestCapsule frc;

		frc = new FlowRequestCapsule(this.getSource(), specializer);
		frc.fromCapsule(this, specializer);
		try
		{
		    this.getSource().send(frc);
		}
		catch(EncodingException e)
		{
		    throw new Panic(e);
		}
		catch(IOException e)
		{
		    Logger.log(e);
		}
	    }
	}
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "FlowResponseCapsule["
	    + super.toString()
	    + "]";
    }
}
