/*
 * InheritedCapabilityCoroutine.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import bees.core.resource.Logger;
import bees.core.resource.SystemClass;
import bees.core.resource.SystemPackage;

import bees.core.kernel.KernelCapabilities;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.bdomain.BDomain;
import bees.core.bdomain.CapabilityTree;
import bees.core.bdomain.DeadBDomainException;

/**
 * Coroutine used to pass capabilities to a BDomain.
 */
public final class InheritedCapabilityCoroutine
    extends ResponderCoroutine
{
    static  {
	EventChannel.sub.addHandler("passCapability", new JSIHandler() {
		public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
		{
		    return this;
		}
		
		public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
		    throws MissingPairException,
			   DeadBDomainException
		{
		    switch( jm.getWhat() )
		    {
		    case JSIMessage.EXECUTE_PROPERTY:
			{
			    InheritedCapabilityCoroutine icc;
			    Vector envelopes = new Vector();
			    String str, name, cap;
			    EventChannel ec;
			    int lpc;
			    
			    ec = EventChannel.threadChannel();
			    
			    for( lpc = 0;
				 (str = jm.get("named", lpc, null)) != null;
				 lpc++ )
			    {
				Object obj;
				int index;

				index = str.indexOf("=");
				if( index < 0 )
				    throw new IllegalArgumentException();
				name = str.substring(0, index);
				cap = str.substring(index + 1);
				obj = KernelCapabilities.getTree().get(cap);
				if( obj instanceof Capability )
				{
				    CapabilityEnvelope ce;

				    ce = new CapabilityEnvelope(
					name, (Capability)obj);
				    envelopes.addElement(ce);
				}
				else if( obj instanceof ObjectMap )
				{
				    CapabilityTree.flattenMap(envelopes,
							      name,
							      (ObjectMap)obj);
				}
				else if( obj instanceof CapabilityEnvelope )
				{
				    envelopes.addElement(
					new CapabilityEnvelope(
						name,
						(CapabilityEnvelope)obj));
				}
				else
				{
				    throw new Panic(
					"Bogus object in capability tree: "
					+ obj);
				}
			    }

			    for( lpc = 0;
				 (cap = jm.get("extra", lpc, null)) != null;
				 lpc++ )
			    {
				Object obj;
				
				if( cap.equals("") )
				    continue;
				obj = KernelCapabilities.getTree().get(cap);
				if( obj instanceof Capability )
				{
				    CapabilityEnvelope ce;

				    ce = new CapabilityEnvelope(
					(Capability)obj);
				    envelopes.addElement(ce);
				}
				else if( obj instanceof ObjectMap )
				{
				    CapabilityTree.flattenMap(envelopes,
							      null,
							      (ObjectMap)obj);
				}
				else if( obj instanceof CapabilityEnvelope )
				{
				    envelopes.addElement(
					new CapabilityEnvelope(
						(CapabilityEnvelope)obj));
				}
				else
				{
				    throw new Panic(
					"Bogus object in capability tree: "
					+ obj);
				}
			    }

			    icc = new InheritedCapabilityCoroutine();
			    icc.initial(ec, envelopes);
			    if( icc.waitForCompletion() )
			    {
				jm.reply("ok");
			    }
			    else
			    {
				jm.error("oops");
			    }
			}
			break;
		    default:
			jm.error("Unsupported operation");
			break;
		    }
		}

		public void getSuites(JSIMessage jm)
		{
		}
	    });
    }

    public static final int STATE_WAITING = STATE_USER;

    public InheritedCapabilityCoroutine(StateResponder sr)
    {
	super(sr);
    }

    public InheritedCapabilityCoroutine()
    {
	this(null);
    }

    public synchronized void initial(EventChannel ec, CapabilityEnvelope ce[])
    {
	NamedCapabilitiesEvent nce;
	
	nce = new NamedCapabilitiesEvent(this, ce);
	try
	{
	    ec.sendEvent(nce);
	    this.setState(STATE_WAITING);
	}
	catch(DeadBDomainException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void initial(EventChannel ec, Vector vec)
    {
	CapabilityEnvelope ce[];
	
	synchronized( vec )
	{
	    int lpc;
	    
	    ce = new CapabilityEnvelope[vec.size()];
	    for( lpc = 0; lpc < ce.length; lpc++ )
	    {
		ce[lpc] = (CapabilityEnvelope)vec.elementAt(lpc);
	    }
	}
	this.initial(ec, ce);
    }

    private final class Responder
	extends StateResponder
    {
	private final KernelObjectCoroutine parent;
	private final NamedCapabilitiesEvent nce;
	private final CapabilityEnvelope ce[];
	private final ObjectTree ot;
	private int lpc;

	Responder(NamedCapabilitiesEvent nce)
	{
	    super(Coroutine.STATE_ANY);

	    this.parent = null;
	    this.nce = nce;
	    this.ce = nce.getEnvelopes();
	    this.ot = BDomain.currentBDomain().getCapabilities();
	}

	Responder(KernelObjectCoroutine parent)
	{
	    super(Coroutine.STATE_ANY);

	    this.parent = parent;
	    this.nce = null;
	    this.ce = parent.getSubcapabilities();
	    this.ot = BDomain.currentBDomain().getCapabilities();
	}

	void lookupEnvelope()
	{
	    while( (this.lpc < this.ce.length) &&
		   (this.ce[this.lpc] == null) )
	    {
		this.lpc += 1;
	    }
	    if( this.lpc < this.ce.length )
	    {
		this.ce[this.lpc].lookup(this);
	    }
	    else
	    {
		BDomain bdomain = BDomain.currentBDomain();

		if( this.nce != null )
		{
		    synchronized( bdomain )
		    {
			bdomain.notify();
		    }
		    try
		    {
			this.nce.sendReply();
		    }
		    catch(DeadBDomainException e)
		    {
		    }
		}
		else
		{
		    this.parent.setState(KernelObjectCoroutine.
					 STATE_SUB_LOADED);
		}
	    }
	}
	
	public void transition(Coroutine coroutine, int newState)
	{
	    KernelObjectCoroutine koc;

	    koc = (KernelObjectCoroutine)coroutine;
	    switch( newState )
	    {
	    case KernelObjectCoroutine.STATE_LOADED:
		if( koc.getSubcapabilities() != null )
		{
		    new Responder(koc).lookupEnvelope();
		}
		else
		{
		    koc.setState(KernelObjectCoroutine.STATE_SUB_LOADED);
		}
		break;
	    case KernelObjectCoroutine.STATE_SUB_LOADED:
		koc.instantiate(this.ce[this.lpc].getPermissions());
		break;
	    case KernelObjectCoroutine.STATE_COMPLETE:
		try
		{
		    Capability cap;
		    
		    cap = this.ce[this.lpc].unseal(koc);
		    if( (this.nce != null) &&
			(this.ce[this.lpc].getName() != null) )
		    {
			this.ot.put(this.ce[this.lpc].getName(), cap);
		    }
		    else if( cap instanceof SystemClass )
		    {
			((SystemClass)cap).export();
		    }
		    else if( cap instanceof SystemPackage )
		    {
			((SystemPackage)cap).export();
		    }
		    else
		    {
		    }
		}
		catch(MissingCapabilityException e)
		{
		    Logger.log(e);
		}
		catch(Throwable th)
		{
		    th.printStackTrace();
		    Logger.log(th);
		}
		this.lpc += 1;
		this.lookupEnvelope();
		break;
	    default:
		Logger.log("Unable to bind to capability: state="
			   + newState
			   + "; this="
			   + this);
		break;
	    }
	}

	public String toString()
	{
	    return "Responder["
		+ super.toString()
		+ "; nce="
		+ this.nce
		+ "; ce="
		+ this.ce
		+ "; lpc="
		+ this.lpc
		+ "]";
	}
    }

    public synchronized void handle(BeesEvent ae)
    {
	switch( this.getState() )
	{
	case STATE_NEWBORN:
	    if( (ae instanceof NamedCapabilitiesEvent) && !ae.isReply() )
	    {
		NamedCapabilitiesEvent nce;
		Responder responder;

		nce = (NamedCapabilitiesEvent)ae;
		responder = new Responder(nce);
		responder.lookupEnvelope();
		this.setState(STATE_COMPLETE);
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_WAITING:
	    if( (ae instanceof NamedCapabilitiesEvent) && ae.isReply() )
	    {
		NamedCapabilitiesEvent nce;
		Throwable th;

		nce = (NamedCapabilitiesEvent)ae;
		if( (th = nce.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else
		{
		    this.setState(STATE_COMPLETE);
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	default:
	    throw new Panic("Invalid state for coroutine: " + this);
	}
	this.drop();
    }
    
    public String toString()
    {
	return "InheritedCapabilityCoroutine["
	    + super.toString()
	    + "]";
    }
}
