/*
 * InternalPacketAuthorization.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import edu.utah.janos.nodeos.Domain;

import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.CountPermission;
import bees.core.security.PermissionException;

final class InternalPacketAuthorization
    implements Cloneable
{
    static final String TRANSFORM_PREFIX = "transformTo.";
    static final int TRANSFORM_PREFIX_LENGTH = TRANSFORM_PREFIX.length();
    
    static class ClassValuePair
	implements Cloneable
    {
	Class cl;
	int value;
	int maxValue;

	public Object clone()
	{
	    try
	    {
		return super.clone();
	    }
	    catch(CloneNotSupportedException e)
	    {
		throw new Panic(e);
	    }
	}

	public String toString()
	{
	    return "ClassValuePair[cl="
		+ this.cl
		+ "; value="
		+ this.value
		+ "; maxValue="
		+ this.maxValue
		+ "]";
	}
    }

    private final ClassValuePair values[];
    
    InternalPacketAuthorization(ClassValuePair values[])
    {
	int lpc;
	
	this.values = new ClassValuePair[values.length];
	for( lpc = 0; lpc < this.values.length; lpc++ )
	{
	    this.values[lpc] = (ClassValuePair)values[lpc].clone();
	}
    }
    
    InternalPacketAuthorization(PermissionSet ps)
    {
	Domain domain = Domain.currentDomain();
	Permission perms[];
	int lpc;

	perms = ps.toArray();
	this.values = new ClassValuePair[perms.length];
	for( lpc = 0; lpc < perms.length; lpc++ )
	{
	    try
	    {
		this.values[lpc] = new ClassValuePair();
		this.values[lpc].cl = domain.
		    loadClass(perms[lpc].getName().
			      substring(TRANSFORM_PREFIX_LENGTH));
		this.values[lpc].value = this.values[lpc].maxValue =
		    ((CountPermission)perms[lpc]).getMaxCount();
	    }
	    catch(ClassNotFoundException e)
	    {
		Logger.log(Logger.EMERG,
			   "Could not find class referenced by transformTo "
			   + "permission");
		Logger.log(e);
	    }
	    catch(ClassCastException e)
	    {
		Logger.log(Logger.EMERG,
			   "Expected CountPermission for tranformTo");
		Logger.log(e);
	    }
	}
    }

    void consume(Class cl)
    {
	int lpc;

	for( lpc = 0; lpc < this.values.length; lpc++ )
	{
	    if( this.values[lpc].cl == cl )
	    {
		if( this.values[lpc].value > 0 )
		{
		    this.values[lpc].value -= 1;
		    return;
		}
	    }
	}
	throw new PermissionException("Invalid access:  Packet "
				      + "doesn't have permission "
				      + "to transform to: "
				      + cl
				      + "; "
				      + this);
    }

    void replenish()
    {
	int lpc;

	for( lpc = 0; lpc < this.values.length; lpc++ )
	{
	    this.values[lpc].value = this.values[lpc].maxValue;
	}
    }

    public Object clone()
    {
	InternalPacketAuthorization retval;

	retval = new InternalPacketAuthorization(this.values);
	return retval;
    }
    
    public String toString()
    {
	String retval;
	int lpc;
	
	retval = "InternalPacketAuthorization[values={ ";
	for( lpc = 0; lpc < this.values.length; lpc++ )
	{
	    retval += this.values[lpc] + " ";
	}
	retval += " }]";
	return retval;
    }
}
