/*
 * ObjectTree.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.util.Enumeration;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;

/**
 * A class used to maintain a hierarchy of objects in the system.
 */
public final class ObjectTree
{
    /**
     * The character used to separate qualified names.
     */
    public static final char SEPARATOR = '/';
    
    /**
     * A constant used to detect when a name is too deep.
     */
    private static final ObjectMap empty = new ObjectMap() {
	    public Object get(Object key) { return null; }
	    public Object put(Object key, Object value) { return null; }
	    public Object remove(Object key) { return null; }
	    public Enumeration keys() { return null; }
	    public Enumeration elements() { return null; }
	    public int size() { return 0; }
	};

    /**
     * The root of the getTree().
     */
    private final ObjectMap root = new HashMap();

    /**
     * Construct an empty object getTree().
     */
    public ObjectTree()
    {
    }

    /**
     * @return The root ObjectMap of the getTree().
     */
    public ObjectMap getRootMap()
    {
	return this.root;
    }

    /**
     * Get an object from the tree using a fully qualified name.
     *
     * @param name The qualified name of the object.
     * @return The named object.
     * @exception Throws NoSuchElementException if the object couldn't be
     * found.
     */
    public synchronized Object get(String name)
	throws NoSuchElementException
    {
	Object retval = null;

	Assert.require(name);

	if( name.equals("/") )
	{
	    retval = this.root;
	}
	else if( name.indexOf(SEPARATOR) != -1 )
	{
	    StringTokenizer st = new StringTokenizer(name, "" + SEPARATOR);
	    ObjectMap om = this.root;
	    
	    while( st.hasMoreElements() )
	    {
		String sub = st.nextToken();
		Object obj;

		if( sub.equals("") )
		{
		    /* Be nice, ignore empty sub names */
		}
		else if( (obj = om.get(sub)) != null )
		{
		    if( obj instanceof ObjectMap )
			om = (ObjectMap)obj;
		    else
			om = empty;
		    retval = obj;
		}
		else
		{
		    throw new NoSuchElementException(
			"Couldn't find sub-object: "
			+ sub
			+ " in "
			+ name);
		}
	    }
	}
	else 
	{
	    if( (retval = this.root.get(name)) == null )
	    {
		throw new NoSuchElementException("Couldn't find sub-object: "
						 + name);
	    }
	}
	return retval;
    }

    public boolean contains(String name)
    {
	try
	{
	    this.get(name);
	    return true;
	}
	catch(NoSuchElementException e)
	{
	    return false;
	}
    }
    
    /**
     * Remove an object from the tree using a fully qualified name.
     *
     * @param name The qualified name of the object.
     * @return The named object.
     * @exception Throws NoSuchElementException if the object couldn't be
     * @exception found.
     */
    public synchronized Object remove(String name)
	throws NoSuchElementException
    {
	Object retval = null;

	Assert.require(name);
	
	if( name.indexOf(SEPARATOR) != -1 )
	{
	    StringTokenizer st = new StringTokenizer(name, "" + SEPARATOR);
	    ObjectMap om = this.root;
	    
	    while( st.hasMoreElements() )
	    {
		String sub = st.nextToken();
		Object obj;

		if( sub.equals("") )
		{
		}
		else if( (obj = om.get(sub)) != null )
		{
		    if( st.hasMoreElements() )
		    {
			if( obj instanceof ObjectMap )
			    om = (ObjectMap)obj;
			else
			    om = empty;
		    }
		    else
		    {
			retval = om.remove(sub);
		    }
		}
		else
		{
		    throw new NoSuchElementException(
			"Couldn't find sub-object: "
			+ sub);
		}
	    }
	}
	else
	{
	    retval = this.root.remove(name);
	}
	if( retval == null )
	{
	    throw new NoSuchElementException("Couldn't find sub-object: "
					     + name);
	}
	return retval;
    }

    /**
     * Put an object in the getTree().  If parts of the hierarchy as given by
     * the name are missing then they will automatically be filled in with
     * HashMaps.
     *
     * @param name The qualified name of the object.
     * @param newobj The object to put in the getTree().
     */
    public synchronized void put(String name, Object newobj)
    {
	Assert.require(name);
	Assert.require(newobj);
	
	if( name.indexOf(SEPARATOR) != -1 )
	{
	    StringTokenizer st = new StringTokenizer(name, "" + SEPARATOR);
	    ObjectMap om = this.root;
	    
	    while( st.hasMoreElements() )
	    {
		String sub = st.nextToken();
		Object obj;

		if( sub.equals("") )
		{
		}
		else if( st.hasMoreElements() )
		{
		    /* We're at an intermediate directory in the path. */
		    if( (obj = om.get(sub)) != null )
		    {
			if( obj instanceof ObjectMap )
			{
			    /* Descend into the directory. */
			    om = (ObjectMap)obj;
			}
			else
			{
			    /* Something else is there. */
			    throw new Error("Name, "
					    + name
					    + ", already in use."); // XXX
			}
		    }
		    else
		    {
			/* Create a new directory. */
			ObjectMap nom = new HashMap();
			
			om.put(sub, nom);
			om = nom;
		    }
		}
		else if( (obj = om.get(sub)) != null )
		{
		    throw new Error("Name, "
				    + name
				    + ", already in use."); // XXX
		}
		else
		{
		    om.put(sub, newobj);
		}
	    }
	}
	else if( this.root.get(name) == null )
	{
	    this.root.put(name, newobj);
	}
	else
	{
		throw new Error("Name, "
				+ name
				+ ", already in use."); // XXX
	}
    }

    public String toString()
    {
	return "ObjectTree[root="
	    + this.root
	    + "]";
    }
}
