/*
 * ProtocolBDomain.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import java.io.IOException;
import java.io.InputStream;
import java.io.DataInputStream;

import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.Node;

import bees.util.MD5;

import bees.core.kernel.KernelPanic;

import bees.core.resource.File;
import bees.core.resource.Logger;

import bees.core.bdomain.BDomain;

/**
 * The ProtocolBDomain is a BDomain root class used to start new protocol
 * flows.
 *
 * XXX Unfortunately, its also a dumping ground for other functions.  The
 * createCapsuleTypes() junk should be moved elsewhere.
 */
public final class ProtocolBDomain
{
    private static void createCapsuleTypes(FlowType ft,
					   ProtocolType pt,
					   ByteArrayType companionType,
					   CapsuleType companionCapsuleType,
					   ObjectTree capabilities)
	throws IOException
    {
	int lpc, ctsSize, companionsSize;
	Vector cts, companions;
	String session;

	Assert.require(ft);

	cts = pt.getByteArrayTypes();
	companions = pt.getCompanions();
	ctsSize = cts.size();
	companionsSize = companions.size();
	for( lpc = 0; lpc < ctsSize; lpc++ )
	{
	    ByteArrayType bat = (ByteArrayType)cts.elementAt(lpc);
	    CapsuleType capsuleType = null;
	    boolean addToPool = false;
	    CodeType ct;
	    int lpc2;

	    if( bat instanceof CodeType )
	    {
		ct = (CodeType)bat;
		switch( ct.getState() )
		{
		case ByteArrayType.STATE_NEW:
		    break;
		case ByteArrayType.STATE_LOADED:
		case ByteArrayType.STATE_BOUND:
		case ByteArrayType.STATE_BUILTIN:
		    if( (ct.getType() == ByteArrayType.TYPE_CAPSULE) &&
			ct.isA(Capsule.class) /* Make sure */ )
		    {
			capsuleType = new CapsuleType();
			capsuleType.setCodeType(ct);
			capsuleType.setFlowType(ft);
			capsuleType.setProtocolType(pt);
			switch( pt.getCompanionType() )
			{
			case ProtocolType.COMPANION_NO:
			    break;
			case ProtocolType.COMPANION_BYTEARRAY:
			    if( companionType != null )
			    {
				FlowByteArrayType fbat;
				
				fbat = new FlowByteArrayType().
				    setFlowType(ft).
				    setByteArrayType(companionType).
				    snapshotFlowByteArrayType();
				FlowByteArrayType.putBDomainFlowByteArray(fbat);
				
				capsuleType.setSpecializedCapsuleID(
					fbat.getFlowByteArrayID());
			    }
			    break;
			case ProtocolType.COMPANION_CAPSULE:
			    if( (companionType != null) &&
				(companionType.getState() > ByteArrayType.STATE_NEW) &&
				(companionType.getType() == ByteArrayType.TYPE_CAPSULE) /* &&
											   companionType.isFinal() */ )
			    {
				capsuleType.setSpecializedCapsuleID(
					companionCapsuleType.getCapsuleID());
			    }
			    break;
			default:
			    break;
			}
			capsuleType.snapshotCapsuleType();
			if( !CapsuleType.containsBDomainCapsuleType(capsuleType) &&
			    ((pt.getCompanionType() == ProtocolType.COMPANION_NO)
			     || (capsuleType.getSpecializedCapsuleID() != null)
			     || (ft.getPrimaryProtocolType() == pt)) )
			{
			    CapsuleType.putBDomainCapsuleType(capsuleType);
			    addToPool = true;
			    // createInChannels(capsuleType, capabilities);
			}
		    }
		    if( ct.isA(ProtocolSession.class) )
		    {
			SessionType st;
			
			st = new SessionType().
			    setFlowType(ft).
			    setProtocolType(pt).
			    snapshotSessionType();
			SessionType.putBDomainSession(st);
		    }
		    break;
		default:
		    throw new KernelPanic("");
		}
	    }
	    for( lpc2 = 0; lpc2 < companionsSize; lpc2++ )
	    {
		ProtocolType cp;

		cp = (ProtocolType)companions.elementAt(lpc2);
		createCapsuleTypes(ft, cp, bat, capsuleType, capabilities);
	    }
	    if( addToPool )
	    {
		CapsulePool.addBDomainCapsulePool(capsuleType);
	    }
	}
    }

    static void createCapsuleTypes(FlowType ft, ObjectTree capabilities)
	throws IOException
    {
	int lpc, companionsSize;
	Vector companions;
	ProtocolType pt;
	String session;

	FlowType.putBDomainFlow(ft);
	pt = ft.getPrimaryProtocolType();
	createCapsuleTypes(ft, pt, null, null, capabilities);
	ProtocolType.putBDomainProtocol(ft.getPrimaryProtocolType());
	companions = ft.getCompanions();
	companionsSize = companions.size();
	for( lpc = 0; lpc < companionsSize; lpc++ )
	{
	    int lpc2, ctsSize;
	    Vector cts;

	    pt = (ProtocolType)companions.elementAt(lpc);
	    cts = pt.getByteArrayTypes();
	    ctsSize = cts.size();
	    for( lpc2 = 0; lpc2 < ctsSize; lpc2++ )
	    {
		ByteArrayType bat = (ByteArrayType)cts.elementAt(lpc2);

		if( bat instanceof CodeType )
		{
		    CodeType ct;

		    ct = (CodeType)bat;
		    if( ct.getType() == ByteArrayType.TYPE_CAPSULE )
		    {
			CapsuleType capsuleType;

			capsuleType = new CapsuleType();
			capsuleType.setFlowType(ft);
			capsuleType.setProtocolType(pt);
			capsuleType.setCodeType(ct);
			switch( pt.getCompanionType() )
			{
			case ProtocolType.COMPANION_FLOW:
			    capsuleType.
				setSpecializedCapsuleID(ft.getFlowID());
			    break;
			default:
			    throw new KernelPanic("");
			}
			capsuleType.snapshotCapsuleType();
			if( !CapsuleType.containsBDomainCapsuleType(capsuleType) )
			{
			    CapsuleType.putBDomainCapsuleType(capsuleType);
			    CapsulePool.addBDomainCapsulePool(capsuleType);
			    // createInChannels(capsuleType, capabilities);
			}
		    }
		    if( ct.isA(ProtocolSession.class) )
		    {
			SessionType st;
			
			st = new SessionType().
			    setFlowType(ft).
			    setProtocolType(pt).
			    snapshotSessionType();
			SessionType.putBDomainSession(st);
		    }
		}
	    }
	}
    }

    private static String[] split(String str, char token)
    {
	String retval[] = new String[2];
	int index;
	
	if( (index = str.indexOf(token)) != -1 )
	{
	    retval[0] = str.substring(0, index);
	    retval[1] = str.substring(index + 1);
	}
	else
	{
	    throw new IllegalArgumentException();
	}
	return retval;
    }

    /**
     * The BDomain main function, used to start a protocol.  This method will
     * first try to retrieve a file capability using the name "flow-desc".  If
     * this capability is found, it is assumed to contain the flattened version
     * of a flow tuple.  If the capability is not found, a default tuple will
     * be generated automatically.  However, it will not contain any
     * authorization data.  Next, the primary Protocol named by the first
     * argument is loaded and instantiated.  Finally, an instance of the
     * primary protocol's bootstrap capsule is created and evaluated.
     *
     * If the method is called with no arguments, it is assumed that the
     * protocols will be created later using Coroutines.  For example, the
     * Node.register() method will use the LocalFlowCoroutine to initialize
     * a protocol domain.
     *
     * @param args The arguments to the BDomain.
     * @param capabilities The capabilities passed to the BDomain.
     */
    public static void main(String args[], ObjectTree capabilities)
	throws Exception
    {
	Domain domain = Domain.currentDomain();
	FlowType ft = null;
	
	domain.enableAccessControls(false);
	try
	{
	    Capability cap;

	    cap = (Capability)capabilities.get("flow-desc");
	    if( cap instanceof File )
	    {
		File file = (File)cap;
		DataInputStream dis;
		byte data[];
		Class cl;
		
		data = new byte[(int)file.length()];
		dis = new DataInputStream(file.getInputStream());
		dis.readFully(data);
		dis.close();
		ft = FlowType.instantiate(data);
		domain.exportClass(args[0]);
		cl = domain.loadClass(args[0]);
		ft.updatePrimaryProtocolType(((Protocol)cl.newInstance()).
					     getProtocolType());
	    }
	}
	catch(NoSuchElementException e)
	{
	    if( args.length > 0 )
	    {
		try
		{
		    ProtocolFlow pf;
		    int lpc;
		    
		    domain.exportClass(args[0]);
		    pf = new ProtocolFlow();
		    pf.setPrimaryProtocol(args[0]);
		    pf.addCompanion("bees.core.UnknownProtocol");
		    for( lpc = 1; lpc < args.length; lpc += 2 )
		    {
			String spair[];
			
			if( args[lpc].equals("-pair") )
			{
			    spair = split(args[lpc + 1], '=');
			    pf.addPair(spair[0], spair[1]);
			}
		    }
		    ft = pf.getFlowType();
		}
		catch(Throwable th)
		{
		    Node.println("Failed to start protocol: " + args[0]);
		    th.printStackTrace();
		    BDomain.currentBDomain().shutdown();
		}
	    }
	}
	domain.enableAccessControls(true);
	if( ft != null )
	{
	    String bootStrapClassName;
	    
	    createCapsuleTypes(ft, capabilities);
	    Logger.log("Starting up protocol: " + ft.getFlowID());
	    if( (bootStrapClassName = ft.getPrimaryProtocolType().
		 getBootStrapClassName()) != null )
	    {
		Capsule cap;
		Class cl;
		
		cl = domain.loadClass(bootStrapClassName);
		cap = (Capsule)cl.newInstance();
		cap.fromFlowType(ft, CapsuleType.NULL_SPECIALIZER);
		try
		{
		    cap.evaluate(bees.core.Node.currentNode());
		}
		catch(Throwable th)
		{
		    Node.println("Failed to start protocol: ");
		    Node.printStackTrace(th);
		    BDomain.currentBDomain().shutdown();
		}
	    }
	}
    }
}
