/*
 * ReliableDataRequestCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core;

import bees.core.security.IntegerPermission;

/**
 * The ReliableDataRequestCapsule is a base class for Capsule classes that
 * wish to transmit chunks of data reliably.
 */
public abstract class ReliableDataRequestCapsule
    extends DataRequestCapsule
    implements NodeCache.EventHandler
{
    /**
     * Default interval for resending a request Capsule.
     */
    public static final IntegerPermission DEFAULT_RETRY_INTERVAL =
	new IntegerPermission("interval", 500);

    /**
     * Default number of times to try and resend a Capsule.
     */
    public static final int DEFAULT_RETRY_COUNT = 4;

    /**
     * The number of retries left.
     */
    private int retriesLeft = DEFAULT_RETRY_COUNT;

    /**
     * Construct an empty ReliableDataRequestCapsule.
     */
    protected ReliableDataRequestCapsule()
    {
    }
    
    /**
     * Construct a ReliableDataRequestCapsule with the given values.
     *
     * @param da The DataAggregator that is managing the transfer of data.
     */
    protected ReliableDataRequestCapsule(DataAggregator da)
    {
	super(da);
    }

    /**
     * @param retriesLeft The number of retries left.
     */
    public void setRetriesLeft(int retriesLeft)
    {
	this.retriesLeft = retriesLeft;
    }

    /**
     * @return The number of retries left.
     */
    public int getRetriesLeft()
    {
	return this.retriesLeft;
    }

    /**
     * Call back used to resend this capsule.
     *
     * @param node The current Node object.
     * @param ps The ProtocolSession for this flow and protocol.
     */
    protected abstract void resend(Node node, ProtocolSession ps);

    public void removed(Node node, ProtocolSession ps)
    {
    }

    public IntegerPermission collected(Node node, ProtocolSession ps)
    {
	IntegerPermission retval = null;

	if( this.retriesLeft == 0 )
	{
	}
	else
	{
	    this.resend(node, ps);
	    this.retriesLeft -= 1;
	    retval = DEFAULT_RETRY_INTERVAL;
	}
	return retval;
    }

    public String toString()
    {
	return "ReliableDataRequestCapsule["
	    + super.toString()
	    + "; retriesLeft="
	    + this.retriesLeft
	    + "]";
    }
}
