/*
 * ANEPVersion.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.anep;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;

import bees.core.kernel.BeesKernel;

/**
 * Holds versioning data pertaining to ANEP.
 */
public final class ANEPVersion
{
    /**
     * The version number used in packets.
     */
    public static byte PACKET_VERSION = 1;

    /**
     * The version string associated with the forwarding code.
     */
    public static String FORWARDER_VERSION = "Bees Prototype";

    /**
     * Suite describing the root `anep-version' property.
     */
    private static final JSIPropertyInfo JSI_ROOT_SUITES[] = {
	new JSIPropertyInfo("anep-version",
			    (1 << JSIMessage.LIST_PROPERTIES),
			    "",
			    "List version data related to ANEP.\n")
    };
    
    /**
     * Suite describing the properties inside `anep-version'.
     */
    private static final JSIPropertyInfo JSI_VERSION_SUITES[] = {
	new JSIPropertyInfo("packet",
			    (1 << JSIMessage.GET_PROPERTY) |
			    (1 << JSIMessage.SET_PROPERTY),
			    "",
			    "The recognized ANEP packet version number.\n"),
	new JSIPropertyInfo("forwarder",
			    (1 << JSIMessage.GET_PROPERTY) |
			    (1 << JSIMessage.SET_PROPERTY),
			    "",
			    "The version of the ANEP forwarder.\n")
    };
    
    static
    {
	BeesKernel.getJSIRoot().addHandler("anep-version", new JSIHandler() {
		public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
		{
		    return this;
		}

		public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
		{
		    JSIPair jp;
		    
		    switch( jm.getWhat() )
		    {
		    case JSIMessage.LIST_PROPERTIES:
			jm.reply("packet");
			jm.add("result", "forwarder");
			break;
		    case JSIMessage.GET_PROPERTY:
			if( (jp = jm.popSpecifier()) != null )
			{
			    if( jp.getTag().equals("packet") )
			    {
				jm.reply(PACKET_VERSION);
			    }
			    else if( jp.getTag().equals("forwarder") )
			    {
				jm.reply(FORWARDER_VERSION);
			    }
			    else
			    {
				jm.error("There is no property named "
					 + jp.getTag()
					 + " in anep-version.");
			    }
			}
			else
			{
			    jm.reply("ok");
			    jm.add(new JSIPair("packet", PACKET_VERSION));
			    jm.add("forwarder", FORWARDER_VERSION);
			}
			break;
		    case JSIMessage.SET_PROPERTY:
			if( (jp = jm.popSpecifier()) != null )
			{
			    String setData;

			    if( (setData = jm.get(JSIMessage.SET_DATA))
				!= null )
			    {
				if( jp.getTag().equals("packet") )
				{
				    try
				    {
					PACKET_VERSION = (byte)
					    Integer.decode(setData).
					    intValue();
					jm.reply("ok");
				    }
				    catch(NumberFormatException e)
				    {
					jm.error("Set value not a number.");
				    }
				}
				else if( jp.getTag().equals("forwarder") )
				{
				    FORWARDER_VERSION = setData;
				    jm.reply("ok");
				}
				else
				{
				    jm.error("There is no property named "
					     + jp.getTag()
					     + " in anep-version.");
				}
			    }
			    else
			    {
				jm.error("No data to set.");
			    }
			}
			else
			{
			    jm.reply("ok");
			    jm.add(new JSIPair("packet", PACKET_VERSION));
			    jm.add("forwarder", FORWARDER_VERSION);
			}
			break;
		    case JSIMessage.GET_SUITES:
			jm.clear();
			jm.setWhat(JSIMessage.REPLY);
			jm.add("result", JSI_VERSION_SUITES);
			break;
		    default:
			jm.error("Operation not supported by property");
			break;
		    }
		}

		public void getSuites(JSIMessage jm)
		{
		    jm.add("result", JSI_ROOT_SUITES);
		}
	    });
    }

    /**
     * Private constructor, this is a static only class.
     */
    private ANEPVersion()
    {
    }
}
