/*
 * BDomain.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.bdomain;

import java.util.Vector;
import java.util.Hashtable;
import java.util.NoSuchElementException;

import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.lang.reflect.InvocationTargetException;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.Node;
import edu.utah.janos.nodeos.DomainRun;
import edu.utah.janos.nodeos.ClassInstantiationKey;

import edu.utah.janosvm.kit.comm.amiga.FaxPort;
import edu.utah.janosvm.kit.comm.amiga.SignalSet;
import edu.utah.janosvm.kit.comm.amiga.FaxPortHandle;

import edu.utah.janosvm.kit.comm.commspace.CommSpaceHandle;
import edu.utah.janosvm.kit.comm.commspace.CommSpaceElement;

import bees.core.Assert;
import bees.core.Coroutine;
import bees.core.ObjectTree;
import bees.core.Capability;
import bees.core.EventChannel;
import bees.core.CloneFactory;
import bees.core.StateResponder;
import bees.core.CapabilityFactory;
import bees.core.MissingCapabilityException;

import bees.core.kernel.BDomainCoroutine;

import bees.core.resource.Logger;

import bees.core.security.PermissionSet;

public final class BDomain
    extends DomainRun
{
    public static BDomain currentBDomain()
    {
	return (BDomain)Domain.currentDomain().getObject();
    }

    private Domain domain;
    private boolean dead = false;
    private FaxPort port;
    private BDomainCoroutine sc;
    private final Hashtable locals = new Hashtable();
    private final ObjectTree capabilities = new ObjectTree();
    private final Vector cleanupHandlers = new Vector();
    
    public BDomain(ClassInstantiationKey key)
    {
	key.checkEquals(BDomain.class);
    }

    void setName(String name)
    {
	this.domain.setName(name);
    }

    public String getName()
    {
	return this.domain.getName();
    }

    public BDomainCoroutine getBDomainCoroutine()
    {
	return this.sc;
    }

    public FaxPort getPort()
    {
	return this.port;
    }

    Hashtable getLocals()
    {
	return this.locals;
    }

    public ObjectTree getCapabilities()
    {
	return this.capabilities;
    }
    
    public void pushCleanupHandler(Runnable victorTheCleaner)
    {
	if( this.dead )
	{
	    victorTheCleaner.run();
	}
	else
	{
	    this.cleanupHandlers.addElement(victorTheCleaner);
	}
    }
    
    void cleanup()
    {
	boolean runHandlers = false;
	
	synchronized( this )
	{
	    if( !this.dead )
	    {
		runHandlers = true;
		this.dead = true;
	    }
	}
	if( runHandlers )
	{
	    int lpc, size;
	    
	    size = this.cleanupHandlers.size();
	    for( lpc = size - 1; lpc >= 0; lpc-- )
	    {
		Runnable victor;
		
		victor = (Runnable)this.cleanupHandlers.elementAt(lpc);
		victor.run();
	    }
	}
    }

    public void shutdown()
    {
	if( !this.dead )
	{
	    this.cleanup();
	    this.sc.shutdown();
	    this.port.close();
	    this.domain.destroy(); /* XXX Should we do this or the kernel? */
	    throw new ThreadDeath();
	}
    }

    void selectedDevelopment(String selector, String args[])
	throws ClassNotFoundException,
	       NoSuchMethodException,
	       BDomainDeathException
    {
	final Object params[];
	Class paramTypes[];
	final Method m;
	Class cl;

	Assert.require(selector);
	Assert.require(args);

	paramTypes = new Class[] {
	    args.getClass(),
	    this.capabilities.getClass()
	};
	params = new Object[] {
	    args,
	    this.capabilities
	};
	cl = domain.loadClass(selector);
	try
	{
	    m = cl.getMethod("main", paramTypes);
	}
	catch(NoSuchMethodException e)
	{
	    Method meths[];
	    int lpc;
	    
	    meths = cl.getMethods();
	    for( lpc = 0; lpc < meths.length; lpc++ )
	    {
		Node.println("Method["
			     + lpc
			     + "] - "
			     + meths[lpc]);
	    }
	    throw e;
	}
	if( Modifier.isStatic(m.getModifiers()) )
	{
	    this.domain.threadStart(new Runnable() {
		    public void run()
		    {
			try
			{
			    m.invoke(null, params);
			}
			catch(InvocationTargetException e)
			{
			    Throwable sub;
			    
			    // LogEvent.local("Exception in domain main:");
			    sub = e.getTargetException();
			    if( sub instanceof ExceptionInInitializerError )
			    {
				ExceptionInInitializerError eiie;

				eiie = (ExceptionInInitializerError)sub;
				eiie.getException().printStackTrace();
			    }
			    else if( sub instanceof ThreadDeath )
			    {
				/* Ignore */
			    }
			    else
			    {
				sub.printStackTrace();
			    }
			}
			catch(ThreadDeath td)
			{
			    /* Ignore */
			}
			catch(Throwable th)
			{
			    th.printStackTrace();
			}
		    }
		}, "Bees BDomain main thread");
	}
	else
	{
	    throw new BDomainDeathException();
	}
    }

    protected void run(Domain domain, String args[], CommSpaceHandle csh)
    {
	EventHandler eh = new EventHandler(this);
	String selector;
	
	domain.setObject(this);
	Assert.check(BDomain.currentBDomain() == this);
	this.domain = domain;
	this.port = eh.getPort();
	if( csh == null )
	{
	    domain.enableAccessControls(false);
	    domain.addSystemPackage("bees.util");
	    domain.addSystemPackage("bees.core");
	    domain.addSystemPackage("bees.core.kernel");
	    domain.addSystemPackage("bees.core.resource");
	    domain.addSystemPackage("bees.core.security");
	    domain.addSystemPackage("bees.core.bdomain");
	    domain.addSystemPackage("edu.utah.janosvm.kit.comm.jsi");
	    domain.addSystemPackage("edu.utah.janosvm.kit.comm.amiga");
	}
	this.sc = new BDomainCoroutine();
	if( csh != null )
	{
	    this.setName(csh.take(4).getString());
	}
	else
	{
	    this.setName("kernel");
	}
	this.domain.threadStart(eh, "Event Handler - " + this.getName());
	if( csh != null )
	{
	    domain.enableAccessControls(true);
	    this.sc.addResponder(new StateResponder(Coroutine.STATE_COMPLETE) {
		    public void transition(Coroutine coroutine, int newState)
		    {
			if( newState == Coroutine.STATE_COMPLETE )
			{
			    BDomain.this.shutdown();
			}
		    }
		});
	    this.sc.connectToKernel(eh.getPort(), csh);
	    switch( this.sc.waitForState(BDomainCoroutine.STATE_CONNECTED) )
	    {
	    case BDomainCoroutine.STATE_CONNECTED:
		break;
	    case BDomainCoroutine.STATE_ERROR:
		this.domain.destroy();
		break;
	    }
	    selector = csh.take(3).getString();
	    EventChannel.setCurrentChannel(); // XXX
	    try
	    {
		synchronized( this )
		{
		    this.wait(); // XXX Check the state
		}
	    }
	    catch(InterruptedException e)
	    {
	    }
	}
	else
	{
	    selector = "bees.core.kernel.BeesKernel";
	    this.sc.initKernel(eh.getPort());
	}
	try
	{
	    try
	    {
		Logger.setCurrent((Logger)this.capabilities.get("logger"));
	    }
	    catch(NoSuchElementException e)
	    {
	    }
	    this.selectedDevelopment(selector, args);
	}
	catch(ClassNotFoundException e)
	{
	    Logger.log(e);
	}
	catch(NoSuchMethodException e)
	{
	    e.printStackTrace();
	}
	catch(BDomainDeathException e)
	{
	    e.printStackTrace();
	}
	catch(Throwable e)
	{
	    e.printStackTrace();
	}
    }

    public String toString()
    {
	return "BDomain[domain="
	    + this.domain.getName()
	    + "; capabilities="
	    + this.capabilities
	    + "; cleanupHandlers="
	    + this.cleanupHandlers
	    + "]";
    }
}
