/*
 * CapabilityTree.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.bdomain;

import java.util.Vector;
import java.util.Enumeration;

import bees.core.Panic;
import bees.core.Assert;
import bees.core.ObjectMap;
import bees.core.Capability;
import bees.core.CapabilityEnvelope;

/**
 * A CapabilityTree is a helper class used to produce an array of
 * CapabilityEnvelopes from a tree of envelopes and other CapabilityTrees.
 */
public final class CapabilityTree
{
    /**
     * Flatten the given object map into a Vector of CapabilityEnvelopes.
     *
     * @param vec The vector to add the envelopes to or null if a new one
     * should be created.
     * @param prefix The prefix to add to any envelope names or null if the
     * envelopes should be anonymous.
     * @param om The ObjectMap to break down into separate envelopes.
     * @return "vec" or a new Vector containing CapabilityEnvelopes that
     * refer to the capabilities in "om".
     */
    public static Vector flattenMap(Vector vec, String prefix, ObjectMap om)
    {
	Vector retval;
	
	Assert.require(om);

	if( vec != null )
	{
	    retval = vec;
	}
	else
	{
	    retval = new Vector();
	}
	synchronized( om )
	{
	    Enumeration enum;
	    int lpc;
	    
	    enum = om.keys();
	    while( enum.hasMoreElements() )
	    {
		String name;
		Object key, obj;

		key = enum.nextElement();
		name = key.toString();
		obj = om.get(key);
		if( obj instanceof Capability )
		{
		    CapabilityEnvelope ce;

		    ce = new CapabilityEnvelope((prefix == null) ?
						null : prefix + "/" + name,
						(Capability)obj);
		    retval.addElement(ce);
		}
		else if( obj instanceof ObjectMap )
		{
		    flattenMap(retval, prefix + "/" + name, (ObjectMap)obj);
		}
		else
		{
		    throw new Panic("Bogus object in capability tree: "
				    + obj);
		}
	    }
	}
	return retval;
    }

    /**
     * Flatten the given object map into an array of CapabilityEnvelopes.
     *
     * @param prefix The prefix to add to any envelope names or null if the
     * envelopes should be anonymous.
     * @param om The ObjectMap to break down into separate envelopes.
     * @return A new array containing CapabilityEnvelopes that refer to the
     * capabilities in "om".
     */
    public static CapabilityEnvelope[] flattenMap(String prefix, ObjectMap om)
    {
	CapabilityEnvelope retval[];
	Vector vec;

	vec = flattenMap(null, prefix, om);
	retval = new CapabilityEnvelope[vec.size()];
	vec.copyInto(retval);
	return retval;
    }

    /**
     * The array of CapabilityEnvelopes that make up this node in the tree.
     */
    private final CapabilityEnvelope ce[];

    /**
     * The CapabilityTree's that are children of this node.
     */
    private final Vector children = new Vector();
    
    /**
     * Construct a CapabilityTree with the given values.
     *
     * @param ce The array of CapabilityEnvelopes that make up this node in the
     * tree.
     * @param children The CapabilityTree's that are children of this node.
     */
    public CapabilityTree(CapabilityEnvelope ce[], CapabilityTree children[])
    {
	int lpc;

	this.ce = ce;
	if( children != null )
	{
	    for( lpc = 0; lpc < children.length; lpc++ )
	    {
		if( children[lpc] != null )
		{
		    this.children.addElement(children[lpc]);
		}
	    }
	}
    }
    
    /**
     * Construct a CapabilityTree with the given value.
     *
     * @param children The CapabilityTree's that are children of this node.
     */
    public CapabilityTree(CapabilityTree children[])
    {
	this((CapabilityEnvelope[])null, children);
    }
    
    /**
     * Construct a CapabilityTree with the given value.
     *
     * @param ce The array of CapabilityEnvelopes that make up this node in the
     * tree.
     */
    public CapabilityTree(CapabilityEnvelope ce[])
    {
	this(ce, null);
    }
    
    /**
     * Construct a CapabilityTree with the given values.
     *
     * @param prefix The prefix to add to any envelope names or null if the
     * envelopes should be anonymous.
     * @param om The ObjectMap to break down into separate envelopes.
     * @param children The CapabilityTree's that are children of this node.
     */
    public CapabilityTree(String prefix,
			  ObjectMap om,
			  CapabilityTree children[])
    {
	this(flattenMap(prefix, om), children);
    }
    
    /**
     * Construct a CapabilityTree with the given values.
     *
     * @param prefix The prefix to add to any envelope names or null if the
     * envelopes should be anonymous.
     * @param om The ObjectMap to break down into separate envelopes.
     */
    public CapabilityTree(String prefix, ObjectMap om)
    {
	this(prefix, om, null);
    }
    
    /**
     * Construct a CapabilityTree with the given values.
     *
     * @param om The ObjectMap to break down into separate anonymous envelopes.
     */
    public CapabilityTree(ObjectMap om)
    {
	this(null, om, null);
    }

    /**
     * Add a child tree to this node.
     *
     * @param ct The child CapabilityTree to add to this node.
     * @return this
     */
    public synchronized CapabilityTree addChild(CapabilityTree ct)
    {
	this.children.addElement(ct);
	return this;
    }

    /**
     * @return The number of CapabilityEnvelopes in this tree.
     */
    public synchronized int size()
    {
	int lpc, size, retval = 0;

	if( this.ce != null )
	{
	    retval += this.ce.length;
	}
	size = this.children.size();
	for( lpc = 0; lpc < size; lpc++ )
	{
	    retval += ((CapabilityTree)this.children.elementAt(lpc)).size();
	}
	return retval;
    }

    /**
     * Flatten this tree and its children into the given array, starting at
     * the given position.
     *
     * @param dest The destination for any envelopes in this node.
     * @param destPos The starting point for adding envelopes to the "dest"
     * array.
     * @return The new starting position in the "dest" array for subsequent
     * additions to the array.
     */
    private synchronized int flatten(CapabilityEnvelope dest[], int destPos)
    {
	int lpc, size, retval = destPos;

	if( this.ce != null )
	{
	    System.arraycopy(this.ce,
			     0,
			     dest,
			     destPos,
			     this.ce.length);
	    retval += this.ce.length;
	}
	size = this.children.size();
	for( lpc = 0; lpc < size; lpc++ )
	{
	    CapabilityTree ct;

	    ct = (CapabilityTree)this.children.elementAt(lpc);
	    retval = ct.flatten(dest, retval);
	}
	return retval;
    }

    /**
     * @return The array of CapabilityEnvelopes that make up this tree.
     */
    public synchronized CapabilityEnvelope[] flatten()
    {
	CapabilityEnvelope retval[];

	retval = new CapabilityEnvelope[this.size()];
	this.flatten(retval, 0);
	return retval;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	String retval = "CapabilityTree[ce={";
	int lpc;

	for( lpc = 0; lpc < this.ce.length; lpc++ )
	{
	    retval += " " + this.ce;
	}
	retval += "}; children="
	    + this.children
	    + "]";
	return retval;
    }
}
