/*
 * BDomainRegisterCoroutine.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import java.util.Hashtable;

import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

import edu.utah.janosvm.kit.comm.amiga.FaxPortHandle;

import edu.utah.janos.nodeos.ChanStats;
import edu.utah.janos.nodeos.DomainStats;
import edu.utah.janos.nodeos.DomainHandle;

import bees.core.Strict;
import bees.core.BeesEvent;
import bees.core.Coroutine;

import bees.core.bdomain.DeadBDomainException;

/**
 * The BDomainRegisterCoroutine maintains the table of BDomain's in the system.
 */
public final class BDomainRegisterCoroutine
    extends Coroutine
{
    public static final int STATE_WAITING = STATE_USER + 0;
    public static final int STATE_ALIVE = STATE_USER + 1;

    private long peerID;

    private final Hashtable idToEvent = new Hashtable();
    
    public BDomainRegisterCoroutine()
    {
    }

    public synchronized BDomainQueryEvent takeQueryEvent(BDomainIdentifier id)
    {
	BDomainQueryEvent retval;

	retval = (BDomainQueryEvent)this.idToEvent.remove(id);
	return retval;
    }

    public synchronized void create(FaxPortHandle local, FaxPortHandle remote)
    {
	try
	{
	    BDomainRegisterEvent sre;

	    sre = new BDomainRegisterEvent(this,
					   BDomainRegisterEvent.METHOD_CREATE);
	    sre.setReplyPort(local);
	    remote.putFacsimile(sre);
	    this.setState(STATE_WAITING);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void queryStats(FaxPortHandle local,
					FaxPortHandle remote,
					BDomainIdentifier ai)
    {
	try
	{
	    BDomainQueryEvent aqe;

	    aqe = new BDomainQueryEvent(this, ai);
	    aqe.setReplyPort(local);
	    aqe.setServerCoroutineID(this.peerID);
	    remote.putFacsimile(aqe);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void deleteDomain(FaxPortHandle local,
					  FaxPortHandle remote,
					  BDomainIdentifier ai)
    {
	try
	{
	    BDomainEndingEvent aee;

	    aee = new BDomainEndingEvent(this, ai);
	    aee.setReplyPort(local);
	    aee.setServerCoroutineID(this.peerID);
	    remote.putFacsimile(aee);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void delete(FaxPortHandle local, FaxPortHandle remote)
    {
	try
	{
	    BDomainRegisterEvent sre;

	    sre = new BDomainRegisterEvent(this,
					   BDomainRegisterEvent.METHOD_DELETE);
	    sre.setServerCoroutineID(this.peerID);
	    sre.setReplyPort(local);
	    remote.putFacsimile(sre);
	    this.setState(STATE_WAITING);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void handle(BeesEvent ae)
    {
	BDomainRegister ar;
	
	switch( this.getState() )
	{
	case STATE_NEWBORN:
	    if( (ae instanceof BDomainRegisterEvent) && !ae.isReply() )
	    {
		BDomainRegisterEvent sre;

		sre = (BDomainRegisterEvent)ae;
		switch( sre.getMethod() )
		{
		case BDomainRegisterEvent.METHOD_CREATE:
		    try
		    {
			BDomainRegister.registry.set(new BDomainRegister());
		    }
		    catch(Throwable th)
		    {
			sre.setThrowable(th);
		    }
		    break;
		case BDomainRegisterEvent.METHOD_DELETE:
		    throw new ThreadDeath();
		}
		this.setState(STATE_ALIVE);
		try
		{
		    sre.sendReply();
		}
		catch(DeadBDomainException e)
		{
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_WAITING:
	    if( (ae instanceof BDomainRegisterEvent) && ae.isReply() )
	    {
		BDomainRegisterEvent sre;
		Throwable th;
		
		sre = (BDomainRegisterEvent)ae;
		if( (th = sre.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else
		{
		    this.peerID = sre.getServerCoroutineID();
		    this.setState(STATE_ALIVE);
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_ALIVE:
	    if( (ae instanceof BDomainQueryEvent) && !ae.isReply() )
	    {
		BDomainQueryEvent aqe;
		BDomainCoroutine ac;
		DomainHandle dh;

		ar = (BDomainRegister)BDomainRegister.registry.get();
		aqe = (BDomainQueryEvent)ae;
		try
		{
		    if( (ac = ar.get(aqe.getIdentifier())) != null )
		    {
			ac.fillEvent(aqe);
		    }
		    aqe.sendReply();
		}
		catch(Throwable th)
		{
		    th.printStackTrace();
		    aqe.setThrowable(th);
		    try
		    {
			aqe.sendReply();
		    }
		    catch(DeadBDomainException e)
		    {
		    }
		}
	    }
	    else if( (ae instanceof BDomainQueryEvent) && ae.isReply() )
	    {
		BDomainQueryEvent aqe;
		Throwable th;
		
		aqe = (BDomainQueryEvent)ae;
		if( (th = aqe.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else
		{
		    this.idToEvent.put(aqe.getIdentifier(), aqe);
		    this.notifyAll();
		}
	    }
	    else if( (ae instanceof BDomainEndingEvent) && !ae.isReply() )
	    {
		BDomainEndingEvent aee;
		BDomainCoroutine ac;

		ar = (BDomainRegister)BDomainRegister.registry.get();
		aee = (BDomainEndingEvent)ae;
		try
		{
		    if( (ac = ar.get(aee.getIdentifier())) != null )
		    {
			ac.shutdown();
		    }
		}
		catch(Throwable th)
		{
		    th.printStackTrace();
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	default:
	    throw new KernelPanic("Coroutine in invalid state: " + this);
	}
    }

    public String toString()
    {
	return "BDomainRegisterCoroutine["
	    + super.toString()
	    + "]";
    }
}
