/*
 * CreateBDomainCoroutine.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import java.util.Vector;
import java.util.Hashtable;

import edu.utah.janosvm.sys.DeadTeamException;
import edu.utah.janosvm.sys.TeamEjectionException;

import edu.utah.janosvm.kit.comm.amiga.FaxPort;
import edu.utah.janosvm.kit.comm.amiga.FaxPortHandle;

import edu.utah.janos.nodeos.MemSpec;
import edu.utah.janos.nodeos.ChanStats;
import edu.utah.janos.nodeos.DomainStats;
import edu.utah.janos.nodeos.DomainHandle;
import edu.utah.janos.nodeos.ThreadPoolSpec;
import edu.utah.janos.nodeos.DomainClassSpec;
import edu.utah.janos.nodeos.DomainStartException;

import bees.core.Assert;
import bees.core.Strict;
import bees.core.BeesEvent;
import bees.core.Coroutine;
import bees.core.Capability;
import bees.core.FlowIdentifier;
import bees.core.StateResponder;
import bees.core.ResponderCoroutine;
import bees.core.StateResponderList;
import bees.core.CapabilityReference;
import bees.core.MissingCapabilityException;

import bees.core.resource.Meter;
import bees.core.resource.Logger;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;

import bees.core.bdomain.EventHandler;
import bees.core.bdomain.DeadBDomainException;

/**
 * Coroutine used to create a new domain.
 */
public final class CreateBDomainCoroutine
    extends ResponderCoroutine
{
    /**
     * Constant used when the domain's main class arguments are null.
     */
    private static final String EMPTY_ARGS[] = new String[0];

    /**
     * State used when the coroutine is waiting for a reply message.
     */
    public static final int STATE_CREATING = STATE_USER + 0;
    public static final int STATE_ALIVE = STATE_USER + 1;

    /**
     * The object store identifier that contains the domain's byte array
     * identifier.
     */
    private long storeID = KernelObjectStore.INVALID_ID;

    private long peerID = Coroutine.INVALID_ID;

    private FaxPortHandle remote;

    private BDomainCoroutine ac;

    private BDomainQueryEvent aqe;

    /**
     * Construct an empty CreateBDomainCoroutine.
     */
    public CreateBDomainCoroutine()
    {
	super(null);
    }

    /**
     * Construct an empty CreateBDomainCoroutine.
     */
    public CreateBDomainCoroutine(StateResponder sr)
    {
	super(sr);
    }

    /**
     * @return The object store identifier that contains the domain's byte array
     * identifier.
     */
    public synchronized long getStoreID()
    {
	return this.storeID;
    }

    public synchronized BDomainQueryEvent takeQueryEvent()
    {
	BDomainQueryEvent retval;

	retval = this.aqe;
	this.aqe = null;
	return retval;
    }

    /**
     * Create a new domain with the given values.
     *
     * @param local The local fax port, used as the event's reply port.
     * @param remote The kernel fax port.
     * @param description An opaque string used as the description of the domain.
     * @param selector The domain's main class name.
     * @param args The arguments to the domain's main class.
     * @param capabilities The boot capabilities, needed to get the domain
     * started.
     */
    public synchronized void create(FaxPortHandle local,
				    FaxPortHandle remote,
				    String description,
				    String selector,
				    String args[],
				    Capability capabilities[],
				    PermissionSet meterSpec)
    {
	try
	{
	    final Vector meterVector = new Vector();
	    CapabilityReference meters[];
	    CreateBDomainEvent cse;
	    byte id[] = null;
	    int lpc;
	    
	    Assert.require(local);
	    Assert.require(remote);
	    Assert.require(description);
	    Assert.require(selector);
	    Assert.require(capabilities);

	    this.remote = remote;
	    if( args == null )
		args = EMPTY_ARGS;
	    for( lpc = 0; lpc < capabilities.length; lpc++ )
	    {
		Capability cap;

		/* XXX This is a bit goofy */
		if( (cap = capabilities[lpc]) != null )
		{
		    if( cap instanceof FlowIdentifier )
		    {
			id = ((FlowIdentifier)cap).getFlowID();
		    }
		    else if( cap instanceof Meter )
		    {
			meterVector.addElement(new CapabilityReference(cap));
		    }
		    else
		    {
			Logger.log("Unrecognized initial capability passed to "
				   + "CreateBDomainCoroutine.create: "
				   + cap);
		    }
		}
	    }
	    meters = new CapabilityReference[meterVector.size()];
	    meterVector.copyInto(meters);
	    cse = new CreateBDomainEvent(this,
					 id,
					 description,
					 selector,
					 args,
					 meters,
					 meterSpec);
	    cse.setReplyPort(local);
	    remote.putFacsimile(cse);
	    this.setState(STATE_CREATING);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }
    
    /**
     * Create a new domain with the given values.
     *
     * @param local The local fax port, used as the event's reply port.
     * @param remote The kernel fax port.
     * @param description An opaque string used as the description of the domain.
     * @param selector The domain's main class name.
     * @param args The arguments to the domain's main class.
     * @param capabilities The boot capabilities, needed to get the domain
     * started.
     */
    public synchronized void create(FaxPortHandle local,
				    FaxPortHandle remote,
				    String description,
				    String selector,
				    String args[],
				    Vector capabilities,
				    PermissionSet transfer)
    {
	Capability cap[];

	Assert.require(capabilities);

	cap = new Capability[capabilities.size()];
	capabilities.copyInto(cap);
	this.create(local,
		    remote,
		    description,
		    selector,
		    args,
		    cap,
		    transfer);
    }
    
    /**
     * Create a new domain with the given values.
     *
     * @param sc The current domain's coroutine.
     * @param description An opaque string used as the description of the domain.
     * @param selector The domain's main class name.
     * @param args The arguments to the domain's main class.
     * @param capabilities The boot capabilities, needed to get the domain
     * started.
     */
    public synchronized void create(BDomainCoroutine sc,
				    String description,
				    String selector,
				    String args[],
				    Vector capabilities,
				    PermissionSet transfer)
    {
	this.create(sc.getLocalPort(),
		    sc.getRemotePort(),
		    description,
		    selector,
		    args,
		    capabilities,
		    transfer);
    }

    /**
     * Create a new domain with the given values.
     *
     * @param local The local fax port, used as the event's reply port.
     * @param remote The kernel fax port.
     * @param description An opaque string used as the description of the domain.
     * @param selector The domain's main class name.
     * @param args The arguments to the domain's main class.
     * @param capabilities The boot capabilities, needed to get the domain
     * started.
     */
    public synchronized void create(BDomainCoroutine sc,
				    String description,
				    String selector,
				    String args[],
				    Capability capabilities[],
				    PermissionSet transfer)
    {
	this.create(sc.getLocalPort(),
		    sc.getRemotePort(),
		    description,
		    selector,
		    args,
		    capabilities,
		    transfer);
    }

    public synchronized void queryStats(FaxPortHandle local,
					FaxPortHandle remote)
    {
	try
	{
	    BDomainQueryEvent aqe;

	    aqe = new BDomainQueryEvent(this);
	    aqe.setReplyPort(local);
	    aqe.setServerCoroutineID(this.peerID);
	    remote.putFacsimile(aqe);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    synchronized void finished()
    {
	try
	{
	    BDomainEndingEvent see;

	    see = new BDomainEndingEvent(this);
	    see.setServerCoroutineID(this.peerID);
	    this.remote.putFacsimile(see);
	    this.setState(STATE_COMPLETE);
	}
	catch(DeadTeamException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(TeamEjectionException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }
    
    private void handleNewbornCreate(final CreateBDomainEvent cse)
	throws MissingCapabilityException
    {
	final BDomainIdentifier si;
	final BDomainRegister sr;
	final long storeID;
	byte id[];

	this.ac = new BDomainCoroutine();
	/*
	 * Construct the BDomainIdentifier using either the requested ID
	 * or the coroutine's unique identifier.
	 */
	if( (id = cse.getID()) != null )
	{
	    si = new BDomainIdentifier(id, cse.getDescription());
	}
	else
	{
	    si = new BDomainIdentifier(this.ac.getID(), cse.getDescription());
	}
	this.ac.setBDomainIdentifier(si);
	sr = (BDomainRegister)BDomainRegister.registry.get();
	synchronized( sr )
	{
	    if( sr.containsKey(si) )
	    {
		/*
		 * A domain with the requested ID already exists, grab
		 * it from the registrar and its storeID.
		 */
		this.ac = sr.get(si);
		storeID = this.ac.getStoreID();
	    }
	    else
	    {
		CapabilityReference cr[];
		Meter meters[];
		int lpc;
		
		cr = cse.getMeters();
		meters = new Meter[cr.length];
		for( lpc = 0; lpc < cr.length; lpc++ )
		{
		    meters[lpc] = (Meter)cr[lpc].dereference();
		}
		/* Hasn't been created yet, do so now. */
		storeID = KernelObjectStore.getPrimary().
		    put(si.getID(), null, si.getDescription().getString());
		this.ac.setStoreID(storeID);
		this.ac.setLocalPort(EventHandler.getHandlerPort());
		sr.put(this.ac);
		this.ac.setBDomainRegister(sr);
		this.ac.createDomain(cse.getSelector(),
				     cse.getArgs(),
				     meters,
				     cse.getMeterSpec());
	    }
	}
	this.ac.addCreator(this, new StateResponder(BDomainCoroutine.
						    STATE_CONNECTED) {
		public void transition(Coroutine coroutine, int newState)
		{
		    switch( newState )
		    {
		    case BDomainCoroutine.STATE_CONNECTED:
		    case BDomainCoroutine.STATE_COMPLETE:
			cse.setStoreID(storeID);
			break;
		    case BDomainCoroutine.STATE_ERROR:
			cse.setThrowable(coroutine.getThrowable());
			break;
		    default:
			throw new KernelPanic("Unhandled BDomainCoroutine state: "
					      + coroutine);
		    }
		    try
		    {
			cse.sendReply();
		    }
		    catch(DeadBDomainException e)
		    {
		    }
		}
	    });
    }

    public synchronized void handle(BeesEvent ae)
    {
	switch( this.getState() )
	{
	case STATE_NEWBORN:
	    if( (ae instanceof CreateBDomainEvent) && !ae.isReply() )
	    {
		final CreateBDomainEvent cse = (CreateBDomainEvent)ae;

		try
		{
		    this.peerID = cse.getClientCoroutineID();
		    this.remote = cse.getReplyPort();
		    this.handleNewbornCreate(cse);
		}
		catch(Throwable th)
		{
		    th.printStackTrace();
		    cse.setThrowable(th);
		    try
		    {
			cse.sendReply();
		    }
		    catch(DeadBDomainException e)
		    {
		    }
		}
		this.setState(STATE_ALIVE);
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_CREATING:
	    if( (ae instanceof CreateBDomainEvent) && ae.isReply() )
	    {
		CreateBDomainEvent cse = (CreateBDomainEvent)ae;
		Throwable th;

		if( (th = cse.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else if( (this.storeID = cse.getStoreID()) !=
			 KernelObjectStore.INVALID_ID )
		{
		    this.peerID = ae.getServerCoroutineID();
		    this.setState(STATE_ALIVE);
		}
		else
		{
		    throw new KernelPanic(
			"Received reply without a throwable or a valid "
			+ "object store identifier.");
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_ALIVE:
	    if( (ae instanceof BDomainQueryEvent) && !ae.isReply() )
	    {
		BDomainQueryEvent aqe;
		DomainHandle dh;

		aqe = (BDomainQueryEvent)ae;
		try
		{
		    this.ac.fillEvent(aqe);
		    aqe.sendReply();
		}
		catch(Throwable th)
		{
		    th.printStackTrace();
		    aqe.setThrowable(th);
		    try
		    {
			aqe.sendReply();
		    }
		    catch(DeadBDomainException e)
		    {
		    }
		}
	    }
	    else if( (ae instanceof BDomainQueryEvent) && ae.isReply() )
	    {
		BDomainQueryEvent aqe;
		Throwable th;
		
		aqe = (BDomainQueryEvent)ae;
		if( (th = aqe.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else
		{
		    this.aqe = aqe;
		    this.notifyAll();
		}
	    }
	    else if( (ae instanceof BDomainEndingEvent) && !ae.isReply() )
	    {
		this.setState(STATE_COMPLETE);
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	default:
	    throw new KernelPanic("Invalid state in coroutine: " + this);
	}
    }

    public String toString()
    {
	return "CreateBDomainCoroutine["
	    + super.toString()
	    + "; storeID="
	    + this.storeID
	    + "]";
    }
}
