/*
 * JSICoroutine.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerRoot;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.JSIEncodingException;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import edu.utah.janos.nodeos.Domain;

import bees.core.Assert;
import bees.core.Strict;
import bees.core.BeesEvent;
import bees.core.ByteArray;
import bees.core.Coroutine;
import bees.core.EventChannel;

import bees.core.bdomain.BDomain;
import bees.core.bdomain.BDomainLocal;
import bees.core.bdomain.DeadBDomainException;

/**
 * Coroutine used to pass a JSI message to a domain.
 */
public final class JSICoroutine
    extends Coroutine
{
    static  {
	EventChannel.sub.addHandler("user", new JSIHandler() {
		public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
		{
		    return this;
		}
		
		public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
		{
		    JSICoroutine jc;

		    jc = new JSICoroutine();
		    jc.send(EventChannel.threadChannel(), jm);
		    if( jm.getWhat() == JSIMessage.SHUTDOWN )
		    {
			jm.clear();
		    }
		    else
		    {
			switch( jc.waitForState(Coroutine.STATE_COMPLETE,
						30 * 1000) )
			{
			case Coroutine.STATE_COMPLETE:
			    jm.clear();
			    try
			    {
				jm.unflatten(jc.getReply());
			    }
			    catch(JSIEncodingException e)
			    {
				jm.error("Internal error");
			    }
			    break;
			case Coroutine.STATE_ERROR:
			    jm.error(jc.getThrowable().toString());
			    break;
			case Coroutine.STATE_ERROR_TIMEOUT:
			    jm.error("Timeout");
			    break;
			default:
			    Strict.fail();
			    break;
			}
		    }
		}

		public void getSuites(JSIMessage jm)
		{
		}
	    });
    }
    
    /**
     * The JSIHandlerRoot for each domain.
     */
    private static final BDomainLocal handlers = new BDomainLocal() {
	    protected Object initialValue()
	    {
		return new JSIHandlerContext(new JSIHandlerRoot(), null);
	    }
	};

    /**
     * Add a JSIHandler to the current domain's JSIHandlerRoot.
     *
     * @param name The name of the handler.
     * @param jh The JSIHandler to insert.
     */
    public static void addRootHandler(String name, JSIHandler jh)
    {
	JSIHandlerContext jhc;

	jhc = (JSIHandlerContext)handlers.get();
	jhc.getRootHandlers().addHandler(name, jh);
    }

    /**
     * State used when the request message was sent and the reply is expected.
     */
    public static final int STATE_SENT = STATE_USER;

    /**
     * The reply to the JSI message.
     */
    private byte reply[];

    /**
     * Construct an empty JSICoroutine.
     */
    public JSICoroutine()
    {
    }

    /**
     * @return The reply to the JSI message.
     */
    public synchronized byte[] getReply()
    {
	return this.reply;
    }

    /**
     * Send a JSIMessage to another domain.
     *
     * @param ec The EventChannel used to send the message.
     * @param jm The JSIMessage to pass to another domain.
     */
    public synchronized void send(EventChannel ec, JSIMessage jm)
    {
	try
	{
	    JSIEvent je;

	    Assert.require(ec);
	    Assert.require(jm);
	    
	    je = new JSIEvent(this, jm);
	    je.useBDomainReplyPort();
	    ec.sendEvent(je);
	    this.setState(STATE_SENT);
	}
	catch(DeadBDomainException e)
	{
	    this.setState(STATE_ERROR_CAGED);
	}
	catch(Throwable th)
	{
	    super.setThrowable(th);
	}
    }

    public synchronized void handle(BeesEvent ae)
    {
	switch( this.getState() )
	{
	case STATE_NEWBORN:
	    if( (ae instanceof JSIEvent) && !ae.isReply() )
	    {
		Domain domain = Domain.currentDomain();
		final JSIEvent je;

		je = (JSIEvent)ae;
		domain.threadStart(new Runnable() {
			public void run()
			{
			    try
			    {
				final JSIHandlerContext jhc;
				final JSIMessage jm;
				
				jm = je.getMessage();
				switch( jm.getWhat() )
				{
				case JSIMessage.SHUTDOWN:
				    if( jm.getRemainingSpecifierCount() == 0 )
				    {
					BDomain.currentBDomain().shutdown();
					throw new ThreadDeath();
				    }
				    /* FALLTHROUGH */
				default:
				    jhc = (JSIHandlerContext)handlers.get();
				    jhc.getRootHandlers().
					dispatchMessage(jhc, jm);
				    je.setMessage(jm);
				    break;
				}
			    }
			    catch(JSIEncodingException e)
			    {
				throw new KernelPanic(
					"Internally flattened JSIMessage "
					+ "couldn't be instantiated",
					e);
			    }
			    catch(ThreadDeath td)
			    {
				throw td;
			    }
			    catch(Throwable th)
			    {
				je.setThrowable(th);
			    }
			    finally
			    {
				JSICoroutine.this.setState(STATE_COMPLETE);
			    }
			    try
			    {
				je.sendReply();
			    }
			    catch(DeadBDomainException e)
			    {
			    }
			}
		    },
				 "JSI handler");
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	case STATE_SENT:
	    if( (ae instanceof JSIEvent) && ae.isReply() )
	    {
		Throwable th;
		JSIEvent je;

		je = (JSIEvent)ae;
		if( (th = je.getThrowable()) != null )
		{
		    super.setThrowable(th);
		}
		else if( (this.reply = je.getFlattenedMessage()) != null )
		{
		    this.setState(STATE_COMPLETE);
		}
		else
		{
		    throw new KernelPanic("Invalid reply message.");
		}
	    }
	    else
	    {
		Strict.fail();
	    }
	    break;
	default:
	    throw new KernelPanic("Invalid state for coroutine: " + this);
	}
    }
    
    public String toString()
    {
	return "JSICoroutine["
	    + super.toString()
	    + "; reply="
	    + ByteArray.toString(this.reply, 32)
	    + "]";
    }
}
