/*
 * JSIHandlerThread.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.Node;

import edu.utah.janosvm.kit.comm.jsi.JSIChannel;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerRoot;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.JSIEncodingException;

import bees.core.Assert;

/**
 * Runnable used to dispatch and reply to messages received over a given
 * JSIChannel.
 */
final class JSIHandlerThread
    implements Runnable
{
    /**
     * The JSIChannel that should be used for all communication.
     */
    private final JSIChannel jc;

    /**
     * Construct a JSIHandlerThread that will process the messages received
     * from the given JSIChannel.
     *
     * @param jc A connected JSIChannel.
     */
    JSIHandlerThread(JSIChannel jc)
    {
	Assert.require(jc);
	
	this.jc = jc;
    }

    /**
     * @return The JSIChannel that should be used for all communication.
     */
    JSIChannel getChannel()
    {
	return this.jc;
    }
    
    public void run()
    {
	JSIHandlerContext jhc;
	JSIHandlerRoot root;
	byte msg[];

	root = (JSIHandlerRoot)BeesKernel.getJSIRoot();
	jhc = new JSIHandlerContext(root, this.jc);
	try
	{
	    while( (msg = this.jc.recvMessage()) != null )
	    {
		JSIMessage jm;
		
		jm = JSIMessage.instantiate(msg);
		/* Act on the request... */
		switch( jm.getWhat() )
		{
		case JSIMessage.SHUTDOWN:
		    if( jm.getRemainingSpecifierCount() == 0 )
		    {
			Domain.currentDomain().destroy();
			throw new ThreadDeath();
		    }
		    /* FALLTHROUGH */
		default:
		    root.dispatchMessage(jhc, jm);
		    break;
		}
		/* Verify there is a reply */
		switch( jm.getWhat() )
		{
		case JSIMessage.SHUTDOWN:
		    continue;
		case JSIMessage.REPLY:
		case JSIMessage.ERROR:
		    break;
		default:
		    jm.error("Internal error: Handler didn't create a reply.");
		    break;
		}
		msg = jm.flatten();
		this.jc.replyMessage(msg);
	    }
	}
	catch(ThreadDeath td)
	{
	    throw td;
	}
	catch(Throwable th)
	{
	    Node.println("JSIHandlerThread died with exception:");
	    Node.printStackTrace(th);
	}
	finally
	{
	    this.jc.disconnect();
	}
    }

    public String toString()
    {
	return "JSIHandlerThread[jc="
	    + this.jc
	    + "]";
    }
}
