/*
 * KernelObject.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import bees.core.Assert;
import bees.core.TypeID;
import bees.core.ByteArray;
import bees.core.CapabilityEnvelope;

import bees.core.resource.Meter;

import bees.core.security.PermissionSet;

import bees.util.MD5;

/**
 * Basic object for storing the capability material and a comment describing
 * the capability.
 */
public final class KernelObject
{
    /**
     * The capability material.
     */
    private byte data[];

    private CapabilityEnvelope sub[];

    private PermissionSet ps;

    /**
     * An opaque string describing the capability.
     */
    private String comment;

    private final TypeID hash;

    private Meter meter;

    /**
     * Construct a KernelObject with the given values.
     *
     * @param data The capability material.
     * @param comment An opaque string describing the capability.
     */
    KernelObject(KernelObjectStore kos,
		 byte data[],
		 CapabilityEnvelope sub[],
		 PermissionSet ps,
		 String comment,
		 TypeID hash)
    {
	this.data = data;
	this.sub = sub;
	this.ps = ps;
	this.comment = comment;
	if( hash == null )
	{
	    MD5 digest;

	    digest = new MD5();
	    digest.update(this.data);
	    if( ps != null )
	    {
		ps.updateHash(digest);
	    }
	    digest.update(this.comment);
	    CapabilityEnvelope.updateHash(kos, sub, digest);
	    this.hash = new TypeID(digest.hash());
	}
	else
	{
	    this.hash = hash;
	}
    }

    KernelObject(KernelObjectStore kos,
		 byte data[],
		 CapabilityEnvelope sub[],
		 PermissionSet ps,
		 Meter meter,
		 String comment)
    {
	this(kos, data, sub, ps, comment, null);

	this.meter = meter;
    }

    KernelObject(TypeID hash)
    {
	this(null, null, null, null, null, hash);
    }

    public void setData(byte data[])
    {
	this.data = data;
    }
    
    /**
     * @return The capability material.
     */
    public byte[] getData()
    {
	return this.data;
    }
    
    public void setSubcapabilities(CapabilityEnvelope sub[])
    {
	this.sub = sub;
    }

    public CapabilityEnvelope[] getSubcapabilities()
    {
	return this.sub;
    }

    public void setPermissionSet(PermissionSet ps)
    {
	this.ps = ps;
    }

    public PermissionSet getPermissionSet()
    {
	return this.ps;
    }

    public void setComment(String comment)
    {
	this.comment = comment;
    }

    /**
     * @return An opaque string describing the capability.
     */
    public String getComment()
    {
	return this.comment;
    }

    public Meter getMeter()
    {
	return this.meter;
    }

    public TypeID getHash()
    {
	return this.hash;
    }

    public int computeSize()
    {
	int retval;

	retval = this.data.length;
	if( this.ps != null )
	{
	    retval += this.ps.marshalledLength();
	}
	return retval;
    }

    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof KernelObject )
	{
	    KernelObject ko = (KernelObject)obj;

	    if( (this.data.length == ko.data.length) &&
		this.ps.equals(ko.ps) &&
		this.comment.equals(ko.comment) )
	    {
		int lpc;

		retval = true;
		for( lpc = 0; (lpc < this.data.length) && retval; lpc++ )
		{
		    retval = (this.data[lpc] == ko.data[lpc]);
		}
	    }
	}
	return retval;
    }

    public String toString()
    {
	return "KernelObject[data="
	    + ByteArray.toString(this.data, 32)
	    + "; sub="
	    + this.sub
	    + "; ps="
	    + this.ps
	    + "; comment="
	    + this.comment
	    + "; hash="
	    + this.hash
	    + "]";
    }
}
