/*
 * KernelObjectEvent.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import bees.core.Assert;
import bees.core.ByteArray;
import bees.core.CloneFactory;
import bees.core.TwoWayBeesEvent;
import bees.core.CapabilityEnvelope;

import bees.core.security.PermissionSet;

import bees.core.bdomain.DeadBDomainException;

/**
 * Event used by the KernelObjectCoroutine.
 */
final class KernelObjectEvent
    extends TwoWayBeesEvent
{
    /**
     * The minimum valid method identifier.
     */
    static final int METHOD_MIN = 0;
    
    /**
     * Create an object.
     */
    static final int METHOD_CREATE = 0;
    
    /**
     * Lookup the value of an object.
     */
    static final int METHOD_LOOKUP = 1;
    
    /**
     * Delete an object.
     */
    static final int METHOD_DELETE = 2;
    
    /**
     * The maximum method identifier.
     */
    static final int METHOD_MAX = 3;

    /**
     * The event's method id.
     */
    private int method;

    private CapabilityEnvelope sub[];
    
    /**
     * The capability material.
     */
    private byte data[];

    private PermissionSet ps;

    /**
     * The comment to attach to the KernelObject.
     */
    private KernelString comment;

    /**
     * The KernelObject's unique identifier.
     */
    private long id;

    /**
     * Construct a KernelObjectEvent with the given values.
     *
     * @param koc The Coroutine thats using this event.
     * @param method The event's method id.
     * @param data The capability material.
     * @param comment The comment to attach to the KernelObject.
     * @param id The KernelObject's unique identifier.
     */
    KernelObjectEvent(KernelObjectCoroutine koc,
		      int method,
		      byte data[],
		      CapabilityEnvelope sub[],
		      PermissionSet ps,
		      String comment,
		      long id)
    {
	super(koc);

	Assert.require(method >= METHOD_MIN);
	Assert.require(method < METHOD_MAX);
	Assert.require((id == KernelObjectStore.INVALID_ID) || (id >= 0));
	
	this.method = method;
	this.data = data;
	this.sub = sub;
	this.ps = ps;
	if( comment != null )
	    this.comment = new KernelString(comment);
	this.id = id;
    }

    /**
     * @return The event's method id.
     */
    int getMethod()
    {
	return this.method;
    }

    /**
     * @param data The capability material.
     */
    void setData(byte data[])
    {
	this.data = data;
    }

    /**
     * @return The capability material.
     */
    byte[] getData()
    {
	return this.data;
    }

    CapabilityEnvelope[] getSubcapabilities()
    {
	return this.sub;
    }

    void setSubcapabilities(CapabilityEnvelope sub[])
    {
	this.sub = sub;
    }

    PermissionSet getPermissionSet()
    {
	return this.ps;
    }

    void setPermissionSet(PermissionSet ps)
    {
	this.ps = ps;
    }

    /**
     * @param comment The comment to attach to the KernelObject.
     */
    void setComment(String comment)
    {
	if( comment != null )
	    this.comment = new KernelString(comment);
    }

    /**
     * @param comment The comment to attach to the KernelObject.
     */
    void setComment(KernelString comment)
    {
	this.comment = comment;
    }

    /**
     * @return The comment to attach to the KernelObject.
     */
    KernelString getComment()
    {
	return this.comment;
    }

    /**
     * @param id The KernelObject's unique identifier.
     */
    void setID(long id)
    {
	this.id = id;
    }

    /**
     * @return The KernelObject's unique identifier.
     */
    long getID()
    {
	return this.id;
    }

    protected void sendReply()
	throws DeadBDomainException
    {
	super.sendReply();
    }
    
    public Object clone()
    {
	Object retval = super.clone();
	KernelObjectEvent koe;

	koe = (KernelObjectEvent)retval;
	if( this.data != null )
	{
	    koe.data = (byte[])this.data.clone();
	}
	if( this.ps != null )
	{
	    koe.ps = (PermissionSet)this.ps.clone();
	}
	if( this.comment != null )
	{
	    koe.comment = (KernelString)this.comment.clone();
	}
	if( this.sub != null )
	{
	    int lpc;

	    koe.sub = new CapabilityEnvelope[this.sub.length];
	    for( lpc = 0; lpc < this.sub.length; lpc++ )
	    {
		if( this.sub[lpc] != null )
		{
		    koe.sub[lpc] = (CapabilityEnvelope)this.sub[lpc].clone();
		}
	    }
	}
	return retval;
    }

    public String toString()
    {
	return "KernelObjectEvent["
	    + super.toString()
	    + "; method="
	    + this.method
	    + "; data="
	    + ByteArray.toString(this.data, 32)
	    + "; sub="
	    + this.sub
	    + "; ps="
	    + this.ps
	    + "; comment="
	    + this.comment
	    + "; id="
	    + this.id
	    + "]";
    }
}
