/*
 * KernelObjectStoreHandler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import java.util.Enumeration;
import java.util.NoSuchElementException;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import edu.utah.janosvm.kit.util.SparseArray;
import edu.utah.janosvm.kit.util.SparseArrayElement;
import edu.utah.janosvm.kit.util.ObjectSparseArrayElement;

/**
 * Handler class for the primary KernelObjectStore.
 */
public final class KernelObjectStoreHandler
    implements JSIHandler
{
    /**
     * The root level properties for this object.
     */
    private static final JSIPropertyInfo properties[] = {
	new JSIPropertyInfo("object",
			    (1 << JSIMessage.CREATE_PROPERTY),
			    "value:byte[] - The capability material.\n"
			    + "comment:string - An optional comment to attach "
			    + "to the object.\n"
			    + "\n"
			    + "Create a new kernel object.\n"),
	new JSIPropertyInfo("object",
			    (1 << JSIMessage.DELETE_PROPERTY),
			    "id:int",
			    "Delete the specified kernel object.\n"),
	new JSIPropertyInfo("object",
			    (1 << JSIMessage.GET_PROPERTY),
			    "id:int",
			    "Get the value of a kernel object.\n"),
	new JSIPropertyInfo("object",
			    (1 << JSIMessage.LIST_PROPERTIES),
			    "List the set of kernel objects.\n"),
    };

    /**
     * Construct a KernelObjectStoreHandler.
     */
    public KernelObjectStoreHandler()
    {
    }

    public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
    {
	return this;
    }
		
    public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	throws MissingPairException
    {
	JSIPair jp;
		    
	switch( jm.getWhat() )
	{
	case JSIMessage.CREATE_PROPERTY:
	    if( jm.popSpecifier() == null )
	    {
		String value, comment;
		byte parsedValue[];
		long id;
		int lpc;

		value = jm.expect("value");
		comment = jm.get("comment");
		parsedValue = new byte[value.length() / 2];
		for( lpc = 0; lpc < parsedValue.length; lpc++ )
		{
		    String sub;

		    sub = value.substring(lpc * 2, lpc * 2 + 2);
		    parsedValue[lpc] = (byte)Integer.parseInt(sub, 16);
		}
		id = KernelObjectStore.getPrimary().put(parsedValue,
							null,
							comment);
		jm.reply(Long.toString(id));
	    }
	    else
	    {
		jm.error("Too many specifiers");
	    }
	    break;
	case JSIMessage.DELETE_PROPERTY:
	    if( jm.popSpecifier() == null )
	    {
		long id;

		jm.restoreSpecifier();
		jp = jm.popSpecifier();
		id = jp.getIntValue();
		KernelObjectStore.getPrimary().remove(id);
		jm.reply("ok");
	    }
	    else
	    {
		jm.error("Too many specifiers");
	    }
	    break;
	case JSIMessage.GET_PROPERTY:
	    if( jm.popSpecifier() == null )
	    {
		KernelObject ko;
		long id;

		jm.restoreSpecifier();
		jp = jm.popSpecifier();
		id = jp.getIntValue();
		ko = KernelObjectStore.getPrimary().get(id);
		jm.clear();
		jm.setWhat(JSIMessage.REPLY);
		jm.add(new JSIPair("data", ko.getData()));
		jm.add("comment", ko.getComment());
	    }
	    else
	    {
		jm.error("Too many specifiers");
	    }
	    break;
	case JSIMessage.LIST_PROPERTIES:
	    if( jm.popSpecifier() == null )
	    {
		synchronized( KernelObjectStore.getPrimary() )
		{
		    Enumeration elem;
		    
		    elem = KernelObjectStore.getPrimary().elements();
		    jm.clear();
		    jm.setWhat(JSIMessage.REPLY);
		    while( elem.hasMoreElements() )
		    {
			ObjectSparseArrayElement sae;
			KernelObject ko;
			
			sae = (ObjectSparseArrayElement)elem.nextElement();
			ko = (KernelObject)sae.getValue();
			jm.add("id", Long.toString(sae.getID()));
			jm.add("comment", ko.getComment());
		    }
		}
	    }
	    else
	    {
		jm.error("Too many specifiers");
	    }
	    break;
	default:
	    throw new UnsupportedOperationException();
	}
    }

    public void getSuites(JSIMessage jm)
    {
	jm.add("result", properties);
    }

    public String toString()
    {
	return "KernelObjectStoreHandler["
	    + "]";
    }
}
