/*
 * KernelObjectStreamer.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.kernel;

import java.io.IOException;
import java.io.DataInputStream;
import java.io.DataOutputStream;

import bees.core.Xdr;
import bees.core.Assert;
import bees.core.TypeID;
import bees.core.XdrByteArray;
import bees.core.CapabilityFactory;
import bees.core.CapabilityEnvelope;

import bees.core.security.PermissionSet;

/**
 * Simple object used to write and read KernelObject's to/from a stream.
 */
public final class KernelObjectStreamer
{
    /**
     * Construct an empty KernelObjectStreamer.
     */
    public KernelObjectStreamer()
    {
    }

    /**
     * Read a KernelObject from the stream and put in the store.
     *
     * @param kos The KernelObjectStore to add the KernelObject to.
     * @param is The stream to read the KernelObject from.
     * @return The KernelObject's unique identifier.
     * @exception Throws an IOException if theres a problem while reading.
     */
    public long read(KernelObjectStore kos, DataInputStream is)
	throws ClassNotFoundException,
	       IOException
    {
	long retval = KernelObjectStore.INVALID_ID;
	byte object[], hash[];
	KernelObject ko;
	int lpc, length;
	String comment;

	Assert.require(kos);
	Assert.require(is);

	hash = new byte[Xdr.TYPEID];
	is.readFully(hash);
	ko = kos.get(retval = kos.putLate(new TypeID(hash)));
	object = new byte[is.readInt()];
	is.readFully(object);
	ko.setData(object);
	comment = is.readUTF();
	if( !comment.equals("") )
	{
	    ko.setComment(comment);
	}
	if( (length = is.readInt()) > 0 )
	{
	    XdrByteArray xba;

	    xba = new XdrByteArray(length);
	    is.readFully(xba.buf);
	    ko.setPermissionSet(PermissionSet.instantiate(xba));
	}
	length = is.readInt();
	for( lpc = 0; lpc < length; lpc++ )
	{
	    CapabilityEnvelope sub[];
	    
	    sub = new CapabilityEnvelope[length];
	    for( lpc = 0; lpc < length; lpc++ )
	    {
		sub[lpc] = CapabilityFactory.readEnvelope(kos, is);
	    }
	    ko.setSubcapabilities(sub);
	}
	return retval;
    }

    /**
     * Write a KernelObject to the stream.
     *
     * @param ko The KernelObject to write to the stream.
     * @param is The stream to write the KernelObject to.
     * @exception Throws an IOException if theres a problem while writing.
     */
    public void write(KernelObjectStore kos,
		      KernelObject ko,
		      DataOutputStream os)
	throws IOException
    {
	CapabilityEnvelope sub[];
	PermissionSet ps;
	
	Assert.require(ko);
	Assert.require(ko.getData());
	Assert.require(os);

	os.write(ko.getHash().cloneHash());
	os.writeInt(ko.getData().length);
	os.write(ko.getData());
	if( ko.getComment() == null )
	{
	    os.writeUTF("");
	}
	else
	{
	    os.writeUTF(ko.getComment());
	}
	if( (ps = ko.getPermissionSet()) != null )
	{
	    XdrByteArray xba;
	    
	    xba = new XdrByteArray(ps.marshalledLength());
	    ps.marshal(xba);
	    os.writeInt(xba.buf.length);
	    os.write(xba.buf);
	}
	else
	{
	    os.writeInt(0);
	}
	if( (sub = ko.getSubcapabilities()) != null )
	{
	    int lpc;

	    os.writeInt(sub.length);
	    for( lpc = 0; lpc < sub.length; lpc++ )
	    {
		CapabilityFactory.writeEnvelope(kos, os, sub[lpc]);
	    }
	}
	else
	{
	    os.writeInt(0);
	}
    }

    public String toString()
    {
	return "KernelObjectStreamer["
	    + "]";
    }
}
