/*
 * CapabilityHandler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;

import bees.core.Capability;
import bees.core.ObjectTree;
import bees.core.CapabilityFactory;

import bees.core.kernel.KernelCapabilities;

/**
 * CapabilityHandler is a base class that implements methods needed by
 * JSIHandler's that wish to create Capabilities.
 */
public abstract class CapabilityHandler
    implements JSIHandler
{
    /**
     * A cached pointer to the kernel's capability tree.
     */
    protected final ObjectTree tree = KernelCapabilities.getTree();

    /**
     * Construct an empty CapabilityHandler.
     */
    public CapabilityHandler()
    {
    }

    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#resolveSpecifier(JSIMessage, JSIPair)
     */
    public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
	throws Throwable
    {
	JSIHandler retval;

	if( jp.getValue().length == 0 )
	{
	    retval = this;
	}
	else
	{
	    Object obj;

	    obj = KernelCapabilities.getTree().get(jp.getStringValue());
	    if( obj instanceof JSIHandler )
	    {
		retval = (JSIHandler)obj;
		if( (jp = jm.popSpecifier()) != null )
		{
		    retval = retval.resolveSpecifier(jm, jp);
		}
	    }
	    else
	    {
		retval = this;
	    }
	}
	return retval;
    }

    /**
     * @return The Class corresponding to the type of capability constructed
     * by this handler.
     */
    public abstract Class getCapabilityClass();

    /**
     * Abstract method called when the handler needs to create a capability.
     *
     * @param jm The JSIMessage that contains any creation arguments.
     * @param name The name to use when adding the capability to the kernel
     * tree.
     * @param comment The comment to be attached to the capability.
     * @return The created capability or an ObjectMap if more than one was
     * created.
     * @throws Throwable if anything goes wrong.
     */
    public abstract Object createCapability(JSIMessage jm,
					    String name,
					    String comment)
	throws Throwable;
    
    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#handleMessage(JSIHandlerContext, JSIMessage)
     */
    public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	throws Throwable
    {
	JSIPair cap, jp;
	
	switch( jm.getWhat() )
	{
	case JSIMessage.CREATE_PROPERTY:
	    if( jm.popSpecifier() == null )
	    {
		String name, comment;
		Object rc;
		long id;
		
		name = jm.expect("name");
		comment = jm.get("comment",
				 this.getCapabilityClass().getName());
		if( (rc = this.createCapability(jm, name, comment)) != null )
		{
		    this.tree.put(name, rc);
		    if( rc instanceof Capability )
		    {
			jm.reply(CapabilityFactory.getID((Capability)rc));
		    }
		    else
		    {
			jm.reply("ok");
		    }
		}
		else if( (jm.getWhat() != JSIMessage.REPLY) ||
			 (jm.getWhat() != JSIMessage.ERROR) )
		{
		    jm.error("Internal error...");
		}
	    }
	    else
	    {
		jm.error("Too many specifiers");
	    }
	    break;
	case JSIMessage.GET_PROPERTY:
	    jm.restoreSpecifier();
	    cap = jm.popSpecifier();
	    if( (jp = jm.popSpecifier()) != null )
	    {
		if( jp.getTag().equals("type") )
		{
		    jm.reply(this.getCapabilityClass().getName());
		}
		else if( jp.getTag().equals("children") )
		{
		    KernelCapabilities.getChildren(jm,
						   cap,
						   this.getCapabilityClass());
		}
		else
		{
		    jm.error("Unknown property: " + jp.getTag());
		}
	    }
	    else
	    {
		jm.error("No property to get");
	    }
	    break;
	default:
	    throw new UnsupportedOperationException(
		"What: " +
		JSIMessage.MESSAGE_TYPE_STRINGS[jm.getWhat()]);
	}
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CapabilityHandler["
	    + "]";
    }
}
