/*
 * Clock.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;

import bees.core.Capability;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.IntegerPermission;
import bees.core.security.PermissionException;
import bees.core.security.MinimumIntegerPermission;

/**
 * The clock capability is used to read/write the system clock.  The capability
 * will also behave as an up time clock if the user doesn't have the permission
 * to read the real time.
 */
public final class Clock
    extends Capability
    implements JSIHandler
{
    /**
     * JSI Property info.
     */
    private static final JSIPropertyInfo JSI_PROPERTIES[] = {
	new JSIPropertyInfo("current",
			    (1 << JSIMessage.GET_PROPERTY),
			    "Get the current time from this clock.\n"),
    };
    
    /**
     * Permission to read the local time.
     */
    public static final Permission READ_LOCAL_TIME =
	new Permission("local-time", "read");

    /**
     * Permission to write the local time.
     */
    public static final Permission WRITE_LOCAL_TIME =
	new Permission("local-time", "write");

    /**
     * Permission to write the local time.
     */
    public static final Permission GRANULARITY =
	new Permission("granularity", "read");

    /**
     * Default permissions.
     */
    public static final PermissionSet DEFAULT_PERMISSIONS =
	new PermissionSet(new Permission[] {
	    new Permission("local-time", "read, write"),
	    new MinimumIntegerPermission("granularity", 1),
	});

    /**
     * If the capability doesn't have the READ_LOCAL_TIME permission, this is
     * set to the current time at the initialization of the object and used
     * as the zero point for any time values returned to the user.
     *
     * XXX May not behave correctly during a time change.
     */
    private long startTime;

    /**
     * The clock granularity.
     */
    private long granularity = 0;

    /**
     * Construct an empty Clock capability.
     */
    public Clock()
    {
    }

    /**
     * @see bees.core.Capability#fromBytes(byte[])
     */
    protected synchronized void fromBytes(byte flattened[])
	throws Exception
    {
	super.fromBytes(flattened);

	IntegerPermission ip;

	try
	{
	    super.checkPermission(READ_LOCAL_TIME);
	    this.startTime = 0;
	}
	catch(PermissionException e)
	{
	    this.startTime = System.currentTimeMillis();
	}
	ip = (IntegerPermission)super.clonePermission(GRANULARITY);
	this.granularity = ip.getValue();
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();

	this.granularity = 0;
    }

    /**
     * @return The time at initialization of this object or zero if this
     * capability has the READ_LOCAL_TIME permission.
     */
    synchronized long getStartTime()
    {
	if( super.valid() )
	{
	    return this.startTime;
	}
	else
	{
	    return -1;
	}
    }

    /**
     * @return The clock granularity.
     */
    public synchronized long getGranularity()
    {
	if( super.valid() )
	{
	    return this.granularity;
	}
	else
	{
	    return -1;
	}
    }

    /**
     * @return The uptime for this object or the current time, if this
     * capability has the appropriate permissions.
     */
    public long current()
    {
	long retval = -1;

	if( super.id != INVALID_ID )
	{
	    retval = System.currentTimeMillis() - this.startTime;
	    retval -= (retval % this.granularity);
	}
	return retval;
    }

    /**
     * Compare a time against the current time to determine if it is or isn't
     * in the future.
     *
     * @param time The time value to test.
     * @return True if "time" is in the past, false if it is in the future.
     */
    public synchronized boolean expired(long time)
    {
	return (this.current() >= time);
    }

    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#resolveSpecifier(JSIMessage, JSIPair)
     */
    public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
    {
	return this;
    }
    
    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#handleMessage(JSIHandlerContext, JSIMessage)
     */
    public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
    {
	JSIPair jp;

	switch( jm.getWhat() )
	{
	case JSIMessage.GET_PROPERTY:
	    if( (jp = jm.popSpecifier()) != null )
	    {
		if( jp.getTag().equals("current") )
		{
		    jm.reply(this.current());
		}
		else
		{
		    jm.error("Unknown Clock property: " + jp.getTag());
		}
	    }
	    else
	    {
		jm.reply(this.toString());
	    }
	    break;
	case JSIMessage.GET_SUITES:
	    jm.clear();
	    jm.setWhat(JSIMessage.REPLY);
	    this.getSuites(jm);
	    break;
	default:
	    throw new UnsupportedOperationException();
	}
    }
    
    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#getSuites(JSIMessage)
     */
    public void getSuites(JSIMessage jm)
    {
	jm.add("result", JSI_PROPERTIES);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Clock["
	    + super.toString()
	    + "; granularity="
	    + this.granularity
	    + "]";
    }
}
