/*
 * Console.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import java.io.InputStream;
import java.io.OutputStream;

import bees.core.Capability;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;

/**
 * The Console capability provides access to standard I/O.
 */
public final class Console
    extends Capability
{
    /**
     * Permission to read from the console.
     */
    public static final Permission CONSOLE_READ_PERMISSION =
	new Permission("console", "read");

    /**
     * Permission to write to the console.
     */
    public static final Permission CONSOLE_WRITE_PERMISSION =
	new Permission("console", "write");

    /**
     * Default permissions for a console.
     */
    public static final PermissionSet DEFAULT_PERMISSIONS =
	new PermissionSet(new Permission[] {
	    new Permission("console", "read, write"),
	});

    /**
     * The OutputStream wrapping System.out.
     */
    private RevokableOutputStream ros;
    
    /**
     * The InputStream wrapping System.in.
     */
    private RevokableInputStream ris;

    /**
     * Construct an empty Console object.
     */
    public Console()
    {
    }

    /**
     * @see bees.core.Capability#fromBytes(byte[])
     */
    protected synchronized void fromBytes(byte flattened[])
    {
	this.ros = new RevokableOutputStream(System.out);
	this.ris = new RevokableInputStream(System.in);
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();

	this.ros.revoke();
	this.ros = null;
	this.ris.revoke();
	this.ris = null;
    }

    /**
     * @return The console OutputStream.
     */
    public synchronized OutputStream getOutputStream()
    {
	super.checkPermission(CONSOLE_WRITE_PERMISSION);
	
	return this.ros;
    }

    /**
     * @return The console InputStream.
     */
    public synchronized InputStream getInputStream()
    {
	super.checkPermission(CONSOLE_READ_PERMISSION);
	
	return this.ris;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Console["
	    + super.toString()
	    + "; ros="
	    + this.ros
	    + "; ris="
	    + this.ris
	    + "]";
    }
}
