/*
 * Directory.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import java.io.IOException;
import java.io.FilenameFilter;
import java.io.FileNotFoundException;

import bees.core.Panic;
import bees.core.Assert;
import bees.core.CapabilityEnvelope;

import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.FilePermission;
import bees.core.security.PermissionException;

import bees.core.bdomain.BDomain;

/**
 * The Directory capability is used to designate a directory relative to a
 * FileSystemRoot.
 */
public final class Directory
    extends PathSpec
{
    /**
     * The string used to prefix file permission names.
     */
    public static final String FILE_PREFIX = "file.";

    /**
     * Permission for creating files.
     */
    public static final FilePermission FILE_CREATE_PERMISSION =
	new FilePermission("file.", "create");
    
    /**
     * Permission for deleting files.
     */
    public static final FilePermission FILE_DELETE_PERMISSION =
	new FilePermission("file.", "delete");
    
    /**
     * Permission for opening files.
     */
    public static final FilePermission FILE_EXECUTE_PERMISSION =
	new FilePermission("file.", "execute");

    /**
     * Permission for listing files.
     */
    public static final FilePermission FILE_LIST_PERMISSION =
	new FilePermission("file.", "list");

    /**
     * The string used to prefix directory permission names.
     */
    public static final String DIRECTORY_PREFIX = "dir.";
    
    /**
     * Permission for creating directories.
     */
    public static final FilePermission DIRECTORY_CREATE_PERMISSION =
	new FilePermission("dir.", "create");
    
    /**
     * Permission for deleting directories.
     */
    public static final FilePermission DIRECTORY_DELETE_PERMISSION =
	new FilePermission("dir.", "delete");
    
    /**
     * Permission for opening directories.
     */
    public static final FilePermission DIRECTORY_EXECUTE_PERMISSION =
	new FilePermission("dir.", "execute");

    /**
     * Permission for listing directories.
     */
    public static final FilePermission DIRECTORY_LIST_PERMISSION =
	new FilePermission("dir.", "list");

    /**
     * The default permissions given to new Directory capabilities.
     */
    public static final PermissionSet DEFAULT_PERMISSIONS =
	new PermissionSet(new Permission[] {
	    new FilePermission("file.", "create, delete, execute, list"),
	    new FilePermission("dir.", "create, delete, execute, list"),
	});

    /**
     * A more restricted set of permissions for Directory capabilities.
     */
    public static final PermissionSet DEFAULT_RESTRICTED_PERMISSIONS =
	new PermissionSet(new Permission[] {
	    new FilePermission("file.", "list"),
	    new FilePermission("dir.", "list"),
	});

    /**
     * The FileSystemRoot that this directory is contained within.
     */
    private FileSystemRoot root;

    /**
     * The full path to the directory on the local file system.
     */
    private java.io.File directory;

    /**
     * Construct an empty Directory object.
     */
    public Directory()
    {
    }

    /**
     * @see bees.core.Capability#fromBytes(byte[], CapabilityEnvelope[])
     */
    protected synchronized void fromBytes(byte flattened[],
					  CapabilityEnvelope sub[])
	throws Exception
    {
	super.fromBytes(flattened);
	super.specFromBytes(flattened);
	
	this.root = (FileSystemRoot)sub[0].unseal();
	this.directory = new java.io.File(this.root.getFile(),
					  super.getPath());

	if( !this.directory.isDirectory() )
	{
	    throw new IllegalArgumentException("Not a directory: "
					       + super.getPath());
	}
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.revoke();
	
	this.root = null;
	this.directory = null;
    }

    /**
     * @return A list of files/directories contained within this directory.
     * Note: If a file doesn't have a "list" permission it will not in the
     * returned array.
     */
    public synchronized String[] list(/* XXX final FilenameFilter fnf */)
    {
	try
	{
	    return this.directory.list(new FilenameFilter() {
		    public boolean accept(java.io.File dir, String name)
		    {
			boolean retval = false;
			java.io.File file;

			file = new java.io.File(dir, name);
			try
			{
			    if( file.isDirectory() )
			    {
				Directory.this.checkPathPermission(
					DIRECTORY_PREFIX + name,
					DIRECTORY_LIST_PERMISSION);
				retval = true;
			    }
			    else if( file.isFile() )
			    {
				Directory.this.checkPathPermission(
					FILE_PREFIX + name,
					FILE_LIST_PERMISSION);
				retval = true;
			    }
			}
			catch(PermissionException e)
			{
			    /* Ignore */
			}
			return retval;
		    }
		});
	}
	catch(NullPointerException e)
	{
	    return null;
	}
    }

    /**
     * Check for permissions on a given file/directory.
     *
     * @param name The name of the file/directory with the appropriate prefix.
     * @param what The permission to check for.
     * @throws PermissionException if the capability doesn't contain the given
     * permission.
     */
    private void checkPathPermission(String name, Permission what)
    	throws PermissionException
    {
	try
	{
	    super.checkPermission(new Permission(name, what.getAction()));
	}
	catch(PermissionException e)
	{
	    super.checkPermission(what);
	}
    }

    /**
     * Create/lookup a path name in this directory.
     *
     * @param name The name of the file/directory.
     * @param type Null or a File or a Directory object to be initialized with
     * the new capability.
     * @param ps The permissions to give to the new capability.
     * @param create True if the file/directory should be created.
     * @return The File/Directory object if the operation was a success.
     * @throws IOException if there is a problem accessing the given path.
     */
    private synchronized PathSpec path(String name,
				       PathSpec type,
				       PermissionSet ps,
				       boolean create)
	throws IOException
    {
	java.io.File file;
	PathSpec retval;
	String path;
	
	verifyFilePath(name);

	file = new java.io.File(this.directory, name);
	path = file.getCanonicalPath();
	path = path.substring(this.directory.getCanonicalPath().length());
	if( create )
	{
	    boolean success;
	    
	    Assert.require(type);

	    if( type instanceof File )
	    {
		this.checkPathPermission(FILE_PREFIX + name,
					 FILE_CREATE_PERMISSION);
		success = file.createNewFile();
	    }
	    else if( type instanceof Directory )
	    {
		this.checkPathPermission(DIRECTORY_PREFIX + name,
					 DIRECTORY_CREATE_PERMISSION);
		success = file.mkdir();
	    }
	    else
	    {
		throw new Panic("Unknown PathSpec type");
	    }
	}
	if( file.isDirectory() )
	{
	    try
	    {
		this.checkPathPermission(DIRECTORY_PREFIX + name,
					 DIRECTORY_EXECUTE_PERMISSION);
	    }
	    catch(PermissionException e)
	    {
		if( create )
		{
		    return null;
		}
		throw e;
	    }
	    if( type == null )
	    {
		retval = new Directory();
		ps = DEFAULT_PERMISSIONS;
	    }
	    else if( type instanceof Directory )
	    {
		retval = type;
	    }
	    else
	    {
		throw new FileNotFoundException("Not a directory: " + name);
	    }
	}
	else if( file.isFile() )
	{
	    try
	    {
		this.checkPathPermission(FILE_PREFIX + name,
					 FILE_EXECUTE_PERMISSION);
	    }
	    catch(PermissionException e)
	    {
		if( create )
		{
		    return null;
		}
		throw e;
	    }
	    if( type == null )
	    {
		retval = new File();
		ps = File.DEFAULT_PERMISSIONS;
	    }
	    else if( type instanceof File )
	    {
		retval = type;
	    }
	    else
	    {
		throw new FileNotFoundException("Not a file: " + name);
	    }
	}
	else
	{
	    throw new FileNotFoundException(file.toString());
	}
	retval = (PathSpec)KernelObjectCoroutine.
	    create(retval,
		   BDomain.currentBDomain().getBDomainCoroutine(),
		   marshal(path),
		   new CapabilityEnvelope[] {
		       new CapabilityEnvelope(this.root),
		   },
		   name,
		   ps);
	return retval;
    }

    /**
     * Lookup an object contained within this directory.
     * 
     * @param name The name of the file or directory to lookup.
     * @return If the path resolves to a directory, then a Directory capability
     * is returned, otherwise a File is returned.
     * @throws IOException if the file could not be found or there was another
     * problem.
     */
    public PathSpec lookup(String name)
	throws IOException
    {
	return this.path(name, null, null, false);
    }

    /**
     * Reference a directory or create a new one contained within this
     * directory.
     *
     * @param name The name of the directory.
     * @param ps The set of permissions to give the Directory capability.
     * @param create True if the directory should be created, false otherwise.
     * @return A new Directory capability.
     * @throws IOException if the directory could not be found or if there was
     * a problem creating the directory.
     */
    public Directory directory(String name, PermissionSet ps, boolean create)
	throws IOException
    {
	return (Directory)this.path(name, new Directory(), ps, create);
    }
    
    /**
     * Reference a directory contained within this directory.
     *
     * @param name The name of the directory.
     * @param ps The set of permissions to give the Directory capability.
     * @return A new Directory capability.
     * @throws IOException if the directory could not be found.
     */
    public Directory directory(String name, PermissionSet ps)
	throws IOException
    {
	return (Directory)this.path(name, new Directory(), ps, false);
    }
    
    /**
     * Reference a directory contained within this directory
     *
     * @param name The name of the directory.
     * @return A new Directory capability with the default set of permissions.
     * @throws IOException if the directory could not be found.
     */
    public Directory directory(String name)
	throws IOException
    {
	return this.directory(name, DEFAULT_PERMISSIONS);
    }
    
    /**
     * Reference a file or create a new one contained within this directory.
     *
     * @param name The name of the file.
     * @param ps The set of permissions to give the file capability.
     * @param create True if the file should be created, false otherwise.
     * @return A new File capability.
     * @throws IOException if the file could not be found or if there was
     * a problem creating the file.
     */
    public File file(String name, PermissionSet ps, boolean create)
	throws IOException
    {
	return (File)this.path(name,
			       new File(),
			       ps,
			       create);
    }

    /**
     * Reference a file contained within this directory.
     *
     * @param name The name of the file.
     * @param ps The set of permissions to give the file capability.
     * @return A new File capability.
     * @throws IOException if the file could not be found.
     */
    public File file(String name, PermissionSet ps)
	throws IOException
    {
	return (File)this.path(name,
			       new File(),
			       ps,
			       false);
    }

    /**
     * Reference a file contained within this directory.
     *
     * @param name The name of the file.
     * @return A new File capability with the default set of permissions.
     * @throws IOException if the file could not be found.
     */
    public File file(String name)
	throws IOException
    {
	return this.file(name, File.DEFAULT_PERMISSIONS);
    }

    /**
     * Delete a file/directory contained within this directory.
     *
     * @param name The name of the file/directory to delete.
     */
    public void delete(String name)
    {
	java.io.File file;
	
	Assert.require(name);
	
	verifyFilePath(name);

	file = new java.io.File(this.directory, name);
	if( file.isDirectory() )
	{
	    this.checkPathPermission(DIRECTORY_PREFIX + name,
				     DIRECTORY_DELETE_PERMISSION);
	}
	else if( file.isFile() )
	{
	    this.checkPathPermission(FILE_PREFIX + name,
				     FILE_DELETE_PERMISSION);
	}
	else
	{
	    this.checkPathPermission(FILE_PREFIX + name,
				     FILE_DELETE_PERMISSION);
	}
	file.delete();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "Directory["
	    + super.toString()
	    + "; root="
	    + this.root
	    + "; directory="
	    + this.directory
	    + "]";
    }
}
