/*
 * FileSystemRoot.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import bees.core.CapabilityEnvelope;

import bees.core.kernel.BDomainCoroutine;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.PermissionSet;

import bees.core.bdomain.BDomain;

/**
 * The FileSystemRoot capability is used to designate a "root" in the local
 * file system.
 */
public final class FileSystemRoot
    extends PathSpec
{
    /**
     * The starting point for this FileSystemRoot.
     */
    private java.io.File root;

    /**
     * Construct an empty FileSystemRoot.
     */
    public FileSystemRoot()
    {
    }

    /**
     * @see bees.core.Capability#fromBytes(byte[])
     */
    protected synchronized void fromBytes(byte flattened[])
	throws Exception
    {
	super.fromBytes(flattened);
	super.specFromBytes(flattened);

	this.root = new java.io.File(super.getPath());

	if( !this.root.isDirectory() )
	{
	    throw new IllegalArgumentException(
		"File system root isn't a directory: "
		+ this.root);
	}
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();
	
	this.root = null;
    }

    /**
     * @return The starting point for this FileSystemRoot.
     */
    java.io.File getFile()
    {
	return this.root;
    }

    /**
     * Create a Directory capability that refers to the root directory in
     * this "file system".
     *
     * @param ps The set of permissions to give the new Directory object.
     * @return The new Directory capability or null if this capability has been
     * revoked.
     */
    public synchronized Directory root(PermissionSet ps)
    {
	Directory retval = null;
	
	if( this.root != null )
	{
	    BDomainCoroutine sc;
	    
	    sc = BDomain.currentBDomain().getBDomainCoroutine();
	    retval = (Directory)KernelObjectCoroutine.
		create(new Directory(),
		       sc,
		       marshal(""),
		       new CapabilityEnvelope[] {
			   new CapabilityEnvelope(this),
		       },
		       "Root directory",
		       ps);
	}
	return retval;
    }

    /**
     * Create a Directory capability, with the default permissions, that refers
     * to the root directory in this "file system".
     *
     * @return The new Directory capability or null if this capability has been
     * revoked.
     */
    public Directory root()
    {
	return this.root(Directory.DEFAULT_PERMISSIONS);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "FileSystemRoot["
	    + super.toString()
	    + "; root="
	    + this.root
	    + "]";
    }
}
