/*
 * LoggerHandler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import edu.utah.janos.nodeos.AddressFormatException;

import bees.core.Capability;
import bees.core.CapabilityEnvelope;

import bees.core.kernel.KernelCapabilities;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.EnumerationException;
import bees.core.security.EnumerationPermission;

import bees.core.bdomain.BDomain;

/**
 * The LoggerHandler is a JSIHandler that can create Logger capabilities.
 */
public final class LoggerHandler
    extends CapabilityHandler
{
    /**
     * The set of supported properties to be added to a JSIMessage.GET_SUITES
     * request.
     */
    private static final JSIPropertyInfo JSI_PROPERTIES[] = {
	new JSIPropertyInfo("logger",
			    (1 << JSIMessage.CREATE_PROPERTY),
			    "name:string - The capability name.\n"
			    + "prefix:string - The logger's prefix.\n"
			    + "output:string - The output capability.  "
			    + "(Default: console).\n"
			    + "level:enum - The maximum level of "
			    + "information to print.  Possible values: "
			    + "emergency, alert, critical, error, info, "
			    + "warning, notice, debug.  (Default: debug)\n"
			    + "comment:string - An optional comment for the "
			    + "capability.\n"
			    + "\n"
			    + "Create a logger capability.\n"),
	new JSIPropertyInfo("logger",
			    (1 << JSIMessage.GET_PROPERTY),
			    "name:string",
			    "Get the value of a Logger capability.\n"),
    };
    
    /**
     * Construct an empty LoggerHandler object.
     */
    public LoggerHandler()
    {
    }
    
    /**
     * @see bees.core.resource.CapabilityHandler#getCapabilityClass()
     */
    public Class getCapabilityClass()
    {
	return Logger.class;
    }

    /**
     * @see bees.core.resource.CapabilityHandler#createCapability(JSIMessage, String, String)
     */
    public Object createCapability(JSIMessage jm,
				   String name,
				   String comment)
	throws AddressFormatException,
	       EnumerationException,
	       MissingPairException
    {
	int flags = LoggerSpec.LF_BDOMAIN_PREFIX;
	String prefix, output, sFlags;
	EnumerationPermission level;
	JSIPair jp;
	
	prefix = jm.expect("prefix");
	output = jm.get("output", "console");
	sFlags = jm.get("bdomainPrefix", "yes");
	if( sFlags.startsWith("n") || sFlags.startsWith("f") )
	{
	    flags &= ~LoggerSpec.LF_BDOMAIN_PREFIX;
	}
	level = new EnumerationPermission("level", jm.get("level", "debug"));
	jm.add("maxIntPermission",
	       "level=write; " + level.mapValue(Logger.LEVEL_TYPES));
	return (Capability)KernelObjectCoroutine.
	    create(new Logger(),
		   BDomain.currentBDomain().getBDomainCoroutine(),
		   LoggerSpec.marshal(prefix, flags),
		   new CapabilityEnvelope[] {
		       new CapabilityEnvelope((Capability)
					      super.tree.get(output)),
		   },
		   comment,
		   KernelCapabilities.toPermissionSet(jm));
    }

    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#getSuites(JSIMessage)
     */
    public void getSuites(JSIMessage jm)
    {
	jm.add("result", JSI_PROPERTIES);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LoggerHandler["
	    + super.toString()
	    + "]";
    }
}
