/*
 * LoggerSpec.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import edu.utah.janos.io.Log;
import edu.utah.janos.io.LogLevel;

import bees.core.Xdr;
import bees.core.Capability;
import bees.core.XdrByteArray;
import bees.core.CapabilitySpec;
import bees.core.SpecCapabilityFactory;

/**
 * A LoggerSpec captures the capability material for a Logger capability.
 */
public class LoggerSpec
    extends Capability
    implements LogLevel, CapabilitySpec
{
    /**
     * Flag that specifies whether or not the Logger should prefix any messages
     * with the BDomain's identifier.
     */
    public static final int LF_BDOMAIN_PREFIX	= (1 << 0);
    
    /**
     * Marshal a prefix string and flag value into a byte array.
     *
     * @param prefix The Logger prefix string.
     * @param flags Logging flags.
     * @return A byte array containing the flattened versions of the arguments.
     */
    static byte[] marshal(String prefix, int flags)
    {
	XdrByteArray xba;

	xba = new XdrByteArray(Xdr.STRING(prefix) + Xdr.INT);
	xba.PUT(prefix);
	xba.PUT(flags);
	return xba.toBytes();
    }

    /**
     * Represents an invalid LogLevel value.
     */
    public static final int INVALID_LEVEL = -1;

    /**
     * The Java NodeOS log object to be used when logging data.
     */
    Log log;

    /**
     * Logging flags.
     */
    int flags;

    /**
     * Construct a LoggerSpec object with the given values.
     *
     * @param prefix The Logger prefix string.
     * @param flags Logging flags.
     */
    public LoggerSpec(String prefix, int flags)
    {
	if( prefix != null )
	{
	    this.log = new Log(prefix);
	}
	this.flags = flags;
    }
    
    /**
     * Construct a LoggerSpec object with the given values.
     *
     * @param prefix The Logger prefix string.
     */
    public LoggerSpec(String prefix)
    {
	this(prefix, LF_BDOMAIN_PREFIX);
    }

    /**
     * Construct an empty LoggerSpec object.
     */
    public LoggerSpec()
    {
	this(null, 0);
    }

    /**
     * @see bees.core.CapabilitySpec#getFactory()
     */
    public SpecCapabilityFactory getFactory()
    {
	return new LoggerFactory();
    }
    
    /**
     * @see bees.core.CapabilitySpec#specFromBytes(byte[])
     */
    public final synchronized CapabilitySpec specFromBytes(byte flattened[])
    {
	XdrByteArray xba;
	
	xba = new XdrByteArray(flattened);
	this.log = new Log(xba.STRING());
	this.flags = xba.INT();
	return this;
    }

    /**
     * @return A byte array containing the flattened version of this object.
     */
    public synchronized byte[] toBytes()
    {
	return marshal(this.log.getPrefix(), this.flags);
    }

    /**
     * Inherit the attributes of a parent LoggerSpec.
     *
     * @param parent The LoggerSpec to inherit the prefix from.
     */
    void inherit(LoggerSpec parent)
    {
	Log pLog;

	pLog = parent.getLog();
	if( pLog == null )
	{
	    throw new SecurityException("Revoked parent");
	}
	if( pLog.getPrefix() != null )
	{
	    this.log.setPrefix(pLog.getPrefix() +
			       ":"
			       + this.log.getPrefix());
	}
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();
	
	this.log = null;
    }

    /**
     * @return The Java NodeOS log object to be used when logging data.
     */
    synchronized Log getLog()
    {
	return this.log;
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LoggerSpec["
	    + super.toString()
	    + "; log="
	    + this.log
	    + "; flags="
	    + Integer.toHexString(this.flags)
	    + "]";
    }
}
