/*
 * NodeOSExternalChannelHandler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import java.io.IOException;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIHandler;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.InChannel;
import edu.utah.janos.nodeos.OutChannel;
import edu.utah.janos.nodeos.DeliverFunc;
import edu.utah.janos.nodeos.NetworkSpec;
import edu.utah.janos.nodeos.BufferHandle;
import edu.utah.janos.nodeos.DemultiplexKey;
import edu.utah.janos.nodeos.ThreadPoolSpec;
import edu.utah.janos.nodeos.ProtocolSpecFormatException;
import edu.utah.janos.nodeos.PacketAddressFormatException;

import bees.core.CapabilityFactory;

import bees.core.kernel.KernelObject;
import bees.core.kernel.KernelObjectStore;
import bees.core.kernel.KernelCapabilities;

import bees.core.security.PermissionSet;

public final class NodeOSExternalChannelHandler
    implements JSIHandler
{
    /**
     * The fake DeliverFunc we use when validating the InChannel's protocol
     * spec and address spec.
     */
    private static final DeliverFunc fakeDeliverFunc = new DeliverFunc() {
	    public void receive(BufferHandle bh, int offset)
	    {
	    }
	};

    /**
     * The fake DemultiplexKey we use when validating the InChannel's protocol
     * spec and address spec.
     */
    private static final DemultiplexKey fakeDemultiplexKey =
	new DemultiplexKey(Domain.currentDomain());

    public NodeOSExternalChannelHandler()
    {
    }
    
    public JSIHandler resolveSpecifier(JSIMessage jm, JSIPair jp)
    {
	JSIHandler retval;

	if( jp.getValue().length == 0 )
	{
	    retval = this;
	}
	else
	{
	    Object obj;

	    obj = KernelCapabilities.getTree().get(jp.getStringValue());
	    if( obj instanceof JSIHandler )
		retval = (JSIHandler)obj;
	    else
		retval = this;
	}
	return retval;
    }
    
    public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	throws ProtocolSpecFormatException,
	       PacketAddressFormatException,
	       InstantiationException,
	       MissingPairException,
	       IOException
    {
	JSIPair cap, jp;
		    
	switch( jm.getWhat() )
	{
	case JSIMessage.CREATE_PROPERTY:
	    {
		Domain domain = Domain.currentDomain();
		String ips, ias, ops, oas;
		String name, comment;
		byte flattened[];
		KernelObject ko;
		OutChannel oc;
		InChannel ic;
		long id;

		name = jm.expect("name");
		ips = jm.expect("incomingProtocolSpec");
		ias = jm.expect("incomingAddressSpec");
		/* XXX Find a better way to verify the specs */
		ic = InChannel.create(domain,
				      fakeDemultiplexKey,
				      ips,
				      ias,
				      fakeDeliverFunc);
		ic.destroy();
		ic = null;
		ops = jm.expect("outgoingProtocolSpec");
		oas = jm.expect("outgoingAddressSpec");
		/* XXX Find a better way to verify the specs */
		oc = OutChannel.create(domain, ops, oas);
		oc.destroy();
		oc = null;
		comment = jm.get("comment", "NodeOSExternalChannel");
		flattened = NodeOSNeighborFactory.marshal(ips,
							  ias,
							  ops,
							  oas);
		id = KernelObjectStore.getPrimary().put(flattened, comment);
		ko = KernelObjectStore.getPrimary().get(id);
		KernelCapabilities.getTree().
		    put(name, CapabilityFactory.
			instantiate(new NodeOSExternalChannel(),
				    id,
				    ko.getData(),
				    new PermissionSet()));
		jm.reply(id);
	    }
	    break;
	case JSIMessage.GET_PROPERTY:
	    jm.restoreSpecifier();
	    cap = jm.popSpecifier();
	    if( (jp = jm.popSpecifier()) != null )
	    {
		if( jp.getTag().equals("children") )
		{
		    KernelCapabilities.getChildren(jm,
						   cap,
						   NodeOSNeighbor.class);
		}
		else
		{
		    jm.error("Unknown property: " + jp.getTag());
		}
	    }
	    else
	    {
		jm.error("No property to get");
	    }
	    break;
	default:
	    throw new UnsupportedOperationException();
	}
    }

    public void getSuites(JSIMessage jm)
    {
    }

    public String toString()
    {
	return "NodeOSExternalChannelHandler["
	    + "]";
    }
}
