/*
 * NodeOSNeighborFactory.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import java.util.Vector;

import edu.utah.janos.nodeos.Info;
import edu.utah.janos.nodeos.TransportInterface;

import bees.core.Xdr;
import bees.core.NeighborImpl;
import bees.core.XdrByteArray;

import bees.core.kernel.KernelObjectCoroutine;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.EnumerationPair;
import bees.core.security.IntegerPermission;
import bees.core.security.PermissionException;
import bees.core.security.EnumerationException;
import bees.core.security.EnumerationPermission;

import bees.core.bdomain.BDomain;

/**
 * The NodeOSNeighborFactory is used to produce NodeOSNeighbor capabilities
 * from other capabilities.
 */
public final class NodeOSNeighborFactory
{
    /**
     * Permission to access a builtin protocol (e.g. eth0, if/ipv4/udp).
     */
    private static final Permission PROTOCOL_PERMISSION =
	new Permission("protocol");

    /**
     * An enumeration containing the types of supported protocols.
     * <p>
     * XXX awful...
     */
    private static final EnumerationPair PROTOCOL_TYPES[];

    static {
	Vector nics, protocolTypes;
	int lpc, size;
	
	protocolTypes = new Vector();
	protocolTypes.addElement(new EnumerationPair("if/ipv4/udp", 0));
	
	nics = Info.getInterfaces();

	size = nics.size();
	for( lpc = 0; lpc < size; lpc++ )
	{
	    TransportInterface ti;

	    ti = (TransportInterface)nics.elementAt(lpc);
	    protocolTypes.addElement(new EnumerationPair(ti.getName(), 2));
	    protocolTypes.addElement(new EnumerationPair(ti.getName()
							 + "/ipv4/udp",
							 0));
	}

	PROTOCOL_TYPES = new EnumerationPair[protocolTypes.size()];
	protocolTypes.copyInto(PROTOCOL_TYPES);
    }

    /**
     * An AddressComponent for IPv4 addresses.
     */
    static class IPAddressComponent
	implements AddressComponent
    {
        /**
         * @see bees.core.resource.AddressComponent#map(String, Permission)
         */
	public String map(String type, Permission what)
	{
	    return ((IPAddressPermission)what).getStringValue();
	}
    }

    /**
     * An AddressComponent for TCP/UDP ports.
     */
    static class PortAddressComponent
	implements AddressComponent
    {
        /**
         * @see bees.core.resource.AddressComponent#map(String, Permission)
         */
	public String map(String type, Permission what)
	{
	    return Long.toString(((IntegerPermission)what).getValue());
	}
    }

    /**
     * An array of AddressComponents that convert Permissions to a valid
     * NodeOS address specification.  Each element in the array corresponds to
     * the number value associated with the protocols in the PROTOCOL_TYPES
     * enumeration.
     */
    static final AddressComponent ADDRESS_CONVERTER[][] = {
	{
	    /* if/ipv4/udp */
	    new IPAddressComponent() {
		    public String map(String type, Permission what)
		    {
			String retval;

			/*
			 * XXX The jnodeos doesn't allow restriction of
			 * OutChannel source addresses so we have to shove
			 * a "*" in here.
			 */
			if( type.equals("out-source-address") )
			{
			    retval = "*";
			}
			else
			{
			    retval = super.map(type, what);
			}
			return retval;
		    }
		},
	    new PortAddressComponent(),
	    new IPAddressComponent(),
	    new PortAddressComponent(),
	}
    };

    /**
     * An array of Permission names that are needed to produce an InChannel
     * address specification.  Similar to ADDRESS_CONVERTER the order of the
     * array elements is dependent on the PROTOCOL_TYPES enumeration.
     */
    static final String IN_ADDRESS_MAPPING[][] = {
	{
	    "in-source-address",
	    "in-destination-udp-port",
	    "in-destination-address",
	    "in-source-udp-port",
	},
	{ "in-if-address", "in-if-type" },
	{ }, 
    };
    
    /**
     * An array of Permission names that are needed to produce an OutChannel
     * address specification.  Similar to ADDRESS_CONVERTER the order of the
     * array elements is dependent on the PROTOCOL_TYPES enumeration.
     */
    static final String OUT_ADDRESS_MAPPING[][] = {
	{
	    "out-source-address",
	    "out-destination-udp-port",
	    "out-destination-address",
	    "out-source-udp-port",
	},
	{ "out-if-address", "out-if-type" },
	{ }, 
    };

    /**
     * Flatten the given arguments into a byte array that can be given to a
     * NodeOSNeighbor capability.
     *
     * @param ips Incoming protocol specification.
     * @param ias Incoming address specification.
     * @param ops Outgoing protocol specification.
     * @param oas Outgoing address specification.
     * @return A byte array containing the flattened versions of the arguments.
     */
    static byte[] marshal(String ips, String ias, String ops, String oas)
    {
	XdrByteArray xba;

	xba = new XdrByteArray(Xdr.STRING(ips)
			       + Xdr.STRING(ias)
			       + Xdr.STRING(ops)
			       + Xdr.STRING(oas));
	xba.PUT(ips);
	xba.PUT(ias);
	xba.PUT(ops);
	xba.PUT(oas);
	return xba.toBytes();
    }

    /**
     * A cached pointer to the current BDomain.
     */
    private final BDomain bdomain;

    /**
     * Construct a NodeOSNeighborFactory.
     */
    public NodeOSNeighborFactory()
    {
	this.bdomain = BDomain.currentBDomain();
    }
    
    synchronized NeighborImpl create(String incomingProtocolSpec,
				     String incomingAddressSpec,
				     String outgoingProtocolSpec,
				     String outgoingAddressSpec,
				     String comment)
    {
	return (NeighborImpl)KernelObjectCoroutine.
	    create(new NodeOSNeighbor(),
		   this.bdomain.getBDomainCoroutine(),
		   marshal(incomingProtocolSpec,
			   incomingAddressSpec,
			   outgoingProtocolSpec,
			   outgoingAddressSpec),
		   comment,
		   NodeOSNeighbor.DEFAULT_PERMISSIONS);
    }
    
    synchronized NeighborImpl create(String ips,
				     String ias,
				     String ops,
				     String oas)
    {
	return this.create(ips, ias, ops, oas, "Unnamed Neighbor");
    }

    private String computeAddressSpec(String addressMapping[],
				      NetworkService local,
				      NetworkService remote,
				      PermissionSet address,
				      AddressComponent addressConverter[])
    {
	String retval = "";
	int lpc;

	for( lpc = 0; lpc < addressMapping.length; lpc++ )
	{
	    Permission spec, perm;
	    int lpc2;
	    
	    perm = new Permission(addressMapping[lpc], "read");
	    if( (spec = address.hasPermission(perm)) != null )
	    {
		boolean valid = false;

		/*
		 * Check every capability in turn for the permission.
		 *
		 * XXX Probably not the best way to do this.
		 */
		try
		{
		    local.checkPermission(spec);
		    valid = true;
		}
		catch(PermissionException pe)
		{
		    /* Ignore... */
		}
		try
		{
		    local.getNetworkInterface().checkPermission(spec);
		    valid = true;
		}
		catch(PermissionException pe)
		{
		    /* Ignore... */
		}
		try
		{
		    remote.checkPermission(spec);
		    valid = true;
		}
		catch(PermissionException pe)
		{
		    /* Ignore... */
		}
		try
		{
		    remote.getNetworkInterface().checkPermission(spec);
		    valid = true;
		}
		catch(PermissionException pe)
		{
		    /* Ignore... */
		}
		if( !valid )
		{
		    throw new PermissionException("Invalid permission: "
						  + spec);
		}
	    }
	    else
	    {
		throw new PermissionException("Couldn't find "
					      + perm.toString()
					      + " in "
					      + address);
	    }
	    if( lpc > 0 )
	    {
		retval += ":";
	    }
	    retval += addressConverter[lpc].map(addressMapping[lpc], spec);
	}
	return retval;
    }

    /**
     * Create a NodeOSNeighbor from the given capabilities.
     *
     * @param local The local NetworkService permissions.
     * @param remote The remote NetworkService permissions.
     * @param address A set of Permissions that describe the desired connection
     * between the local and remote NetworkServices.
     * @return A NodeOSNeighbor capability.
     * @throws EnumerationException if the NetworkInterface doesn't have a
     * supported "protocol" permission.
     */
    public NeighborImpl create(NetworkService local, NetworkService remote,
			       PermissionSet address)
	throws EnumerationException
    {
	EnumerationPermission ep;
	NetworkInterface nic;
	String ias, oas, ps;
	int slot;

	nic = local.getNetworkInterface();
	if( remote.getNetworkInterface().getLocal() != nic )
	{
	    throw new IllegalArgumentException(
		"Remote interface isn't connected to the local interface");
	}

	ep = (EnumerationPermission)nic.clonePermission(PROTOCOL_PERMISSION);
	ps = ep.getValue();
	slot = ep.mapValue(PROTOCOL_TYPES);
	ias = this.computeAddressSpec(IN_ADDRESS_MAPPING[slot],
				      local,
				      remote,
				      address,
				      ADDRESS_CONVERTER[slot]);
	oas = this.computeAddressSpec(OUT_ADDRESS_MAPPING[slot],
				      local,
				      remote,
				      address,
				      ADDRESS_CONVERTER[slot]);
	return this.create(ps, ias, ps, oas);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "NodeOSNeighborFactory["
	    + "]";
    }
}
