/*
 * NodeOSNeighborHandler.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import java.io.IOException;

import edu.utah.janosvm.kit.comm.jsi.JSIPair;
import edu.utah.janosvm.kit.comm.jsi.JSIMessage;
import edu.utah.janosvm.kit.comm.jsi.JSIPropertyInfo;
import edu.utah.janosvm.kit.comm.jsi.JSIHandlerContext;
import edu.utah.janosvm.kit.comm.jsi.MissingPairException;

import edu.utah.janos.nodeos.Info;
import edu.utah.janos.nodeos.Domain;
import edu.utah.janos.nodeos.InChannel;
import edu.utah.janos.nodeos.OutChannel;
import edu.utah.janos.nodeos.DeliverFunc;
import edu.utah.janos.nodeos.BufferHandle;
import edu.utah.janos.nodeos.DemultiplexKey;
import edu.utah.janos.nodeos.TransportInterface;
import edu.utah.janos.nodeos.AddressFormatException;
import edu.utah.janos.nodeos.ProtocolSpecFormatException;
import edu.utah.janos.nodeos.PacketAddressFormatException;

import bees.core.Capability;

import bees.core.kernel.KernelCapabilities;
import bees.core.kernel.KernelObjectCoroutine;

import bees.core.bdomain.BDomain;

/**
 * The NodeOSNeighborHandler is a JSIHandler that can create NodeOSNeighbor
 * capabilities.
 */
public final class NodeOSNeighborHandler
    extends CapabilityHandler
{
    /**
     * The set of supported properties to be added to a JSIMessage.GET_SUITES
     * request.
     */
    private static final JSIPropertyInfo JSI_PROPERTIES[] = {
	new JSIPropertyInfo("nodeOSNeighbor",
			    (1 << JSIMessage.CREATE_PROPERTY),
			    "incomingProtocolSpec:string - The NodeOS style "
			    + "InChannel protocol spec.\n"
			    + "incomingAddressSpec:string - The NodeOS style "
			    + "InChannel address spec.\n"
			    + "outgoingProtocolSpec:string - The NodeOS style "
			    + "OutChannel protocol spec.\n"
			    + "outgoingAddressSpec:string - The NodeOS style "
			    + "OutChannel address spec.\n\n"
			    + "Create a NodeOSNeighbor capability.\n"),
	new JSIPropertyInfo("nodeOSNeighbor",
			    (1 << JSIMessage.GET_PROPERTY),
			    "objectID:int",
			    "Get the values of a NodeOSNeighbor object.\n"),
	new JSIPropertyInfo("nodeOSNeighbor",
			    (1 << JSIMessage.GET_SUITES),
			    "objectID:int",
			    "Get supported suites of a NodeOSNeighbor "
			    + "object.\n"),
    };

    /**
     * The fake DeliverFunc we use when validating the InChannel's protocol
     * spec and address spec.
     */
    private static final DeliverFunc FAKE_DELIVER_FUNC = new DeliverFunc() {
	    public void receive(BufferHandle bh, int offset)
	    {
	    }
	};

    /**
     * The fake DemultiplexKey we use when validating the InChannel's protocol
     * spec and address spec.
     */
    private static final DemultiplexKey FAKE_DEMULTIPLEX_KEY =
	new DemultiplexKey(Domain.currentDomain());

    /**
     * Construct an empty NodeOSNeighborHandler object.
     */
    public NodeOSNeighborHandler()
    {
    }
    
    /**
     * @see bees.core.resource.CapabilityHandler#getCapabilityClass()
     */
    public Class getCapabilityClass()
    {
	return NodeOSNeighbor.class;
    }

    /**
     * @see bees.core.resource.CapabilityHandler#createCapability(JSIMessage, String, String)
     */
    public Object createCapability(JSIMessage jm,
				   String name,
				   String comment)
	throws PacketAddressFormatException,
	       ProtocolSpecFormatException,
	       AddressFormatException,
	       MissingPairException,
	       IOException
    {
	Domain domain = Domain.currentDomain();
	String ips, ias, ops, oas;
	byte flattened[];
	OutChannel oc;
	InChannel ic;
	
	ips = jm.expect("incomingProtocolSpec");
	ias = jm.expect("incomingAddressSpec");
	/* XXX Find a better way to verify the specs */
	ic = InChannel.create(domain,
			      FAKE_DEMULTIPLEX_KEY,
			      ips,
			      ias,
			      FAKE_DELIVER_FUNC);
	ic.destroy();
	ic = null;
	ops = jm.expect("outgoingProtocolSpec");
	oas = jm.expect("outgoingAddressSpec");
	/* XXX Find a better way to verify the specs */
	oc = OutChannel.create(domain, ops, oas);
	oc.destroy();
	oc = null;
	flattened = NodeOSNeighborFactory.marshal(ips,
						  ias,
						  ops,
						  oas);
	return (Capability)KernelObjectCoroutine.
	    create(new NodeOSNeighbor(),
		   BDomain.currentBDomain().getBDomainCoroutine(),
		   flattened,
		   null,
		   comment,
		   KernelCapabilities.toPermissionSet(jm));
    }

    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#handleMessage(JSIHandlerContext, JSIMessage)
     */
    public void handleMessage(JSIHandlerContext jhc, JSIMessage jm)
	throws Throwable
    {
	JSIPair jp;
	
	switch( jm.getWhat() )
	{
	case JSIMessage.EXECUTE_PROPERTY:
	    if( (jp = jm.popSpecifier()) != null )
	    {
		if( jp.getTag().equals("getInterfaceByAddress") )
		{
		    TransportInterface ti;
		    String addr;

		    addr = jm.expect("address");
		    if( (ti = Info.getInterfaceByAddress(addr)) != null )
		    {
			jm.reply(ti.getName());
		    }
		    else
		    {
			jm.error("Unknown interface address: " + addr);
		    }
		}
		else
		{
		    jm.error("Unknown property: " + jp.getTag());
		}
	    }
	    else
	    {
		jm.error("No property to execute");
	    }
	    break;
	default:
	    super.handleMessage(jhc, jm);
	    break;
	}
    }

    /**
     * @see edu.utah.janosvm.kit.comm.jsi.JSIHandler#getSuites(JSIMessage)
     */
    public void getSuites(JSIMessage jm)
    {
	jm.add("result", JSI_PROPERTIES);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "NodeOSNeighborHandler["
	    + super.toString()
	    + "]";
    }
}
