/*
 * PathSpec.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import bees.core.Xdr;
import bees.core.Assert;
import bees.core.Capability;
import bees.core.XdrByteArray;
import bees.core.CapabilitySpec;
import bees.core.SpecCapabilityFactory;

/**
 * PathSpec is used as a base for the Directory and File capabilities.  It
 * encodes the FileSystemRoot relative path to files and directories.
 */
public class PathSpec
    extends Capability
    implements CapabilitySpec
{
    /**
     * Marshal a path into a byte array.
     *
     * @param path The path to marshal.
     * @return A byte array containing the flattened version of the argument.
     */
    static byte[] marshal(String path)
    {
	XdrByteArray xba;

	xba = new XdrByteArray(Xdr.STRING(path));
	xba.PUT(path);
	return xba.toBytes();
    }

    /**
     * Verify that a path doesn't contain any file separators.
     *
     * @param path The path to verify.
     * @return path
     */
    static String verifyFilePath(String path)
    {
	if( path.indexOf(java.io.File.separator) != -1 )
	{
	    throw new IllegalArgumentException("File separators not allowed");
	}
	return path;
    }

    /**
     * The FileSystemRoot relative path.
     */
    private String path;

    /**
     * Construct a PathSpec with the given values.
     *
     * @param path The FileSystemRoot relative path.
     */
    public PathSpec(String path)
    {
	this.path = path;
    }

    /**
     * Construct an empty PathSpec.
     */
    public PathSpec()
    {
	this(null);
    }

    /**
     * @see bees.core.CapabilitySpec#specFromBytes(byte[])
     */
    public final synchronized CapabilitySpec specFromBytes(byte flattened[])
    {
	XdrByteArray xba;
	String newPath;

	Assert.require(flattened);
	
	xba = new XdrByteArray(flattened);
	this.path = xba.STRING();
	if( this.path == null )
	{
	    this.path = "";
	}
	return this;
    }

    /**
     * @return A flattened version of this object.
     */
    public synchronized byte[] toBytes()
    {
	return marshal(this.path);
    }

    /**
     * @see bees.core.CapabilitySpec#getFactory()
     */
    public SpecCapabilityFactory getFactory()
    {
	return null;
    }
    
    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();

	this.path = null;
    }

    /**
     * @return The FileSystemRoot relative path.
     */
    synchronized String getPath()
    {
	return this.path;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "PathSpec["
	    + super.toString()
	    + "; path="
	    + this.path
	    + "]";
    }
}
