/*
 * SystemPackage.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.resource;

import edu.utah.janos.nodeos.Domain;

import bees.core.Xdr;
import bees.core.Capability;
import bees.core.XdrByteArray;

import bees.core.security.PermissionSet;

/**
 * A SystemPackage governs access to a single system package.
 */
public final class SystemPackage
    extends Capability
{
    /**
     * Marshal the argument into a byte array.
     *
     * @param name The name of the package.
     * @return A flattened version of the argument.
     */
    public static byte[] marshal(String name)
    {
	XdrByteArray xba;

	xba = new XdrByteArray(Xdr.STRING(name));
	xba.PUT(name);
	return xba.toBytes();
    }
    
    /**
     * Default permissions for a SystemPackage object.
     */
    public static final PermissionSet DEFAULT_PERMISSIONS =
	new PermissionSet();

    /**
     * The name of the package to allow access to.
     */
    private String name;
    
    /**
     * Construct an empty SystemPackage capability.
     */
    public SystemPackage()
    {
    }

    /**
     * @see bees.core.Capability#fromBytes(byte[])
     */
    protected synchronized void fromBytes(byte flattened[])
	throws Exception
    {
	XdrByteArray xba;
	
	super.fromBytes(flattened);

	xba = new XdrByteArray(flattened);
	this.name = xba.STRING();
    }

    /**
     * @see bees.core.Capability#revoke()
     */
    public synchronized void revoke()
    {
	super.clear();

	this.name = null;
    }

    /**
     * @return The name of the package to allow access to.
     */
    public String getName()
    {
	return this.name;
    }

    /**
     * Export the contained package from the current Domain's class loader.  In
     * other words, give the current Domain access to the package.
     */
    public synchronized void export()
    {
	Domain domain = Domain.currentDomain();

	domain.addSystemPackage(this.name);
	domain.sealPackage(this.name);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "SystemPackage["
	    + super.toString()
	    + "; name="
	    + this.name
	    + "]";
    }
}
