/*
 * Cipher.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import java.security.SecureRandom;
import java.security.AlgorithmParameters;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.InvalidAlgorithmParameterException;

import java.security.spec.AlgorithmParameterSpec;

import javax.crypto.BadPaddingException;
import javax.crypto.ShortBufferException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.IllegalBlockSizeException;

/**
 * Cipher is a thin veneer over the standard javax.crypto.Cipher class that
 * uses bees.core.security classes.
 */
public class Cipher
{
    /**
     * Permission to use this key for decryption.
     */
    public static final Permission DECRYPT_PERMISSION =
	new Permission("decrypt", "execute");

    /**
     * Permission to use this key for encryption.
     */
    public static final Permission ENCRYPT_PERMISSION =
	new Permission("encrypt", "execute");

    public static final int DECRYPT_MODE = javax.crypto.Cipher.DECRYPT_MODE;
    public static final int ENCRYPT_MODE = javax.crypto.Cipher.ENCRYPT_MODE;
    public static final int PRIVATE_KEY = javax.crypto.Cipher.PRIVATE_KEY;
    public static final int PUBLIC_KEY = javax.crypto.Cipher.PUBLIC_KEY;
    public static final int SECRET_KEY = javax.crypto.Cipher.SECRET_KEY;
    public static final int UNWRAP_MODE = javax.crypto.Cipher.UNWRAP_MODE;
    public static final int WRAP_MODE = javax.crypto.Cipher.WRAP_MODE;
    
    private final javax.crypto.Cipher cipher;

    public Cipher(String transformation)
	throws NoSuchAlgorithmException,
	       NoSuchPaddingException
    {
	this.cipher = javax.crypto.Cipher.getInstance(transformation);
    }

    public String getAlgorithm()
    {
	return this.cipher.getAlgorithm();
    }

    public int getBlockSize()
    {
	return this.cipher.getBlockSize();
    }

    public int getOutputSize(int inputLength)
    {
	return this.cipher.getOutputSize(inputLength);
    }

    public byte[] getIV()
    {
	return this.cipher.getIV();
    }

    public AlgorithmParameters getParameters()
    {
	return this.cipher.getParameters();
    }

    private void checkKeyMode(int opmode, KeySpec key)
    {
	switch( opmode )
	{
	case UNWRAP_MODE:
	case DECRYPT_MODE:
	    key.checkPermission(DECRYPT_PERMISSION);
	    break;
	case WRAP_MODE:
	case ENCRYPT_MODE:
	    key.checkPermission(ENCRYPT_PERMISSION);
	    break;
	}
    }

    public void init(int opmode, KeySpec key)
	throws InvalidKeyException
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey());
    }

    public void init(int opmode, KeySpec key, SecureRandom sr)
	throws InvalidKeyException
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey(), sr);
    }

    public void init(int opmode, KeySpec key, AlgorithmParameterSpec params)
	throws InvalidAlgorithmParameterException,
	       InvalidKeyException
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey(), params);
    }

    public void init(int opmode, KeySpec key,
		     AlgorithmParameterSpec params, SecureRandom sr)
	throws InvalidAlgorithmParameterException,
	       InvalidKeyException
	       
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey(), params, sr);
    }

    public void init(int opmode, KeySpec key, AlgorithmParameters params)
	throws InvalidAlgorithmParameterException,
	       InvalidKeyException
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey(), params);
    }

    public void init(int opmode, KeySpec key,
		     AlgorithmParameters params, SecureRandom sr)
	throws InvalidAlgorithmParameterException,
	       InvalidKeyException
    {
	this.checkKeyMode(opmode, key);
	this.cipher.init(opmode, key.getKey(), params, sr);
    }

    public byte[] update(byte input[])
	throws IllegalStateException
    {
	return this.cipher.update(input);
    }

    public byte[] update(byte input[], int inputOffset, int inputLen)
	throws IllegalStateException
    {
	return this.cipher.update(input, inputOffset, inputLen);
    }

    public int update(byte input[], int inputOffset, int inputLen,
		      byte output[])
	throws ShortBufferException,
	       IllegalStateException
    {
	return this.cipher.update(input, inputOffset, inputLen, output);
    }

    public int update(byte input[], int inputOffset, int inputLen,
		      byte output[], int outputOffset)
	throws ShortBufferException,
	       IllegalStateException
    {
	return this.cipher.update(input, inputOffset, inputLen,
				  output, outputOffset);
    }

    public byte[] doFinal()
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.cipher.doFinal();
    }

    public int doFinal(byte output[], int outputOffset)
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       ShortBufferException,
	       BadPaddingException
    {
	return this.cipher.doFinal(output, outputOffset);
    }

    public byte[] doFinal(byte input[])
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.cipher.doFinal(input);
    }

    public byte[] doFinal(byte input[], int inputOffset, int inputLen)
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.cipher.doFinal(input, inputOffset, inputLen);
    }

    public int doFinal(byte input[], int inputOffset, int inputLen,
		       byte output[])
	throws IllegalStateException,
	       ShortBufferException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.cipher.doFinal(input, inputOffset, inputLen, output);
    }

    public int doFinal(byte input[], int inputOffset, int inputLen,
		       byte output[], int outputOffset)
	throws IllegalStateException,
	       ShortBufferException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.cipher.doFinal(input, inputOffset, inputLen,
				   output, outputOffset);
    }

    public byte[] wrap(KeySpec key)
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       InvalidKeyException
    {
	return this.cipher.wrap(key.getKey());
    }

    public KeySpec unwrap(byte wrappedKey[],
			  String wrappedKeyAlgorithm,
			  int wrappedKeyType)
	throws NoSuchAlgorithmException,
	       IllegalStateException,
	       InvalidKeyException
    {
	java.security.Key key;

	key = this.cipher.unwrap(wrappedKey,
				 wrappedKeyAlgorithm,
				 wrappedKeyType);
	switch( wrappedKeyType )
	{
	case SECRET_KEY:
	    return new SecretKeySpec((javax.crypto.SecretKey)key);
	case PRIVATE_KEY:
	    return new PrivateKeySpec((java.security.PrivateKey)key);
	case PUBLIC_KEY:
	    return new PublicKeySpec((java.security.PublicKey)key);
	default:
	    throw new IllegalArgumentException("Invalid wrappedKeyType value");
	}
    }

    public String toString()
    {
	return "Cipher[cipher="
	    + this.cipher
	    + "]";
    }
}
