/*
 * CountPermission.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import bees.core.Xdr;
import bees.core.Assert;
import bees.core.XdrByteArray;

/**
 * A CountPermission holds a positive integer value, most likely used to count
 * something (e.g. operations, objects).
 */
public class CountPermission
    extends Permission
{
    /**
     * The maximum count value.
     */
    private int maxCount;

    /**
     * Construct a CountPermission object with the given values.
     *
     * @param name The name of the permission.
     * @param maxCount The maximum count value.
     */
    public CountPermission(String name, int maxCount)
    {
	super(name, "execute");

	if( maxCount < 0 )
	{
	    throw new IllegalArgumentException(
		"Negative maxCount isn't allowed");
	}

	this.maxCount = maxCount;
    }

    /**
     * Construct an empty CountPermission.
     */
    public CountPermission()
    {
    }

    /**
     * @see bees.core.security.Permission#getTypeID()
     */
    protected int getTypeID()
    {
	return PERMISSION_COUNT;
    }

    /**
     * @return The maximum count value.
     */
    public int getMaxCount()
    {
	return this.maxCount;
    }

    /**
     * @see bees.core.security.Permission#revoke(Permission)
     */
    public void revoke(Permission what)
    {
	Assert.require(what);

	if( this.implies(what) )
	{
	    if( what instanceof CountPermission )
	    {
		CountPermission cp = (CountPermission)what;
		
		Assert.require(cp.maxCount >= 0);
		
		this.maxCount -= cp.maxCount;
		if( this.maxCount < 0 )
		{
		    this.maxCount = 0;
		}
	    }
	    else
	    {
		super.revoke(what);
	    }
	}
    }

    /**
     * @see bees.core.security.Permission#grant(Permission)
     */
    public void grant(Permission what)
    {
	Assert.require(what);

	if( super.nameImplies(what) )
	{
	    if( what instanceof CountPermission )
	    {
		CountPermission cp = (CountPermission)what;

		Assert.require(cp.maxCount >= 0);
		
		this.maxCount += cp.maxCount;
		if( this.maxCount < 0 )
		{
		    this.maxCount = Integer.MAX_VALUE;
		}
	    }
	    else
	    {
		super.revoke(what);
	    }
	}
    }

    public boolean implies(Permission what)
    {
	boolean retval = false;
	
	if( super.implies(what) )
	{
	    if( what instanceof CountPermission )
	    {
		CountPermission cp;
		
		cp = (CountPermission)what;
		if( cp.maxCount <= this.maxCount )
		{
		    retval = true;
		}
	    }
	    else
	    {
		retval = true;
	    }
	}
	return retval;
    }

    /**
     * @see bees.core.security.Permission#marshalledLength()
     */
    public int marshalledLength()
    {
	return super.marshalledLength() + Xdr.INT;
    }
    
    /**
     * @see bees.core.security.Permission#marshal(XdrByteArray)
     */
    public void marshal(XdrByteArray xba)
    {
	super.marshal(xba);

	xba.PUT(this.maxCount);
    }

    /**
     * @see bees.core.security.Permission#unmarshal(XdrByteArray)
     */
    public void unmarshal(XdrByteArray xba)
    {
	super.unmarshal(xba);
	
	this.maxCount = xba.INT();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "CountPermission["
	    + super.toString()
	    + "; maxCount="
	    + this.maxCount
	    + "]";
    }
}
