/*
 * FilePermission.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

/**
 * FilePermission extends the standard Permission class to support file
 * specific actions.
 */
public class FilePermission
    extends Permission
{
    /**
     * Action determining whether or not a file is included in a directory
     * listing.
     */
    public static final int LIST	= 0x10;

    /**
     * Action determining whether or not a file can be created.
     */
    public static final int CREATE	= 0x20;
    
    /**
     * Action determining whether or not a file can be deleted.
     */
    public static final int DELETE	= 0x40;

    /**
     * Construct a FilePermission with the given values.
     *
     * @param name The name of the permission.
     * @param actions The actions granted by this permission.
     */
    public FilePermission(String name, String actions)
    {
	super(name, actions);

	int extraMask = 0;

	if (actions.indexOf("list") != -1)
	{
	    extraMask |= LIST;
	}
	if (actions.indexOf("create") != -1)
	{
	    extraMask |= CREATE;
	}
	if (actions.indexOf("delete") != -1)
	{
	    extraMask |= DELETE;
	}
	super.orActionMask(extraMask);
    }

    /**
     * Construct an empty FilePermission.
     */
    public FilePermission()
    {
    }

    /**
     * @see bees.core.security.Permission#getTypeID()
     */
    public int getTypeID()
    {
	return PERMISSION_FILE;
    }

    /**
     * @see bees.core.security.Permission#getAction()
     */
    public String getAction()
    {
	StringBuffer retval = new StringBuffer(super.getAction());
	int actionMask;

	actionMask = super.getActionMask();
	if ( (actionMask & LIST) != 0 )
	{
	    retval.append(" list");
	}
	if ( (actionMask & CREATE) != 0 )
	{
	    retval.append(" create");
	}
	if ( (actionMask & LIST) != 0 )
	{
	    retval.append(" delete");
	}
	
	return retval.toString().trim();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "FilePermission["
	    + super.toString()
	    + "]";
    }
}
