/*
 * IntegerPermission.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import bees.core.Xdr;
import bees.core.XdrByteArray;

/**
 * An IntegerPermission binds a permission name to an integer value.
 */
public class IntegerPermission
    extends Permission
{
    /**
     * The permission's integer value.
     */
    long value;

    /**
     * Construct an IntegerPermission with the given values.
     *
     * @param name The name of the permission.
     * @param actions The actions granted by this permission.
     * @param value The permission's integer value.
     */
    public IntegerPermission(String name, String actions, long value)
    {
	super(name, actions);

	this.value = this.verify(value);
    }

    /**
     * Construct an IntegerPermission with the given values and a default
     * action value of "read, write, execute".
     *
     * @param name The name of the permission.
     * @param value The permission's integer value.
     */
    public IntegerPermission(String name, long value)
    {
	this(name, "read,write,execute", value);
    }

    /**
     * Construct an empty IntegerPermission.
     */
    public IntegerPermission()
    {
    }

    /**
     * @see bees.core.security.Permission#getTypeID()
     */
    public int getTypeID()
    {
	return PERMISSION_INTEGER;
    }

    /**
     * @return The permission's integer value.
     */
    public long getValue()
    {
	return this.value;
    }

    /**
     * Verify an integer value.
     *
     * @param value The unverified integer value.
     * @return value, if it is valid.
     */
    public long verify(long value)
    {
	return value;
    }

    /**
     * @see bees.core.security.Permission#marshalledLength()
     */
    public int marshalledLength()
    {
	return super.marshalledLength() + Xdr.LONG;
    }

    /**
     * @see bees.core.security.Permission#marshal(XdrByteArray)
     */
    public void marshal(XdrByteArray xba)
    {
	super.marshal(xba);

	xba.PUT(this.value);
    }

    /**
     * @see bees.core.security.Permission#unmarshal(XdrByteArray)
     */
    public void unmarshal(XdrByteArray xba)
    {
	super.unmarshal(xba);
	
	this.value = xba.LONG();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "IntegerPermission["
	    + super.toString()
	    + "; value="
	    + this.value
	    + "]";
    }
}
