/*
 * KeyGenerator.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import java.security.NoSuchAlgorithmException;

import bees.core.kernel.KernelObjectCoroutine;

import bees.core.bdomain.BDomain;

/**
 * A KeyGenerator is used to manufacture new SecretKey capabilities.
 */
public final class KeyGenerator
{
    /**
     * Cached reference to the current domain.
     */
    private final BDomain bdomain;

    /**
     * Construct an empty KeyGenerator.
     */
    public KeyGenerator()
    {
	this.bdomain = BDomain.currentBDomain();
    }

    /**
     * Generate a new key from the given values.
     *
     * @param algorithm The type of key to generate.
     * @param strength The suggested number of bits the key should have.
     * @param comment A comment to attach to the capability.
     * @param ps The permissions for the new capability.
     * @return A SecretKey capability.
     * @throws NoSuchAlgorithmException if "algorithm" is not available.
     */
    public synchronized SecretKey generateKey(String algorithm,
					      int strength,
					      PermissionSet ps,
					      String comment)
	throws NoSuchAlgorithmException
    {
	javax.crypto.KeyGenerator keyGen;
	
	keyGen = javax.crypto.KeyGenerator.getInstance(algorithm);
	if( strength > 0 )
	{
	    keyGen.init(strength);
	}
	return (SecretKey)KernelObjectCoroutine.
	    create(new SecretKey(),
		   this.bdomain.getBDomainCoroutine(),
		   KeySpec.marshal(keyGen.generateKey()),
		   comment,
		   ps);
    }

    /**
     * Generate a new key from the given values.
     *
     * @param algorithm The type of key to generate.
     * @param strength The suggested number of bits the key should have.
     * @param ps The permissions for the new capability.
     * @return A SecretKey capability.
     * @throws NoSuchAlgorithmException if "algorithm" is not available.
     */
    public synchronized SecretKey generateKey(String algorithm,
					      int strength,
					      PermissionSet ps)
	throws NoSuchAlgorithmException
    {
	return this.generateKey(algorithm, strength, ps, "Secret Key");
    }
    
    /**
     * Generate a new key from the given values and default permissions.
     *
     * @param algorithm The type of key to generate.
     * @param strength The suggested number of bits the key should have.
     * @return A SecretKey capability.
     * @throws NoSuchAlgorithmException if "algorithm" is not available.
     */
    public synchronized SecretKey generateKey(String algorithm, int strength)
	throws NoSuchAlgorithmException
    {
	return this.generateKey(algorithm,
				strength,
				SecretKey.DEFAULT_PERMISSIONS,
				"Secret Key");
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "KeyGenerator["
	    + "]";
    }
}
