/*
 * Mac.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import java.security.SecureRandom;
import java.security.AlgorithmParameters;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.InvalidAlgorithmParameterException;

import java.security.spec.AlgorithmParameterSpec;

import javax.crypto.BadPaddingException;
import javax.crypto.ShortBufferException;
import javax.crypto.NoSuchPaddingException;
import javax.crypto.IllegalBlockSizeException;

/**
 * Mac is a thin veneer over the standard javax.crypto.Mac class that
 * uses bees.core.security classes.
 */
public class Mac
{
    /**
     * Permission to use this key for decryption.
     */
    public static final Permission MAC_PERMISSION =
	new Permission("mac", "execute");

    private final javax.crypto.Mac mac;

    public Mac(String algorithm)
	throws NoSuchAlgorithmException
    {
	this.mac = javax.crypto.Mac.getInstance(algorithm);
    }

    javax.crypto.Mac getInternal()
    {
	return this.mac;
    }

    public String getAlgorithm()
    {
	return this.mac.getAlgorithm();
    }

    public int getMacLength()
    {
	return this.mac.getMacLength();
    }

    private void checkKey(KeySpec key)
    {
	key.checkPermission(MAC_PERMISSION);
    }
    
    public void init(KeySpec key)
	throws InvalidKeyException
    {
	checkKey(key);
	this.mac.init(key.getKey());
    }

    public void init(KeySpec key, AlgorithmParameterSpec params)
	throws InvalidAlgorithmParameterException,
	       InvalidKeyException
    {
	checkKey(key);
	this.mac.init(key.getKey(), params);
    }

    public void update(byte input)
	throws IllegalStateException
    {
	this.mac.update(input);
    }

    public void update(byte input[])
	throws IllegalStateException
    {
	this.mac.update(input);
    }

    public void update(byte input[], int inputOffset, int inputLen)
	throws IllegalStateException
    {
	this.mac.update(input, inputOffset, inputLen);
    }

    public byte[] doFinal()
	throws IllegalStateException
    {
	return this.mac.doFinal();
    }

    public void doFinal(byte output[], int outputOffset)
	throws IllegalStateException,
	       ShortBufferException
    {
	this.mac.doFinal(output, outputOffset);
    }

    public byte[] doFinal(byte input[])
	throws IllegalStateException,
	       IllegalBlockSizeException,
	       BadPaddingException
    {
	return this.mac.doFinal(input);
    }

    public void reset()
    {
	this.mac.reset();
    }

    public String toString()
    {
	return "Mac[mac="
	    + this.mac
	    + "]";
    }
}
