/*
 * MaximumFloatPermission.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

/**
 * A MaximumFloatPermission limits a FloatPermission to a given maximum.
 */
public class MaximumFloatPermission
    extends FloatPermission
{
    /**
     * Construct a MaximumFloatPermission with the given values.
     *
     * @param name The name of the permission.
     * @param actions The actions granted by this permission.
     * @param value The floating point maximum.
     */
    public MaximumFloatPermission(String name, String actions, double value)
    {
	super(name, actions, value);
    }

    /**
     * Construct a MaximumFloatPermission with the given values.
     *
     * @param name The name of the permission.
     * @param value The floating point maximum.
     */
    public MaximumFloatPermission(String name, double value)
    {
	super(name, value);
    }

    /**
     * Construct an empty MaximumFloatPermission.
     */
    public MaximumFloatPermission()
    {
    }

    /**
     * @see bees.core.security.Permission#getTypeID()
     */
    public int getTypeID()
    {
	return PERMISSION_MAX_FLOAT;
    }

    /**
     * Limit this permission by the given one.  For example, if this permission
     * has a value of 128 and the given one has a value of 64, this
     * permission will now have a value of 64.
     *
     * @param perm The Permission to limit this one by.
     */
    private void limitBy(Permission perm)
    {
	if( super.nameImplies(perm) )
	{
	    if( perm instanceof FloatPermission )
	    {
		if( !(perm instanceof MinimumFloatPermission) )
		{
		    FloatPermission fp;
		    
		    fp = (FloatPermission)perm;
		    if( fp.value < this.value )
		    {
			this.value = fp.value;
		    }
		}
	    }
	}
    }

    /**
     * @see bees.core.security.Permission#bound(Permission)
     */
    public void bound(Permission perm)
    {
	super.bound(perm);

	this.limitBy(perm);
    }
    
    /**
     * @see bees.core.security.Permission#revoke(Permission)
     */
    public void revoke(Permission perm)
    {
	super.revoke(perm);

	this.limitBy(perm);
    }

    /**
     * @see bees.core.security.Permission#grant(Permission)
     */
    public void grant(Permission perm)
    {
	super.grant(perm);

	if( super.nameImplies(perm) )
	{
	    if( perm instanceof FloatPermission )
	    {
		if( !(perm instanceof MinimumFloatPermission) )
		{
		    FloatPermission fp;
		    
		    fp = (FloatPermission)perm;
		    if( fp.value > this.value )
		    {
			this.value = fp.value;
		    }
		}
	    }
	}
    }

    /**
     * @see bees.core.security.Permission#implies(Permission)
     */
    public boolean implies(Permission what)
    {
	boolean retval = false;
	
	if( super.implies(what) )
	{
	    if( what instanceof FloatPermission )
	    {
		FloatPermission ip;
		
		ip = (FloatPermission)what;
		if( ip.value <= this.value )
		{
		    retval = true;
		}
	    }
	    else
	    {
		retval = true;
	    }
	}
	return retval;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "MaximumFloatPermission["
	    + super.toString()
	    + "]";
    }
}
