/* 
 * Permission.java
 *
 * Copyright (c) 2001, 2002 The University of Washington
 * All rights reserved.
 * 
 * This file is part of the Bees distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to bees-dist@cs.utah.edu
 *         
 */
package bees.core.security;

import edu.utah.janosvm.sys.CrossTeamCloneable;

import bees.core.Xdr;
import bees.core.Panic;
import bees.core.Assert;
import bees.core.CloneFactory;
import bees.core.XdrByteArray;

import bees.util.MD5;

/**
 * Describes an abstract permission, consisting of a string name and
 * an optional set of actions from the set {read,write,execute}.
 *
 * @author Andrew Whitaker
 */
public class Permission
    implements Cloneable, CrossTeamCloneable, PermissionTypes
{
    /**
     * Action determining whether or not the property specified by the
     * permission name is readable.
     */
    public static final int READ	= 0x01;
    
    /**
     * Action determining whether or not the property specified by the
     * permission name is writable.
     */
    public static final int WRITE	= 0x02;
    
    /**
     * Action determining whether or not the property specified by the
     * permission name is executable.
     */
    public static final int EXECUTE	= 0x04;
    
    /**
     * Action determining whether or not the property specified by the
     * permission name is appendable.
     */
    public static final int APPEND	= 0x08;

    /**
     * The set of actions granted/required by this permission.
     */
    private int actionMask;

    /**
     * The full name of the permission.
     */
    private String name;

    /**
     * Creates a permission with a set of actions.  Actions are
     * specified by passing an argument string consisting of a subset of
     * {"read","write","execute"}.  The actions must be lower-case.  For
     * example: new Permission ("Routing table","read write")
     *
     * @param name The name of the permission.
     * @param actions The actions granted by this permission.
     */
    public Permission(String name, String actions) 
    {
	Assert.require(name);
	Assert.require(actions);
	
	/* XXX Put name as all lower case */
	this.name = name;
	if (actions.indexOf("read") != -1)
	{
	    this.actionMask |= READ;
	}
	if (actions.indexOf("write") != -1)
	{
	    this.actionMask |= WRITE;
	}
	if (actions.indexOf("execute") != -1)
	{
	    this.actionMask |= EXECUTE;
	}
	if (actions.indexOf("append") != -1)
	{
	    this.actionMask |= APPEND;
	}
    }

    /**
     * Creates a new permission with the given human-readable name and
     * no actions.  Names serve as unique keys.
     *
     * @param name The name of the permission.
     */
    public Permission(String name)
    {
	this(name, "");
    }

    /**
     * Construct an empty Permission.
     */
    public Permission()
    {
	this("", "");
    }

    /**
     * @return An integer from the PermissionTypes enumeration specifying the
     * type of permission.
     */
    protected int getTypeID()
    {
	return PERMISSION_BASIC;
    }

    /**
     * @return The bits that make up the action mask.
     */
    public final int getActionMask()
    {
	return this.actionMask;
    }

    /**
     * @param bits The bits to or into the action mask.
     */
    protected void orActionMask(int bits)
    {
	this.actionMask |= bits;
    }

    /**
     * @return The actions as a human-readable string.
     */
    public String getAction()
    {
	StringBuffer retval = new StringBuffer("");
	
	if ( (this.actionMask & READ) != 0 )
	{
	    retval.append("read ");
	}
	if ( (this.actionMask & WRITE) != 0 )
	{
	    retval.append("write ");
	}
	if ( (this.actionMask & EXECUTE) != 0 )
	{
	    retval.append("execute ");
	}
	if ( (this.actionMask & APPEND) != 0 )
	{
	    retval.append("append ");
	}
	
	return retval.toString().trim();
    }

    /**
     * @return The name of the permission.
     */
    public final String getName()
    {
	return this.name;
    }

    /**
     * Test whether this permission's name implies the given permission.
     *
     * @param p The permission to match.
     * @return True if this permission name matches p.
     */
    public final boolean nameImplies(Permission p)
    {
	boolean retval = false;

	Assert.require(p);
	
	/* XXX this will be slow */
	if( p.getName().equalsIgnoreCase(this.getName()) )
	{
	    retval = true;
	}
	return retval;
    }
    
    /**
     * Test whether this permission implies the given permission.  For
     * example,  read/write implies the read permission.  Names are
     * case-insensitive.
     *
     * @param p The permission to match.
     * @return True if this permission matches p.
     */
    public boolean implies(Permission p)
    {
	boolean retval = false;

	Assert.require(p);

	if( this.nameImplies(p) )
	{
	    if( (p.actionMask | this.actionMask) == this.actionMask )
	    {
		retval = true;
	    }
	}
	return retval;
    }

    /**
     * Grant the actions in "what" to this permission.
     *
     * @param what The permissions that should be granted to this permission.
     */
    public void grant(Permission what)
    {
	Assert.require(what);

	if( this.nameImplies(what) )
	{
	    this.actionMask |= what.actionMask;
	}
    }

    /**
     * Revoke the actions in "what" from this permission.
     *
     * @param what The permissions that should be revoked from this permission.
     */
    public void revoke(Permission what)
    {
	Assert.require(what);
	
	if( this.implies(what) )
	{
	    this.actionMask &= ~what.actionMask;
	}

	// Assert.ensure(!this.implies(what));
    }

    /**
     * Bound this permission's actions to the ones available in what.  In other
     * words, the actions available in this permission will be less than or
     * equal to those in "what".
     *
     * @param what The permission that this permission should be bounded by.
     */
    public void bound(Permission what)
    {
	Assert.require(what);
	
	if( this.nameImplies(what) )
	{
	    this.actionMask &= what.actionMask;
	}
    }

    /**
     * Update a an MD5 object with the value of this permission.
     * <p>
     * XXX A bit hacky.
     *
     * @param digest The digest to update.
     */
    public void updateHash(MD5 digest)
    {
	digest.update(this.toString());
    }

    /**
     * @return The number of bytes needed to marshal this permission.
     */
    public int marshalledLength()
    {
	return Xdr.STRING(this.name) + Xdr.INT;
    }

    /**
     * Marshal this object into the given byte array.
     *
     * @param xba The destination for the flattened version of this Permission.
     */
    public void marshal(XdrByteArray xba)
    {
	Assert.require(xba);
	
	xba.PUT(this.name);
	xba.PUT(this.actionMask);
    }

    /**
     * Unmarshal this object from the given byte array.
     *
     * @param xba The source of the flattened version of this Permission.
     */
    public void unmarshal(XdrByteArray xba)
    {
	Assert.require(xba);
	
	this.name = xba.STRING();
	this.actionMask = xba.INT();
    }

    /**
     * @return A byte array containing the flattened version of this
     * permission.
     */
    public byte[] toBytes()
    {
	XdrByteArray xba;

	xba = new XdrByteArray(Xdr.INT + this.marshalledLength());
	xba.PUT(this.getTypeID());
	this.marshal(xba);
	return xba.toBytes();
    }

    /**
     * Instantiate a Permission object from the given byte array.
     *
     * @param xba The source of the flattened version of a Permission.
     * @return A new Permission object.
     */
    public static Permission instantiate(XdrByteArray xba)
    {
	Permission retval = null;
	int type;
	Class cl;
	
	Assert.require(xba);

	type = xba.INT();
	if( (type <= PERMISSION_MIN) || (type >= PERMISSION_MAX) )
	{
	    throw new IllegalArgumentException("Illegal permission type: "
					       + type);
	}
	if( (cl = PERMISSION_CLASSES[type]) != null )
	{
	    try
	    {
		retval = (Permission)cl.newInstance();
		retval.unmarshal(xba);
	    }
	    catch(IllegalAccessException e)
	    {
		throw new Panic(e);
	    }
	    catch(InstantiationException e)
	    {
		throw new Panic(e);
	    }
	}
	return retval;
    }
    
    /**
     * Instantiate a Permission object from the given byte array.
     *
     * @param flattened The source of the flattened version of a Permission.
     * @return A new Permission object.
     */
    public static Permission instantiate(byte flattened[])
    {
	XdrByteArray xba;
	
	Assert.require(flattened);

	xba = new XdrByteArray(flattened);
	return instantiate(xba);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return (this.getName() + "=" + this.getAction());
    }

    /**
     * Tests for exact equality: the name and actions must precisely
     * match.  Names are case-insensitive.
     *
     * @param obj The object to compare this one against.
     * @return True if this object equals the given one.
     */
    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof Permission )
	{
	    Permission p = (Permission)obj;

	    if( (this.actionMask == p.actionMask) &&
		(this.name.equalsIgnoreCase(p.name)) )
	    {
		retval = true;
	    }
	}
	return retval;
    }

    /**
     * @see java.lang.Object#clone()
     */
    public Object clone()
    {
	try
	{
	    Object retval = super.clone();
	    Permission p;
	    int lpc;

	    p = (Permission)retval;
	    p.name = CloneFactory.clone(this.name);
	    return retval;
	}
	catch(CloneNotSupportedException e)
	{
	    throw new Panic("Permission isn't Cloneable?");
	}
    }
}
