/*
 * PermissionSet.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.core.security;

import java.util.Vector;

import bees.core.Xdr;
import bees.core.Assert;
import bees.core.XdrByteArray;

import bees.util.MD5;

/**
 * A PermissionSet holds a list of permissions and provides a number of
 * methods for manipulating them.
 */
public final class PermissionSet
    implements Cloneable
{
    /**
     * Construct an array of PermissionSets to a Vector containing all of their
     * elements.
     *
     * @param v The Vector to add the elements to or null.
     * @param ps The array of PermissionSets whose elements to be added to a
     * Vector.
     * @return A Vector containing all of the Permissions from "ps".
     */
    private static Vector toVector(Vector v, PermissionSet ps[])
    {
	Vector retval = v;
	int lpc;

	if( retval == null )
	{
	    retval = new Vector();
	}
	for( lpc = 0; lpc < ps.length; lpc++ )
	{
	    if( ps[lpc] != null )
	    {
		int lpc2;
		
		for( lpc2 = 0; lpc2 < ps[lpc].permissions.length; lpc2++ )
		{
		    retval.addElement(ps[lpc].permissions[lpc2]);
		}
	    }
	}
	return retval;
    }

    /**
     * The Permission's contained by this set.
     */
    private final Permission permissions[];

    /**
     * Construct a PermissionSet with the given value.
     *
     * @param permissions The permissions that should be included in this set.
     */
    public PermissionSet(Permission permissions[])
    {
	if( permissions == null )
	{
	    throw new IllegalArgumentException("Null permissions value");
	}
	this.permissions = permissions;
    }

    /**
     * Construct a PermissionSet from the given value.
     *
     * @param permissions A Vector containing Permission objects that should be
     * wrapped by this set.
     */
    public PermissionSet(Vector permissions)
    {
	Assert.require(permissions);

	synchronized( permissions )
	{
	    int lpc, size;
	    
	    size = permissions.size();
	    this.permissions = new Permission[size];
	    for( lpc = 0; lpc < this.permissions.length; lpc++ )
	    {
		Permission perm;

		perm = (Permission)permissions.elementAt(lpc);
		this.permissions[lpc] = (Permission)perm.clone();
	    }
	}
    }

    /**
     * Construct a PermissionSet from the given value.
     *
     * @param ps An array of PermissionSet's whose elements should make up this
     * set.
     */
    public PermissionSet(PermissionSet ps[])
    {
	this(toVector(null, ps));
    }

    /**
     * Construct an empty PermissionSet.
     */
    public PermissionSet()
    {
	this(new Permission[0]);
    }

    /**
     * @return A copy of the set of Permissions as an array.
     */
    public Permission[] toArray()
    {
	return (Permission[])this.permissions.clone();
    }

    /**
     * Grant the given Permission to this set.  The set must contain a
     * Permission object with the same name, otherwise this will do nothing.
     * For example, granting the permission "foo=read" to the set {
     * "bar=read,write" } will not result in the set having new permission.
     * However, if the set was { "foo=write", "bar=read,write" } it would
     * change to { "foo=read,write", "bar=read,write" }.
     *
     * @param what The Permission to grant.
     */
    public void grantPermission(Permission what)
    {
	int lpc;

	Assert.require(what);
	
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    Permission perm = this.permissions[lpc];

	    if( perm != null )
	    {
		this.permissions[lpc].grant(what);
	    }
	}
    }

    /**
     * Grant all of the Permissions in the given set to this set.
     *
     * @param ps The set of permissions to grant to this one.
     */
    public void grantPermissions(PermissionSet ps)
    {
	int lpc, lpc2;

	Assert.require(ps);
	
	for( lpc = 0; lpc < ps.permissions.length; lpc++ )
	{
	    Permission what = ps.permissions[lpc];
	    
	    for( lpc2 = 0; lpc2 < this.permissions.length; lpc2++ )
	    {
		Permission perm = this.permissions[lpc2];
		
		if( perm != null )
		{
		    this.permissions[lpc2].grant(what);
		}
	    }
	}
    }

    /**
     * Revoke the given Permission from this set.
     *
     * @param what The Permission to revoke from this set.
     */
    public void revokePermission(Permission what)
    {
	int lpc;

	Assert.require(what);
	
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    Permission perm = this.permissions[lpc];
	    
	    if( perm != null )
	    {
		this.permissions[lpc].revoke(what);
	    }
	}
    }
    
    /**
     * Revoke all of the Permissions in the given set from this set.
     *
     * @param ps The set of permissions to revoke from this one.
     */
    public void revokePermissions(PermissionSet ps)
    {
	int lpc, lpc2;

	Assert.require(ps);

	for( lpc = 0; lpc < ps.permissions.length; lpc++ )
	{
	    Permission what = ps.permissions[lpc];
	    
	    for( lpc2 = 0; lpc2 < this.permissions.length; lpc2++ )
	    {
		Permission perm = this.permissions[lpc2];
		
		if( perm != null )
		{
		    perm.revoke(what);
		}
	    }
	}
    }

    /**
     * Bound the Permissions in this set to the ones in the given set.  For
     * example, if the given set is { "foo=read,write" } and this set is {
     * "foo=read", "bar=read,write" }, the result will be { "foo=read" }.
     *
     * @param ps The set of permissions to bound this set to.
     */
    public void boundPermissions(PermissionSet ps)
    {
	boolean touched[];
	int lpc;

	Assert.require(ps);

	touched = new boolean[this.permissions.length];
	for( lpc = 0; lpc < ps.permissions.length; lpc++ )
	{
	    Permission what = ps.permissions[lpc];
	    int lpc2;

	    for( lpc2 = 0; lpc2 < this.permissions.length; lpc2++ )
	    {
		Permission perm = this.permissions[lpc2];
		
		if( perm != null )
		{
		    if( perm.nameImplies(what) )
		    {
			touched[lpc2] = true;
			perm.bound(what);
		    }
		}
	    }
	}
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( !touched[lpc] )
	    {
		this.permissions[lpc].revoke(this.permissions[lpc]);
	    }
	}
    }

    public PermissionSet subset(Permission what)
    {
	final Vector matches = new Vector();
	int lpc;
	
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( this.permissions[lpc] != null )
	    {
		if( what.implies(this.permissions[lpc]) )
		{
		    matches.addElement(this.permissions[lpc]);
		}
	    }
	}
	return new PermissionSet(matches);
    }

    /**
     * Check if this set has the given Permission.
     *
     * @param what The permission to check for.
     * @return The matching permission or null if not found.
     */
    public Permission hasPermission(Permission what)
    {
	Permission retval = null;
	int lpc;

	if( what == null )
	{
	    throw new NullPointerException(
		"Null Permission passed to PermissionSet.hasPermission()");
	}
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( (this.permissions[lpc] != null) &&
		this.permissions[lpc].nameImplies(what) )
	    {
		if( this.permissions[lpc].implies(what) )
		{
		    retval = this.permissions[lpc];
		    break;
		}
		else
		{
		    retval = null;
		}
	    }
	}
	return retval;
    }

    /**
     * Check if this set has the given Permission and throw a
     * PermissionException if it doesn't.
     *
     * @param what The permission to check for.
     */
    public void checkPermission(Permission what)
    {
	if( this.hasPermission(what) == null )
	{
	    throw new PermissionException("Invalid access:  Set, "
					  + this
					  + ", doesn't have permission, "
					  + what);
	}
    }

    /**
     * Update an MD5 hash value with the contents of this set.
     *
     * @param digest The MD5 object to update.
     */
    public void updateHash(MD5 digest)
    {
	int lpc;

	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( this.permissions[lpc] != null )
	    {
		this.permissions[lpc].updateHash(digest);
	    }
	}
    }
    
    /**
     * @return The number of bytes needed to marshal this set.
     */
    public int marshalledLength()
    {
	int lpc, retval;

	retval = Xdr.INT;
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( this.permissions[lpc] != null )
	    {
		retval += Xdr.INT + this.permissions[lpc].marshalledLength();
	    }
	}
	return retval;
    }

    /**
     * Marshal this object into the given byte array.
     *
     * @param xdr The destination for the flattened version of this set.
     */
    public void marshal(XdrByteArray xdr)
    {
	int lpc;
	
	xdr.PUT(this.permissions.length);
	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( this.permissions[lpc] != null )
	    {
		xdr.PUT(this.permissions[lpc].getTypeID());
		this.permissions[lpc].marshal(xdr);
	    }
	    else
	    {
		xdr.PUT(PermissionTypes.PERMISSION_NULL);
	    }
	}
    }

    /**
     * Instantiate a Permission object from the given byte array.
     *
     * @param xdr The source of the flattened version of a set.
     * @return A new PermissionSet object.
     */
    public static PermissionSet instantiate(XdrByteArray xdr)
    {
	Permission permissions[];
	int lpc;
	
	permissions = new Permission[xdr.INT()];
	for( lpc = 0; lpc < permissions.length; lpc++ )
	{
	    permissions[lpc] = Permission.instantiate(xdr);
	}
	return new PermissionSet(permissions);
    }

    /**
     * Instantiate a Permission object from the given byte array.
     *
     * @param flattened The source of the flattened version of a set.
     * @return A new PermissionSet object.
     */
    public static PermissionSet instantiate(byte flattened[])
    {
	PermissionSet retval;
	XdrByteArray xba;

	xba = new XdrByteArray(flattened);
	retval = instantiate(xba);
	if( xba.index != xba.len )
	{
	    throw new IllegalArgumentException(
		"Extraneous data in flattened PermissionSet");
	}
	return retval;
    }
    
    /**
     * @return A byte array containing the flattened version of this set.
     */
    public byte[] toBytes()
    {
	XdrByteArray xba;

	xba = new XdrByteArray(this.marshalledLength());
	this.marshal(xba);
	return xba.toBytes();
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof PermissionSet )
	{
	    PermissionSet ps = (PermissionSet)obj;

	    if( this.permissions.length == ps.permissions.length )
	    {
		int lpc;
		
		retval = true;
		for( lpc = 0;
		     retval && (lpc < this.permissions.length);
		     lpc++ )
		{
		    retval = this.permissions[lpc].equals(ps.permissions[lpc]);
		}
	    }
	}
	return retval;
    }

    /**
     * @see java.lang.Object#clone()
     */
    public Object clone()
    {
	Permission newPermissions[] = new Permission[this.permissions.length];
	int lpc;

	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    if( this.permissions[lpc] != null )
	    {
		newPermissions[lpc] = (Permission)
		    this.permissions[lpc].clone();
	    }
	}
	return new PermissionSet(newPermissions);
    }
    
    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	String retval = "PermissionSet[permissions={";
	int lpc;

	for( lpc = 0; lpc < this.permissions.length; lpc++ )
	{
	    retval += " " + this.permissions[lpc];
	}
	retval += " }]";
	return retval;
    }
}
