/*
 * FileTest.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.tests;

import java.io.InputStream;
import java.io.PrintStream;
import java.io.OutputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.FileNotFoundException;

import bees.core.Node;
import bees.core.ObjectTree;

import bees.core.resource.File;
import bees.core.resource.Clock;
import bees.core.resource.Logger;
import bees.core.resource.Console;
import bees.core.resource.Directory;
import bees.core.resource.LoggerSpec;
import bees.core.resource.LoggerFactory;
import bees.core.resource.FileSystemRoot;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.FilePermission;
import bees.core.security.PermissionException;

public final class FileTest
    implements Test
{
    public void test(Node node)
	throws Throwable
    {
	FileSystemRoot cwd;
	String listing[];
	Directory dir;
	Clock clock;
	File file;
	int lpc;
	
	cwd = (FileSystemRoot)node.getCapability("cwd");
	clock = (Clock)node.getCapability("clock");

	dir = cwd.root(Directory.DEFAULT_RESTRICTED_PERMISSIONS);

	try { dir.file("no_such_file"); }
	catch(FileNotFoundException e)
	{ Logger.log("Unable to open: no_such_file"); }
	try { dir.file("no_such_file", File.DEFAULT_PERMISSIONS, true); }
	catch(PermissionException e)
	{ Logger.log("Unable to create: no_such_file"); }
	try { dir.delete("no_such_file"); }
	catch(PermissionException e)
	{ Logger.log("Unable to delete: no_such_file"); }
	
	try { dir.file("Makefile"); }
	catch(PermissionException e)
	{ Logger.log("Unable to open: Makefile"); }

	listing = dir.list();
	for( lpc = 0; lpc < listing.length; lpc++ )
	{
	    if( listing[lpc].equals("Makefile") )
		Logger.log("Found Makefile");
	}

	dir = cwd.root();
	
	file = dir.file("Makefile", new PermissionSet());
	try { file.getOutputStream(); }
	catch(PermissionException e)
	{ Logger.log("Unable to write to: Makefile"); }
	try { file.getOutputStream(true); }
	catch(PermissionException e)
	{ Logger.log("Unable to append to: Makefile"); }
	try { file.getInputStream(); }
	catch(PermissionException e)
	{ Logger.log("Unable to read from: Makefile"); }
	try { file.length(); }
	catch(PermissionException e)
	{ Logger.log("Unable to get length of: Makefile"); }
	try { file.lastModified(clock); }
	catch(PermissionException e)
	{ Logger.log("Unable to get last modified time of: Makefile"); }
	try { file.delete(); }
	catch(PermissionException e)
	{ Logger.log("Unable to delete: Makefile"); }

	try { dir.directory("Makefile"); }
	catch(FileNotFoundException e)
	{
	    Logger.log("Unable to make directory capability from file: "
		       + "Makefile");
	}

	{
	    final long TIME_TOLERANCE = 2 * 1000;
	    BufferedReader br;
	    PrintStream ps;
	    long ct, lmt;
	    
	    file = dir.file("new-file.extra", File.DEFAULT_PERMISSIONS, true);
	    if( file != dir.lookup("new-file.extra") )
	    {
		Logger.log("Lookup differs from create?");
	    }
	    listing = dir.list();
	    for( lpc = 0; lpc < listing.length; lpc++ )
	    {
		if( listing[lpc].equals("new-file.extra") )
		    Logger.log("Found new-file.extra");
	    }
	    ps = new PrintStream(file.getOutputStream());
	    ps.println("Hello, World!");
	    ps.close();
	    ct = clock.current();
	    lmt = file.lastModified(clock);
	    if( !(((ct - TIME_TOLERANCE) < lmt) &&
		  (lmt < (ct + TIME_TOLERANCE))) )
	    {
		Logger.log("Bad last modified time?");
	    }
	    
	    br = new BufferedReader(
		new InputStreamReader(file.getInputStream()));
	    Logger.log(br.readLine());
	    br.close();
	}
    }
}
