/*
 * PermissionTest.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.tests;

import edu.utah.janos.nodeos.AddressFormatException;

import bees.core.Node;

import bees.core.resource.Logger;
import bees.core.resource.IPAddressPermission;

import bees.core.security.Permission;
import bees.core.security.PermissionSet;
import bees.core.security.FloatPermission;
import bees.core.security.RangePermission;
import bees.core.security.IntegerPermission;
import bees.core.security.PermissionException;
import bees.core.security.MinimumFloatPermission;
import bees.core.security.MaximumFloatPermission;
import bees.core.security.MinimumIntegerPermission;
import bees.core.security.MaximumIntegerPermission;

public final class PermissionTest
    implements Test
{
    public void integer(Node node)
    {
	PermissionSet ps;

	Logger.log("Basic minimum integer test");
	ps = new PermissionSet(new Permission[] {
	    new MinimumIntegerPermission("foo", 128),
	});
	ps.checkPermission(new IntegerPermission("foo", 128));
	ps.checkPermission(new IntegerPermission("foo", 129));
	ps.checkPermission(new IntegerPermission("foo", 256));
	try { ps.checkPermission(new IntegerPermission("foo", 0)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 0"); }
	try { ps.checkPermission(new IntegerPermission("foo", 127)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 127"); }

	Logger.log("Revoke minimum integer");
	ps.revokePermission(new IntegerPermission("foo", "", 256));
	ps.checkPermission(new IntegerPermission("foo", 256));
	try { ps.checkPermission(new IntegerPermission("foo", 128)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 128"); }
	try { ps.checkPermission(new IntegerPermission("foo", 200)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 200"); }
	
	Logger.log("Grant minimum integer");
	ps.grantPermission(new IntegerPermission("foo", "", 128));
	ps.checkPermission(new IntegerPermission("foo", 128));
	ps.checkPermission(new IntegerPermission("foo", 129));
	ps.checkPermission(new IntegerPermission("foo", 256));
	try { ps.checkPermission(new IntegerPermission("foo", 0)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 0"); }
	try { ps.checkPermission(new IntegerPermission("foo", 127)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 127"); }
	
	Logger.log("Basic maximum integer test");
	ps = new PermissionSet(new Permission[] {
	    new MaximumIntegerPermission("foo", 128),
	});
	ps.checkPermission(new IntegerPermission("foo", 128));
	ps.checkPermission(new IntegerPermission("foo", 0));
	ps.checkPermission(new IntegerPermission("foo", 1));
	try { ps.checkPermission(new IntegerPermission("foo", 129)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 129"); }
	try { ps.checkPermission(new IntegerPermission("foo", 500)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 500"); }
	
	Logger.log("Revoke maximum integer");
	ps.revokePermission(new IntegerPermission("foo", "", 64));
	ps.checkPermission(new IntegerPermission("foo", 64));
	try { ps.checkPermission(new IntegerPermission("foo", 65)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 128"); }
	try { ps.checkPermission(new IntegerPermission("foo", 128)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 200"); }
	
	Logger.log("Grant maximum integer");
	ps.grantPermission(new IntegerPermission("foo", "", 128));
	ps.checkPermission(new IntegerPermission("foo", 128));
	ps.checkPermission(new IntegerPermission("foo", 0));
	ps.checkPermission(new IntegerPermission("foo", 1));
	try { ps.checkPermission(new IntegerPermission("foo", 129)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 129"); }
	try { ps.checkPermission(new IntegerPermission("foo", 500)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 500"); }
	
	Logger.log("Simple range test");
	ps = new PermissionSet(new Permission[] {
	    new RangePermission("foo", "read,write,execute",
				128, 129),
	});
	ps.checkPermission(new IntegerPermission("foo", 128));
	ps.checkPermission(new IntegerPermission("foo", 129));
	ps.checkPermission(new IntegerPermission("foo", 255));
	ps.checkPermission(new IntegerPermission("foo", 256));
	try { ps.checkPermission(new IntegerPermission("foo", 0)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 0"); }
	try { ps.checkPermission(new IntegerPermission("foo", 127)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 127"); }
	try { ps.checkPermission(new IntegerPermission("foo", 257)); }
	catch(PermissionException e) { Logger.log("Caught bad integer: 257"); }
    }

    public void range(Node node)
    {
	RangePermission rp;
	PermissionSet ps;
	
	Logger.log("multiple external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 512, 10));
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 0, 65536));
	Logger.log("rp = " + rp);
	ps = new PermissionSet(new Permission[] {
	    rp,
	});
	ps.checkPermission(new IntegerPermission("port", "read", 1000));
	try { ps.checkPermission(new IntegerPermission("port", 65536)); }
	catch(PermissionException e) { Logger.log("Caught bad range: 65536"); }
	    
	Logger.log("same start external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 1024, 2048));
	Logger.log("rp = " + rp);
	    
	Logger.log("same start internal");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 1024, 128));
	Logger.log("rp = " + rp);
	    
	Logger.log("same end external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 0, 2048));
	Logger.log("rp = " + rp);
	    
	Logger.log("same end internal");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 1024 + 128,
				     1024 - 128));
	Logger.log("rp = " + rp);
	    
	Logger.log("internal");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 1024 + 128, 128));
	Logger.log("rp = " + rp);
	    
	Logger.log("begin edge overlap");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 0, 1024));
	Logger.log("rp = " + rp);
	    
	Logger.log("end edge overlap");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 2048, 1024));
	Logger.log("rp = " + rp);
	    
	Logger.log("same");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 1024, 1024));
	Logger.log("rp = " + rp);

	Logger.log(" ** revoke **");
	    
	Logger.log("multiple external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.grant(new RangePermission("port", "read", 512, 10));
	Logger.log("rp = " + rp);
	rp.revoke(new RangePermission("port", "", 0, 65536));
	Logger.log("rp = " + rp);
	    
	Logger.log("same start external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.revoke(new RangePermission("port", "", 1024, 2048));
	Logger.log("rp = " + rp);
	    
	Logger.log("begin edge");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.revoke(new RangePermission("port", "", 0, 1024));
	Logger.log("rp = " + rp);
	    
	Logger.log("same");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.revoke(new RangePermission("port", "", 1024, 1024));
	Logger.log("rp = " + rp);
	    
	Logger.log("same end external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.revoke(new RangePermission("port", "", 0, 2048));
	Logger.log("rp = " + rp);
	    
	Logger.log("same end external");
	rp = new RangePermission("port", "read", 1024, 1024);
	Logger.log("rp = " + rp);
	rp.bound(new RangePermission("port", "", 1024 + 128, 1024 - 256));
	Logger.log("rp = " + rp);
    }
    
    public void ipAddress(Node node)
	throws AddressFormatException
    {
	PermissionSet ps;
	
	try
	{
	    new IPAddressPermission("dst", "foo");
	    Logger.log("Able to make IPAddress with bogus address?");
	}
	catch(AddressFormatException e)
	{
	    Logger.log("Caught bad address: foo");
	}
	try
	{
	    new IPAddressPermission("dst", "192.168.0.0/foo");
	    Logger.log("Able to make IPAddress with bogus mask?");
	}
	catch(AddressFormatException e)
	{
	    Logger.log("Caught bad address: 192.168.0.0/foo");
	}
	
	ps = new PermissionSet(new Permission[] {
	    new IPAddressPermission("dst", "*"),
	});
	ps.checkPermission(new IPAddressPermission("dst", "192.168.0.1"));
	
	ps = new PermissionSet(new Permission[] {
	    new IPAddressPermission("dst", "192.168.0.0/16"),
	});
	ps.checkPermission(new IPAddressPermission("dst", "192.168.0.1"));
	ps.checkPermission(new IPAddressPermission("dst", "192.168.1.1"));
	ps.checkPermission(new IPAddressPermission("dst", "192.168.254.1"));
	try
	{
	    ps.checkPermission(new IPAddressPermission("dst", "192.169.0.1"));
	}
	catch(PermissionException e)
	{
	    Logger.log("Caught bad access: 192.169.0.1");
	}
    }
    
    public void floating(Node node)
    {
	PermissionSet ps;

	Logger.log("Basic minimum float test");
	ps = new PermissionSet(new Permission[] {
	    new MinimumFloatPermission("foo", 128),
	});
	ps.checkPermission(new FloatPermission("foo", 128));
	ps.checkPermission(new FloatPermission("foo", 129));
	ps.checkPermission(new FloatPermission("foo", 256));
	try { ps.checkPermission(new FloatPermission("foo", 0)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 0"); }
	try { ps.checkPermission(new FloatPermission("foo", 127)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 127"); }

	Logger.log("Revoke minimum float");
	ps.revokePermission(new FloatPermission("foo", "", 256));
	ps.checkPermission(new FloatPermission("foo", 256));
	try { ps.checkPermission(new FloatPermission("foo", 128)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 128"); }
	try { ps.checkPermission(new FloatPermission("foo", 200)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 200"); }
	
	Logger.log("Grant minimum float");
	ps.grantPermission(new FloatPermission("foo", "", 128));
	ps.checkPermission(new FloatPermission("foo", 128));
	ps.checkPermission(new FloatPermission("foo", 129));
	ps.checkPermission(new FloatPermission("foo", 256));
	try { ps.checkPermission(new FloatPermission("foo", 0)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 0"); }
	try { ps.checkPermission(new FloatPermission("foo", 127)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 127"); }
	
	Logger.log("Basic maximum float test");
	ps = new PermissionSet(new Permission[] {
	    new MaximumFloatPermission("foo", 128),
	});
	ps.checkPermission(new FloatPermission("foo", 128));
	ps.checkPermission(new FloatPermission("foo", 0));
	ps.checkPermission(new FloatPermission("foo", 1));
	try { ps.checkPermission(new FloatPermission("foo", 129)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 129"); }
	try { ps.checkPermission(new FloatPermission("foo", 500)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 500"); }
	
	Logger.log("Revoke maximum float");
	ps.revokePermission(new FloatPermission("foo", "", 64));
	ps.checkPermission(new FloatPermission("foo", 64));
	try { ps.checkPermission(new FloatPermission("foo", 65)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 128"); }
	try { ps.checkPermission(new FloatPermission("foo", 128)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 200"); }
	
	Logger.log("Grant maximum float");
	ps.grantPermission(new FloatPermission("foo", "", 128));
	ps.checkPermission(new FloatPermission("foo", 128));
	ps.checkPermission(new FloatPermission("foo", 0));
	ps.checkPermission(new FloatPermission("foo", 1));
	try { ps.checkPermission(new FloatPermission("foo", 129)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 129"); }
	try { ps.checkPermission(new FloatPermission("foo", 500)); }
	catch(PermissionException e) { Logger.log("Caught bad float: 500"); }
    }

    public void test(Node node)
	throws Throwable
    {
	this.ipAddress(node);
	this.integer(node);
	this.floating(node);
	this.range(node);
    }
}
