/*
 * LineList.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package bees.util;

import java.util.Vector;

import bees.core.Assert;

/**
 * A helper class for the LineListTokenizer, basically, it just holds the list
 * of tokens from a line.
 */
public final class LineList
{
    /**
     * The components of the line.
     */
    private String components[];

    /**
     * The line number this object corresponds too.
     */
    private int lineNumber;

    /**
     * Construct an empty line list object.
     */
    public LineList()
    {
    }

    /**
     * Copy the token string references out of the given vector.
     *
     * @param vec The vector of tokens from a line.
     */
    void copyComponents(Vector vec)
    {
	Assert.require(vec);
	
	int lpc, size;
	
	size = vec.size();
	this.components = new String[size];
	for( lpc = 0; lpc < size; lpc++ )
	{
	    this.components[lpc] = (String)vec.elementAt(lpc);

	    Assert.ensure(this.components[lpc]);
	}
    }

    /**
     * @param components The components of the line.
     */
    void setComponents(String components[])
    {
	this.components = components;
    }

    /**
     * @return The components of the line.
     */
    public String[] getComponents()
    {
	return this.components;
    }

    /**
     * @param ln The line number.
     */
    void setLineNumber(int ln)
    {
	Assert.require(ln >= 0);
	
	this.lineNumber = ln;
    }

    /**
     * @return The line number.
     */
    public int getLineNumber()
    {
	return this.lineNumber;
    }

    /**
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
	return this.lineNumber;
    }

    /**
     * @see java.lang.Object#equals(Object)
     */
    public boolean equals(Object obj)
    {
	boolean retval = false;

	if( obj instanceof LineList )
	{
	    LineList ll = (LineList)obj;

	    if( (this.components.length == ll.components.length) &&
		(this.lineNumber == ll.lineNumber) )
	    {
		int lpc;

		retval = true;
		for( lpc = 0; retval && (lpc < this.components.length); lpc++ )
		{
		    retval = this.components[lpc].equals(ll.components[lpc]);
		}
	    }
	}
	return retval;
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	String retval = "LineList[components={length="
	    + this.components.length
	    + "; ";
	int lpc;

	for( lpc = 0; lpc < this.components.length; lpc++ )
	{
	    retval += " " + this.components[lpc];
	}
	retval += " }; lineNumber="
	    + this.lineNumber
	    + "]";
	return retval;
    }
}
