/* 
 * LineListTokenizer.java
 *
 * Copyright (c) 1999, 2000, 2001, 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Bees distribution.
 * 
 * Permission to use, copy, modify, distribute, and sell this
 * software and its documentation for any purpose is hereby granted
 * without fee, provided that (i) the above copyright notice(s)
 * appear in all copies and (ii) that both the copyright notice(s)
 * and this permission notice appear in supporting documentation,
 * and (iii) that the name(s) of the copyright holders not be used
 * in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.  The
 * copyright holders make no representations about the suitability
 * of this software for any purpose.  It is provided "as is"
 * without express or implied warranty.
 * 
 * The maintainers of this code ask, but do not require, that
 * improvements and bug fixes be emailed to csl-dist@cs.utah.edu
 *         
 *  
 *  Copyright 1996 Massachusetts Institute of Technology
 * 
 *  Permission to use, copy, modify, distribute, and sell this software and
 *  its documentation for any purpose is hereby granted without fee, provided
 *  that the above copyright notice appear in all copies and that both that
 *  copyright notice and this permission notice appear in supporting
 *  documentation, and that the name of M.I.T. not be used in advertising or
 *  publicity pertaining to distribution of the software without specific,
 *  written prior permission.  M.I.T. makes no representations about the
 *  suitability of this software for any purpose.  It is provided "as is"
 *  without express or implied warranty.
 */
package bees.util;

import java.util.Vector;

import java.io.StreamTokenizer;
import java.io.FileNotFoundException;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;

import bees.core.Assert;

/**
 * File reading utility. Returns each useful line of text as an array
 * of strings, one string per "word."  Comments are ignored and quoted
 * strings are handled (the quotes are stripped off and the quoted
 * string is returned as a single word).
 *
 * @author David Wetherall
 * @author <a href="mailto:janos-discuss@flux.cs.utah.edu">Utah Janos Team</a>
 */
public class LineListTokenizer
    extends StreamTokenizer 
{
    /**
     * Holds the tokenized components of the line.
     */
    private final Vector components = new Vector();

    /**
     * @return The next token type in the line.
     * @throws LineListException if there's an IO error.
     */
    private int ll_nextToken()
	throws LineListException
    {
	try
	{
	    return this.nextToken();
	}
	catch (java.io.IOException ioe)
	{
	    throw new LineListException("IOException: " + ioe.getMessage());
	}
    }

    /**
     * Tokenize the next line in the input.
     *
     * @param ll The LineList object to fill out.
     * @return True if there are more lines to process.
     * @throws LineListException if there's an IO error.
     */
    public boolean nextLine(LineList ll)
	throws LineListException
    {
	boolean done = false, retval = true;
	int tokenType;

	Assert.require(ll);
	
	ll.setLineNumber(this.lineno());
	this.components.removeAllElements();
	while( !done )
	{
	    tokenType = this.ll_nextToken();
	    switch( tokenType )
	    {
	    case TT_EOF:
		if( this.components.size() == 0 )
		{
		    retval = false;
		}
		/** FALLTHROUGH **/
	    case TT_EOL:
		done = true;
		break;
	    case TT_NUMBER:
		this.components.addElement(Double.toString(nval));
		break;
	    default:
		this.components.addElement(sval);
		break;
	    }
	}
	if( retval )
	{
	    ll.copyComponents(this.components);
	}
	else
	{
	    ll.setComponents(null);
	}
	return retval;
    }
    
    /**
     * Create a new LineListTokenizer for the given file.
     *
     * @param file the file to use as input.
     * @exception FileNotFoundException if the given file cannot be opened for
     * @exception reading.
     */
    public LineListTokenizer(String file)
	throws FileNotFoundException
    {
	this(new InputStreamReader(new FileInputStream(file)));
    }
    
    /**
     * Create a new LineListTokenizer from the given readable
     * Stream.
     *
     * @param input the Reader to stream input from.
     */
    public LineListTokenizer(Reader input)
    {
	super(input);
	
	resetSyntax();
	whitespaceChars(0, ' ');
	wordChars('0', '9');
	wordChars('-', '-');
	wordChars('.', '.');
	wordChars(',', ',');
	wordChars(':', ':');
	wordChars('=', '=');
	wordChars('/', '/');
	wordChars('A', 'Z');
	wordChars('a', 'z');
	wordChars('_', '_');
	wordChars('*','*');
	wordChars('@','@');
	wordChars('+','+');
	wordChars('$','$');
	commentChar('#');
	quoteChar('\'');
	quoteChar('\"');    
	eolIsSignificant(true);
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
	return "LineListTokenizer["
	    + super.toString()
	    + "; components="
	    + this.components
	    + "]";
    }
}
