/*
 * InfectCapsule.java
 *
 * Copyright (c) 2002 The University of Utah and the Flux Group.
 * All rights reserved.
 *
 * This file is part of the Janos Bees Software.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation is hereby granted without fee, provided that the above
 * copyright notice and this permission/disclaimer notice is retained in all
 * copies or modified versions, and that both notices appear in supporting
 * documentation.  THE COPYRIGHT HOLDERS PROVIDE THIS SOFTWARE "AS IS" AND
 * WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE.  THE COPYRIGHT HOLDERS DISCLAIM ANY LIABILITY OF ANY KIND FOR
 * ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 * 
 * Users are requested, but not required, to send to csl-dist@cs.utah.edu any
 * improvements that they make and grant Univ. of Utah redistribution rights.
 */

package apps.virus;

import bees.core.Xdr;
import bees.core.Node;
import bees.core.Panic;
import bees.core.Capsule;
import bees.core.NodeCache;
import bees.core.NeighborImpl;
import bees.core.ProtocolSession;
import bees.core.EncodingException;
import bees.core.CapabilityEnvelope;
import bees.core.MulticastNeighborhood;
import bees.core.MissingCapabilityException;

import bees.core.resource.Logger;

import bees.core.security.IntegerPermission;

/**
 * An InfectCapsule is used to infect a neighbor with the VirusProtocol.
 */
public final class InfectCapsule
    extends Capsule
    implements NodeCache.EventHandler
{
    /**
     * The default interval between InfectCapsule transmissions.
     */
    static final IntegerPermission DEFAULT_INTERVAL =
	new IntegerPermission("interval", 1000);

    /**
     * A neighborhood containing all known neighbors.
     */
    private final MulticastNeighborhood neighborhood;

    /**
     * Construct an InfectCapsule for periodic transmission.
     *
     * @param neighborhood The neighborhood that this capsule should be send
     * to periodically.
     */
    public InfectCapsule(MulticastNeighborhood neighborhood)
    {
	this.neighborhood = neighborhood;
    }

    /**
     * Construct an empty InfectCapsule.
     */
    public InfectCapsule()
    {
	this(null);
    }

    public void removed(Node node, ProtocolSession ps)
    {
    }

    public IntegerPermission collected(Node node, ProtocolSession ps)
    {
	CapabilityEnvelope ce[];
	
	Logger.log(Logger.NOTICE, "Requesting neighbors.");
	if( (ce = node.requestCapability("neighbors",
					 "user/neighbors",
					 null)) != null )
	{
	    VirusSession vs;
	    int lpc;

	    vs = (VirusSession)ps;
	    for( lpc = 0; lpc < ce.length; lpc++ )
	    {
		try
		{
		    NeighborImpl ni;

		    ni = (NeighborImpl)ce[lpc].unseal();
		    vs.addNeighbor(node, null, ni);
		    this.neighborhood.addNeighbor(ni);
		}
		catch(InstantiationException e)
		{
		    Logger.log(e);
		}
		catch(MissingCapabilityException e)
		{
		    Logger.log(e);
		}
	    }
	}
	try
	{
	    Logger.log(Logger.NOTICE, "Sending InfectCapsule.");
	    this.neighborhood.send(this);
	}
	catch(EncodingException e)
	{
	    throw new Panic("Unable to encode InfectCapsule?", e);
	}
	catch(Throwable th)
	{
	    Logger.log(th);
	}
	return DEFAULT_INTERVAL;
    }

    public void decode(Xdr xdr)
    {
    }

    public void encode(Xdr xdr)
    {
    }

    public void evaluate(Node node)
    {
	Logger.log(Logger.NOTICE, "Infected!");
    }
    
    public String toString()
    {
	return "InfectCapsule["
	    + super.toString()
	    + "; neighborhood="
	    + this.neighborhood
	    + "]";
    }
}
